C
C      **********************************************************************
C      ********************* FUNCTION AGAUSS  *******************************
C      **********************************************************************
C
       FUNCTION AGAUSS (Z)
C
C      *        EVALUATES THE INTEGRAL OF THE GAUSSIAN PROBABILITY FUNCTION *
C      *        OBTAINED FROM PROGRAM 3-5 ON P. 35 OF "DATA REDUCTION AND   *
C      *        ERROR ANALYSIS FOR THE PHYSICAL SCIENCES", P. R. BEVINGTON, *
C      *        1969, McGRAW HILL, (NY:NY).                                 *
C      *                                                                    *
C
C
       IMPLICIT REAL*8 (A-H,O-Z), INTEGER (I-N)
C     
C
       Z = DABS(Z)
       AGAUSS = 1.0
C
C      *         IF Z>5.0, USE APPROXIMATION FOR PROB TO AVOID ERROR        *
C
       IF (Z .LE. 5.0) THEN
          DENOM = 1.0
          IF (Z .GT. 0.0) THEN
             TERM = 0.7071067812D00 * Z
             SUM = TERM
             Y2 = (Z ** 2) / 2.0
   31        DENOM = DENOM + 2.0
             TERM = TERM * (Y2 * 2.0 / DENOM)
             SUM = SUM + TERM
             IF (TERM / SUM - 1.0E-10 .GT. 0.0) THEN
                GOTO 31
             ELSE
                AGAUSS = 1.128379167D00 * SUM * DEXP(- Y2)
             ENDIF
          ELSE
             AGAUSS = 0.0
          ENDIF
       ENDIF

       RETURN
       END
