procedure x1d (input)

# Note:  There are two sections in this script that need to be changed
# in order to distinguish between the normal and develop versions of x1d.
# These sections are identified by comments using ### rather than just
# a single #.

string	input    {"", prompt="Name of input FITS files"}
string	output   {"", prompt="Name of output FITS files to contain 1-D spectra"}
string	backcorr {"perform", prompt="Subtract background (sky and interorder)?",
                              min="perform|omit"}
string  ctecorr   {"perform", prompt="Apply CTE correction?",
                              min="perform|omit"}
string	dispcorr  {"perform", prompt="Apply 2-D dispersion solutions?",
                              min="perform|omit"}
string	helcorr   {"perform", prompt="Convert to heliocentric wavelengths?",
                              min="perform|omit"}
string	fluxcorr  {"perform", prompt="Convert to absolute flux units?",
                              min="perform|omit"}
int	sporder	  {INDEF, prompt="spectral order to extract"}
real	a2center  {INDEF, prompt="extract one spectrum at this Y location"}
int	maxsrch	  {INDEF, prompt="cross correlation range"}
bool	globalx	  {no,    prompt="use global crosscor offset in all orders ?"}
real	extrsize  {INDEF, prompt="size of extraction box"}
real	bk1size	  {INDEF, prompt="size of 1st background region"}
real	bk2size	  {INDEF, prompt="size of 2nd background region"}
real	bk1offst  {INDEF, prompt="offset of 1st background region"}
real	bk2offst  {INDEF, prompt="offset of 2nd background region"}
real	bktilt	  {INDEF, prompt="background tilt"}
int	backord   {INDEF, prompt="background order",min=0,max=1}
string	bksmode   {"median", prompt="Background smoothing mode",
                             min="off|median|average"}
int	bksorder  {3, prompt="Background smoothing polynomial order",
                      min=0}
real	blazeshift {INDEF, prompt="blaze shift (in pixels)"}
string	algorithm	{"unweighted", prompt="extraction algorithm",
                          min="unweighted|sc2d"}
real	xoffset	{INDEF, prompt="offset in X for slitless extraction"}
### For the develop (xstis) version of x1d, uncomment these three parameters,
### and use:
###                       min="unweighted|optimal|scatter_corr|sc2d"}
### as the list of possible values in the above definition of 'algorithm'.
###
### For the normal (stsdas) version of x1d, comment out the next three
### parameters, and use:
###                       min="unweighted|sc2d"}
### as the list of possible values in the above definition of 'algorithm'.
# real	ccthresh  {INDEF, prompt="cross correlation threshold"}
# bool	extrinf	  {yes, prompt="write extraction info ?"}
# pset	algpars	 {prompt="pset with extraction algorithm parameters"}

bool	verbose  {yes, prompt="verbose time stamps + more info"}
string  Version  {"2.40 (23-May-2012)", prompt="calstis version"}

begin
	string	 inp, out
	string	 x1dstr, exec

	### This parameter distinguishes between the normal and develop
	### versions of x1d.  Assign develop_version = yes for the
	### develop version, or set it to no for the normal version.
	bool develop_version = no

	# Get input parameters from par .

	inp = input  ;  out = output

        # define directory to executable

	if (develop_version) {
	    exec = osfn ("xstis$bin/")
	} else {
	    exec = osfn ("stsdasbin$")
	}

        # build command string

        x1dstr = "!" // exec // "cs6.e"

	x1dstr = x1dstr // " '" // inp // "' '" // out // "'"

        if (verbose)
            x1dstr = x1dstr // " -t -v"

	if (globalx)
	    x1dstr = x1dstr // " -g "

	if (backcorr == "perform")
	    x1dstr = x1dstr // " -back"

        if (ctecorr == "perform")
            x1dstr = x1dstr // " -cte"

	if (dispcorr == "perform")
	    x1dstr = x1dstr // " -disp"

        if (helcorr == "perform")
            x1dstr = x1dstr // " -hel"

        if (fluxcorr == "perform")
            x1dstr = x1dstr // " -flux"

	if ( backcorr == "omit" && dispcorr == "omit" &&
		helcorr == "omit" && fluxcorr == "omit" &&
                ctecorr == "omit")
	     x1dstr = x1dstr // " -x1d"

	# do the switches
	if (sporder != INDEF)
	    x1dstr = x1dstr // " -s " // sporder

	if (a2center != INDEF)
	    x1dstr = x1dstr // " -c " // a2center

	if (maxsrch != INDEF)
	    x1dstr = x1dstr // " -r " // maxsrch

	if (extrsize != INDEF)
	    x1dstr = x1dstr // " -x " // extrsize

	if (bk1size != INDEF)
	    x1dstr = x1dstr // " -b1 " // bk1size

	if (bk2size != INDEF)
	    x1dstr = x1dstr // " -b2 " // bk2size

	if (bk1offst != INDEF)
	    x1dstr = x1dstr // " -o1 " // bk1offst

	if (bk2offst != INDEF)
	    x1dstr = x1dstr // " -o2 " // bk2offst

	if (bktilt != INDEF)
	    x1dstr = x1dstr // " -k " // bktilt

	if (backord != INDEF)
	    x1dstr = x1dstr // " -n " // backord

	if (blazeshift != INDEF)
	    x1dstr = x1dstr // " -bs " // blazeshift

	if (bksmode == "median")
	     x1dstr = x1dstr // " -bm -bo " // bksorder
	if (bksmode == "average")
	     x1dstr = x1dstr // " -bb -bo " // bksorder
	if (bksmode == "off")
	     x1dstr = x1dstr // " -bn "

        if (algorithm == "unweighted")
	    x1dstr = x1dstr // " -a unweighted "
        else if (algorithm == "sc2d")
            x1dstr = x1dstr // " -a unweighted -idt "

	if (xoffset != INDEF)
	    x1dstr = x1dstr // " -st " // xoffset

	if (develop_version) {

	    if (!extrinf)
		x1dstr = x1dstr // " -e "

	    if (ccthresh != INDEF)
		x1dstr = x1dstr // " -p " // ccthresh

	    if (algorithm == "optimal") {

		if (algpars.fprof != "")
		    x1dstr = x1dstr // " -pf " // algpars.fprof

		if (algpars.fflux != "")
		    x1dstr = x1dstr // " -px " // algpars.fflux
	    }

	    if (algpars.sclip != INDEF)
		x1dstr = x1dstr // " -sc " // algpars.sclip

	    if (algpars.fnet)
		x1dstr = x1dstr // " -pa "

	    if (algpars.outw != "")
		x1dstr = x1dstr // " -wf " // algpars.outw

	    if (algorithm == "optimal")
		x1dstr = x1dstr // " -a optimal "
	    else if (algorithm == "scatter_corr")
		x1dstr = x1dstr // " -a scatter_corr -l " // algpars.lfilter

	    if (algpars.sc2dimage != "")
		x1dstr = x1dstr // " -if " // algpars.sc2dimage

	    if (algpars.bval != INDEF)
		x1dstr = x1dstr // " -bk " // algpars.bval

	    if (algpars.berr != INDEF)
		x1dstr = x1dstr // " -be " // algpars.berr

	    if (algpars.varim)
		x1dstr = x1dstr // " -va "
	}

        # execute
	printf ("%s\n", x1dstr)        # to display the string, not run it
	print (x1dstr) | cl

	print ("")

end
