#  get_coeff2 -- Read the Legendre polynomials for the specified chip of WFPC2
#
#  Description:
#  ------------
#  
#  Date		Author			Description
#  ----		------			-----------
#  05-Jul-1994  J.-C. Hsu		modified from get_coeff.x
#  27-Dec-1994  J.-C. Hsu		new coefficients from Gilmozzi (Nov 22)
#------------------------------------------------------------------------------
procedure get_coeff2 (ax, ay, iccd, order)

real	ax[*]
real	ay[*]

int	iccd
int	order

char	text[SZ_LINE]
#==============================================================================
begin
	if (iccd >= 1 && iccd <= 4) 
	    order = 3
	else {
	    call sprintf (text, SZ_LINE, "unreasonable detector number %d")
		call pargi (iccd)
	    call error (1, text)
	}

        # From Gilmozzi's mail dated Nov 22, 1994
        if (iccd == 1) {
	    ax[ 1] =  0.66815E-01   
	    ax[ 2] =  2.1633        
	    ax[ 3] =  0.38095E-01   
	    ax[ 4] =  0.94095      
	    ax[ 5] =  0.51627E-01 
	    ax[ 6] =  0.61692E-01
	    ax[ 7] = -0.46825E-02      
	    ax[ 8] = -0.79901E-02     
	    ax[ 9] =  0.10675        
	    ax[10] =  1.4861        
	    ax[11] =  0.22422E-01  
	    ax[12] =  0.56950E-01 
	    ax[13] = -0.87171E-02
	    ax[14] =  0.34350E-01    
	    ax[15] =  0.15134E-01   
	    ax[16] = -0.31546E-01  
	    ay[ 1] =  0.57416E-01
	    ay[ 2] =  0.17629E-01
	    ay[ 3] =  0.13038    
	    ay[ 4] =  0.16927E-01
	    ay[ 5] =  2.3635    
	    ay[ 6] =  0.80603E-01
	    ay[ 7] =  1.5121    
	    ay[ 8] =  0.22461E-01
	    ay[ 9] = -0.13915E-01
	    ay[10] =  0.25392E-01
	    ay[11] = -0.16964E-01
	    ay[12] =  0.75925E-01
	    ay[13] =  0.96595    
	    ay[14] = -0.44384E-01
	    ay[15] =  0.13129    
	    ay[16] = -0.53270E-01
        }

        if (iccd == 2) {
	    ax[ 1] =  0.83464E-01    
	    ax[ 2] =  1.9334        
	    ax[ 3] =  0.17163      
	    ax[ 4] =  0.84619     
	    ax[ 5] =  0.49975E-01
	    ax[ 6] =  0.15608         
	    ax[ 7] =  0.26701E-02    
	    ax[ 8] = -0.97651E-02  
	    ax[ 9] = -0.12299E-01 
	    ax[10] =  1.4657      
	    ax[11] = -0.15196E-01     
	    ax[12] =  0.42339E-01     
	    ax[13] = -0.33684E-02   
	    ax[14] = -0.11308E-01  
	    ax[15] =  0.32543E-01  
	    ax[16] = -0.81674E-01
	    ay[ 1] = -0.96107E-02
	    ay[ 2] = -0.43238E-01
	    ay[ 3] = -0.57259E-01
	    ay[ 4] =  0.23489E-02
	    ay[ 5] =  1.9128    
	    ay[ 6] =  0.33555    
	    ay[ 7] =  1.4585    
	    ay[ 8] = -0.27291E-01
	    ay[ 9] =  0.41407E-01
	    ay[10] =  0.23959E-01
	    ay[11] =  0.67393E-02
	    ay[12] =  0.20720E-01
	    ay[13] =  0.83414    
	    ay[14] = -0.61361E-01
	    ay[15] =  0.32687E-01
	    ay[16] = -0.33779E-01
        }

        if (iccd == 3) {
	    ax[ 1] = -0.63864E-01    
	    ax[ 2] =  1.9285        
	    ax[ 3] = -0.40329E-01  
	    ax[ 4] =  0.88085     
	    ax[ 5] = -0.67214E-01
	    ax[ 6] =  0.15302         
	    ax[ 7] =  0.58819E-02   
	    ax[ 8] = -0.10204E-01  
	    ax[ 9] = -0.86120E-01 
	    ax[10] =  1.4632     
	    ax[11] = -0.33002E-02     
	    ax[12] =  0.75607E-01    
	    ax[13] =  0.86360E-03   
	    ax[14] = -0.49353E-01  
	    ax[15] = -0.14471E-02 
	    ax[16] = -0.33843E-01
	    ay[ 1] = -0.37607E-01
	    ay[ 2] = -0.44137E-01
	    ay[ 3] = -0.76807E-01
	    ay[ 4] =  0.42059E-02
	    ay[ 5] =  1.9584    
	    ay[ 6] =  0.16362    
	    ay[ 7] =  1.4809    
	    ay[ 8] = -0.27224E-01
	    ay[ 9] =  0.19784E-01
	    ay[10] =  0.25133E-01
	    ay[11] =  0.24144E-02
	    ay[12] =  0.33534E-01
	    ay[13] =  0.86833    
	    ay[14] = -0.23580E-01
	    ay[15] =  0.58449E-01
	    ay[16] = -0.32079E-01
        }

        if (iccd == 4) {
	    ax[ 1] =  0.53754E-02    
	    ax[ 2] =  1.9520        
	    ax[ 3] =  0.75419E-01  
	    ax[ 4] =  0.87782     
	    ax[ 5] = -0.53325E-01
	    ax[ 6] =  0.17116         
	    ax[ 7] =  0.29530E-01    
	    ax[ 8] =  0.18075E-01   
	    ax[ 9] = -0.57838E-01  
	    ax[10] =  1.4679      
	    ax[11] =  0.13657E-01
	    ax[12] =  0.36782E-02    
	    ax[13] =  0.13155E-01   
	    ax[14] = -0.13720E-01  
	    ax[15] =  0.34007E-01 
	    ax[16] = -0.21325E-01
	    ay[ 1] = -0.20904E-01
	    ay[ 2] =  0.45812E-01
	    ay[ 3] = -0.55235E-01
	    ay[ 4] = -0.15033E-01
	    ay[ 5] =  2.0164    
	    ay[ 6] =  0.26925    
	    ay[ 7] =  1.4637    
	    ay[ 8] =  0.17426E-01
	    ay[ 9] =  0.26030E-01
	    ay[10] =  0.37160E-01
	    ay[11] =  0.17742E-01
	    ay[12] = -0.37413E-01
	    ay[13] =  0.88650    
	    ay[14] =  0.22941E-02
	    ay[15] =  0.10848E-01
	    ay[16] =  0.48358E-01
        }
end
