# BRACEWELL -- Transform functions from Bracewell's book
#
# This is an educational script intended to help Fourier analysis neophytes
# develop a feel for how things in the spatial (or time) domain map onto
# structure in the transform domain.  The user may select the functional
# form from among a set of old favorites (for Bracewell readers) and the number
# of points in the data set.  The center of the function may also be shifted
# from the origin of the coordinate system, which is pixel 1 for the routines
# used in this procedure.
#
# The bold and inquisitive user may also adjust the "characteristic width"
# of the function.  This is a term I have coined and has different meaning
# for different functions.  For things like the boxcar or Gaussian, the notion
# of width is pretty intuitive; for things like the impulse pairs, it is the
# separation of the pulses.  It is meaningless for things like a delta function
# or a step function.
#--
procedure bracewell (function, npts, shift)

string	function = "" {prompt="> Function"}
int	npts = 100 {prompt="> Number of points", min=1}
int	shift = 0 {prompt="> Shift origin this many pixels"}

real	const = 1.0 {prompt="> Value of constant function"}
real	fcw = 10 {prompt="> Characteristic width of function", min=1}

string	mode="al"
#------------------------------------------------------------------------------
begin
	string	funfil, tbuf
	real	xinc

	if (!defpac ("plot")) {
	    print ("The plot package must be loaded.")
	    bye()
	}

	xinc = 1.0/fcw
	funfil = function//str(npts)//".dat"
	testfunc (function, npoints=npts, delta=xinc, constant=const,
		oshift=shift, >funfil)

	tbuf = function//": "//str(npts)//" pts, "
	tbuf = tbuf//" width="//str(fcw)//", "
	tbuf = tbuf//str(shift)//" pixel shift"

	graph (funfil, wy1=-1, wy2=2, title=tbuf, >G"brcwll1.vdm")
	testfunc ("const", npoints=npts, delta=xinc, oshift=shift, |
		graph wy1=-1, wy2=2, title="input imaginary = 0.0",
		>G"brcwll2.vdm")
	ncar_fft (funfil, outreal="brcwllr.out", outimag="brcwlli.out")
	graph ("brcwllr.out", title="forward transform: real part",
		>G"brcwll3.vdm")
	graph ("brcwlli.out", title="forward transform: imaginary part",
		>G"brcwll4.vdm")

	gkimosaic ("brcwll?.vdm", nx=2, ny=2)

	delete (funfil)
	delete ("brcwll?.out")
	delete ("brcwll?.vdm")
end
