#
# SHIFTFIND  --  Measures peak position in list of cross-correlation images.
#
#
#
#  Development history:
#  -------------------
#  12-Apr-96: Task created (I.Busko)
#  10-Jun-96: Use new task 'imextreme' to locate crosscor peak.
#  14-Jun-96: Boxsize parameter in imextreme task.
#  14-Mar-97: Box center coordinates in imextreme task.
#  07-Jul-97: Revise calls that rely on the 'imtype' setting.


procedure shiftfind (cclist, outfile)

char	cclist      = ""     {prompt="Cross-corr image list/template"}
char	outfile     = ""     {prompt="Output file name"}
real	xcenter     = INDEF  {prompt="Search box X position"}
real	ycenter     = INDEF  {prompt="Search box Y position"}
real	boxsize     = INDEF  {prompt="Box size where to search for peak"}
real	fwhm        = 7.0    {prompt="Peak FWHM"}
real	ellip       = 0.05   {prompt="Peak ellipticity"}
real	pa          = 45.    {prompt="Peak position angle"}
int	fitbox      = 7      {prompt="Box size used to fit peak"}
bool	kellip      = yes    {prompt="Keep ellipticity fixed ?"}
bool	kpa         = yes    {prompt="Keep position angle fixed ?"}
file	tempdir     = "tmp$" {prompt="Directory for temporary files"}

struct	*filelist            {prompt="Internal parameter, do not use."}
struct	*prlist              {prompt="Internal parameter, do not use."}

char    version  = "14Mar97" {prompt="Date of installation"}

begin
	# These hold input parameters.
	file	ccl, out
	real	xc, yc, box
	# These are for internal use
	file	ccname, temp1, temp2, temp3
	file    tempfile
	char	msg
	real	xcent, ycent, excent, eycent
	real    xshift, yshift

	# Check for the presence of pre-requisite packages.
	msg = ""
	if (!deftask("tcalc"))      msg = msg // " ttools"
	if (!deftask("pixlocate"))  msg = msg // " imgtools"
	if (!deftask("n2gaussfit")) msg = msg // " fitting"
	if (!deftask("imcntr"))     msg = msg // " proto"
	if (strlen(msg) > 0) {
	    printf ("Please, load the following packages: %s\n", msg)
	    bye
	}

	# Read task parameters.
	ccl                 = cclist
	out                 = outfile
	xc                  = xcenter
	yc                  = ycenter
	box                 = boxsize
	tgausspars.a        = 0.0
	tgausspars.fwhm     = fwhm
	tgausspars.ellip    = ellip
	tgausspars.theta    = pa
	tgausspars.boxsize  = fitbox
	tgausspars.va       = no
	tgausspars.vampl    = yes
	tgausspars.vxcent   = yes
	tgausspars.vycent   = yes
	tgausspars.vfwhm    = yes
	tgausspars.vellip   = !kellip
	tgausspars.vtheta   = !kpa
	n2gaussfit.averbose = no
	controlpars.method  = "amoeba"

	# Create temporary file names.
	tempfile = tempdir // "dither"
	temp1 = mktemp (tempfile)
	temp2 = mktemp (tempfile)
	temp3 = mktemp (tempfile)

	# Reset output stream.
	printf ("", > out)

	# Make list with input files
	sections (ccl, option="root", > temp3)

	# Get center of cross-corr image.
	filelist = temp3
	i = fscan (filelist, ccname)
	imgets (ccname, "i_naxis1", mode="h")
	xcent = real(imgets.value) / 2. + 1.
	imgets (ccname, "i_naxis2", mode="h")
	ycent = real(imgets.value) / 2. + 1.

	# Do for each image in input list:
	prlist   = temp3
	filelist = temp3
	while (fscan (filelist, ccname) != EOF) {

	    # Brightest pixel gives peak starting position and amplitude.
	    imextreme (ccname, boxsize=box, x=xc, y=yc, verbose-, mode="h")
	    tgausspars.ampl  = imextreme.max
	    tgausspars.xcent = real(imextreme.xmax)
	    tgausspars.ycent = real(imextreme.ymax)

	    # Fit 2-D gaussian to peak.
	    n2gaussfit (ccname, temp2, rt-, resample+, errtype="uniform", \
                        verbose+, mode="h")

	    # Get results.
	    imgets (ccname, "i_naxis1", mode="h")
	    tabpar (temp2, "coeff3", 1, mode="h")
	    xshift =  real (tabpar.value)
	    xshift = -(xshift - xcent)
	    tabpar (temp2, "err3", 1, mode="h")
	    excent = real (tabpar.value)
	    tabpar (temp2, "coeff4", 1, mode="h")
	    yshift = real (tabpar.value)
	    yshift = -(yshift - ycent)
	    tabpar (temp2, "err4", 1, mode="h")
	    eycent = real (tabpar.value)

	    # Output.
	    # STDOUT
	    i = fscan (prlist, ccname)
	    printf ("Crosscor: %s\n", ccname)
	    printf ("Xshift = %9.4f   Error = %6.4f\n", xshift, excent)
	    printf ("Yshift = %9.4f   Error = %6.4f\n\n", yshift, eycent)
	    # File
	    printf ("%s %9.4f %6.4f %9.4f %6.4f\n", ccname, xshift, excent,
	            yshift, eycent, >> out)

	    # Delete temporaries.
	    delete  (temp1, verify-, >& "dev$null")
	    delete  (temp2 // ".tab", verify-, >& "dev$null")
	}
	delete  (temp3, verify-, >& "dev$null")
	filelist = ""
	prlist   = ""
end
