include <imhdr.h>

procedure t_fixd ()

# Task parameters
char	input[SZ_PATHNAME]
char	pixels[SZ_LINE]
char	dmode[SZ_LINE]
int	boxsz
bool	verbose

# Local variables
int	range[3,50+2], nval, span, npix, ngroups, i, pixel, nxsteps, overscan
int	ngrp, pixsv, list
real	datamin, datamax, scl, mean1, mean2, sigma, slope
pointer	spec, sp, image, im, scale

int	decode_ranges(), get_next_number()
int	imtopen(), imtgetim(), immap(), imgl1r(), impl1r(), imgeti()
int	clgeti()
bool	clgetb(), streq()

begin
	# Allocate memory
	call smark( sp )
	call salloc( image, SZ_FNAME, TY_CHAR)

	# Get parameters
	call clgstr( "images", input, SZ_PATHNAME)
	call clgstr( "pixels", pixels, SZ_LINE)
	call clgstr( "dmode",   dmode, SZ_LINE)
	if (streq(dmode,"noisy")) boxsz = clgeti( "boxsize" )
	verbose = clgetb( "verbose" )

	# Open image list
	list = imtopen( input )

	# Loop over file names specified in input image list
	while( imtgetim( list, Memc[image], SZ_FNAME) != EOF ) {

	   # Open the image
	   im = immap( Memc[image], READ_WRITE, 0 )

	   # Get header parameters
	   npix	 = IM_LEN( im, 1)
	   ngroups  = imgeti( im, "GCOUNT" )
	   nxsteps  = imgeti( im, "NXSTEPS" )
	   overscan = imgeti( im, "OVERSCAN" )
	   span = nxsteps * overscan

	   if (streq(dmode,"dead")) {
	       call malloc( scale, overscan*overscan, TY_REAL)
	       call sclmat( Memr[scale], overscan )
	   }

	   call malloc( spec, npix, TY_REAL)

	   # Loop over image groups
	   do ngrp = 1, ngroups {
	      call gf_opengr( im, ngrp, datamin, datamax, 0)
	      call amovr( Memr[imgl1r(im)], Memr[spec], npix)

	      # Decode the input list of pixel values
	      if (decode_ranges( pixels, range, 50+2, nval ) == ERR )
		  call error ( 1, "Invalid list of pixels specified" )

	      # Loop over list of pixel values
	      pixel = 0
	      pixsv = 0

	      while (get_next_number( range, pixel ) != EOF) {

		 # Fix pixels affected by dead diodes
	   	 if (streq(dmode,"dead")){

		     # Fix scaling of pixels affected by previous correction
		     if (pixsv!=0 && pixel<pixsv+span) {
			 call adivkr(Memr[spec+pixel-1], scl, 
				     Memr[spec+pixel-1], pixsv+span-pixel )
			 call aavgr( Memr[spec+pixel-9],       8, mean1, sigma)
			 call aavgr( Memr[spec+pixel-1], nxsteps, mean2, sigma)
			 call sclfnd(Memr[scale], overscan, mean2/mean1, scl)
			 call amulkr(Memr[spec+pixel-1], scl,
				     Memr[spec+pixel-1], pixsv+span-pixel )
		     }

		     call aavgr (Memr[spec+pixel-9], 8, mean1, sigma)
		     call aavgr (Memr[spec+pixel-1], nxsteps, mean2, sigma)
		     call sclfnd(Memr[scale], overscan, mean2/mean1, scl)

		     if (verbose && ngrp==1) {
		         call printf( "Scale pixels %d - %d by %g\n")
		         call pargi( pixel )
		         call pargi( pixel+span-1 )
		         call pargr( scl )
		         call flush( STDOUT )
		     }

		     call amulkr(Memr[spec+pixel-1], scl, Memr[spec+pixel-1],
				 span)

		     pixsv = pixel
			
		 } else {
		 # Fix pixels affected by noisy diodes

		     if (verbose && ngrp==1) {
			 call printf( "Interpolate over pixels %d - %d\n")
			 call pargi( pixel )
			 call pargi( pixel+span-1 )
			 call flush( STDOUT )
		     }

		     call aavgr(Memr[spec+pixel-boxsz-1], boxsz, mean1, sigma)
		     call aavgr(Memr[spec+pixel+span-1],  boxsz, mean2, sigma)
		     slope = (mean2-mean1)/(span+1)

		     do i = pixel, pixel+span-1 {
			Memr[spec+i-1] = mean1 + (i-pixel+1)*slope
		     }
		 }
	      } # End of loop over bad pixel list

	      # Write fixed spectrum back to image
	      call amovr( Memr[spec], Memr[impl1r(im)], npix)

	   } # End of loop over image groups
	
	   # Close this image and free memory
	   call imunmap( im )
	   call mfree( spec, TY_REAL )
	   call mfree( scale, TY_REAL )

	} # End of loop over image list

	# All done; close image list and free memory
	call imtclose( list )
	call sfree( sp )
end


procedure sclmat( scale, overscan )

real	scale[overscan,overscan]
int	overscan

int	i, j

begin
	do i = 0, overscan-1 {
	   do j = 0, overscan-1 {
	      scale[i+1,j+1] = real(overscan-j) / (overscan-i)
	   }
	}
end


procedure sclfnd( scale, overscan, ratio, scl )

real	scale[overscan,overscan]
int	overscan
real	ratio
real	scl

int	i, j
real	llim, ulim

begin
	scl = 1.0

	do i = 0, overscan-1 {
	   do j = 0, overscan-1 {
	      llim = 0.95 * scale[i+1,j+1]
	      ulim = 1.05 * scale[i+1,j+1]
	      if ((ratio>=llim) && (ratio<=ulim)) scl = 1./scale[i+1,j+1]
	   }
	}
end
