include <imhdr.h>
include	<ctype.h>
include	<time.h>
include	<tbset.h>
include	<foc.h>

define	INIT_COORD_SIZE	2000
define	INC_COORD_SIZE	1000
define	MAX_MODELS	50
define	MAX_MODEL_DIM	50
define	SZ_MODEL_INFO	4
define	MODEL_NUM	Meml[$1]
define	MODEL_IN_RES	Memb[$1+SZ_MODEL_INFO*$2-3]
define	MODEL_PTR	Meml[$1+SZ_MODEL_INFO*$2-2]
define	MODEL_NAXIS1	Meml[$1+SZ_MODEL_INFO*$2-1]
define	MODEL_NAXIS2	Meml[$1+SZ_MODEL_INFO*$2  ]

# RFINDX -- locate reseau marks in an image
#
# D. Giaretta, 01-Aug-1987  Original SPP version
# D. Giaretta, 05-Apr-1988  Correct SAMPBEG->LINEBEG in mark offset
# D. Giaretta, 01-May-1988  Add parameter noindefs to avoid creating indefs
# D. Giaretta, 06-Jul-1988  Correction after xrescorl to put INDEF if reqd
# Phil Hodge,   4-Apr-1990  In xcormod, replace  "if (mod( max1, 2) == 0)"
#					    with "if (mod (max2, 2) == 0)".
# Phil Hodge,  11-Apr-1993  Delete unused stuff from calling sequences.
# Phil Hodge,  19-Aug-1993  In xcormod, include check on zero denominator.

procedure t_rfindx()

char	input[SZ_FNAME]		# input image
char	inres[SZ_FNAME]		# input reseau file
char	refentry[SZ_R_PATTERN]	# input reseau entry
char	outres[SZ_FNAME]	# output reseau file
char	entry[SZ_R_PATTERN]	# output reseau entry
int	search1, search2	# search area size (x, y)
real 	depth     		# Min. contrast in correlation 
char	coordfile[SZ_FNAME]	# File with approximate coords 
int 	rsize1     		# Number of cols in grid ignored if no coordfile
int 	rsize2     		# Number of rows in grid ignored if no coordfile
char	modelfile[SZ_FNAME]	# File with model details 
bool 	verbose			# print a lot of diagnostics?
bool	noindefs		# should indef NOT be inserted if no mark found
int  	inner     		# Number of inner marks found 
int  	outer     		# Number of boundry marks found
#--

pointer	foc_coord		# FOC coord structure
pointer	rpin			# input reseau file pointer
pointer	rpout			# output res. file pointer
pointer	rentp			# run entry structure
pointer	rentpout		# correct output values structure
pointer	rm
pointer	rs_open()
pointer	x_foc_immap(), im

int	nrows, ncols, junk, nowhite()
bool	inplace
bool	streq()
int	clgeti(), imaccf()
real	clgetr()
bool	clgetb()

begin
	rpin   = NULL
	rpout  = NULL
	nrows  = INDEFI
	ncols  = INDEFI
	rsize1 = INDEFI
	rsize2 = INDEFI

	call clgstr("input", input, SZ_FNAME)
	if ( nowhite( input, input, SZ_FNAME) == 0 )
	    call error( 0, "input file must be given")

	# special treatment for FOC coords

	call salloc( foc_coord, SZ_FOC_COORD, TY_STRUCT)
	FOC_COORD_INIT(foc_coord) = false
	im = x_foc_immap(input, foc_coord, READ_ONLY, NULL)

	call clgstr( "inres", inres, SZ_FNAME)
	junk = nowhite( inres,  inres, SZ_FNAME)
	if (  inres[1] != EOS ) {
	    call clgstr( "refentry", refentry, SZ_R_PATTERN)
	}

	call clgstr( "outres",  outres, SZ_FNAME)
	junk = nowhite( outres,   outres, SZ_FNAME)
	if (outres[1] != EOS) {
	    call clgstr( "entry", entry, SZ_R_PATTERN)
	}


	if ( inres[1] != EOS ) {
	    if ( streq(inres, outres) ) {
		inplace = true
		rpin = rs_open( inres, READ_WRITE, 0)
	    } else {
		inplace = false
		rpin = rs_open( inres, READ_ONLY, 0)
	    }
	}

	# ensure search area dimensions are odd integers
	search1 = 2*( clgeti( "search1") / 2) + 1
	search2 = 2*( clgeti( "search2") / 2) + 1
	depth   = clgetr( "depth")

	call clgstr( "coordfile", coordfile, SZ_FNAME)
	junk = nowhite( coordfile, coordfile, SZ_FNAME)

	if ( rpin != NULL ) {
	    nrows = RES_NROWS(rpin)
	    ncols = RES_NCOLS(rpin)
	} else if ( coordfile[1] != EOS) {
	    rsize1 = clgeti("ncols")
	    rsize2 = clgeti("nrows")
	}

	# get the coordinates
	call xrfgcoord( rpin, refentry, coordfile, rsize1, rsize2, 
			rentp, nrows, ncols)

	call clgstr( "modelfile", modelfile, SZ_FNAME)
	if ( nowhite( modelfile,  modelfile, SZ_FNAME) == 0 )
	    call error ( 0, "modelfile must be given")

	# get the model data
	call xrfgmodel (modelfile, rm)

	verbose = clgetb("verbose")
	noindefs= clgetb("noindefs")

	# things to do if we have an output file:

	# ensure that output reseau entry has a name, if we have an o/p res
	if ( outres[1] != EOS ){
	    if ( entry[1] == EOS ) {
		# if not given explicitly then use image ROOTNAME if present
		if ( imaccf( im, "ROOTNAME") == YES )
		    call imgstr( im, "ROOTNAME", entry, SZ_R_ENTNAME)

		# if all else fails then use a dummy
		if ( entry[1] == EOS ) 
		    call strcpy( "DUMMY", entry, SZ_R_ENTNAME)
	    }

	    # open the output, unless we are writing in place
	    if ( inplace ) {
	    	rpout = rpin
	    # otherwise create new file (rpin may be NULL)
	    } else {	
	    	rpout = rs_open( outres, NEW_FILE, rpin)
		call rs_new( 20, nrows*ncols, nrows, ncols, TY_REAL, rpout)
	    }
	} 

	# get space for output coords
	call rs_alloc( rpout, TY_REAL, rentpout)

	# Now locate the reseau marks
	call xresfind (im, foc_coord, search1, search2, depth, 
		verbose, noindefs,
		rm, nrows*ncols, 
	     	Memr[RES_XPT(rentp)], Memr[RES_YPT(rentp)], 
		Memb[RES_FPT(rentp)], 
		Memr[RES_XPT(rentpout)], Memr[RES_YPT(rentpout)], 
		Memb[RES_FPT(rentpout)] )

	# copy pointers so we have appropriate entry header info
	RES_XPT(rentp) = RES_XPT(rentpout)
	RES_YPT(rentp) = RES_YPT(rentpout)
	RES_FPT(rentp) = RES_FPT(rentpout)

	# figure out how well we did
	call rf_well( ncols, nrows, Memb[RES_FPT(rentp)], inner, outer)
	call clputi( "ncols", ncols)
	call clputi( "nrows", nrows)
	call clputi( "inner", inner)
	call clputi( "outer", outer)

	# close things down
	call imunmap(im)
	if (rpout != NULL) {
	    call xrfresout( rpin, rpout, rentp, entry, nrows, ncols, 
				foc_coord )
	    call rs_close( rpout)
	}
	call rs_close( rpin )
end

# XRFGCOORD -- get approximate coordinates, either from coord file 
# if present, or from input reseau file. In the former case we may not
# know how many data values to expect

procedure xrfgcoord( rpin, refentry, coordfile, rsize1, rsize2,
				rentp, nrows, ncols )

pointer	rpin			# i: res. pointer
char	refentry[SZ_R_PATTERN]	# i: entry name
char	coordfile[SZ_FNAME]	# i: coordfile name
int	rsize1, rsize2		# i: grid size if coord file given
pointer	rentp			# o: pointer to o/p res entry structure
pointer	nrows, ncols		# o: grid size
#--

pointer	open()
pointer	fdcoord			# text file pointer
pointer	temp, rs_template(), rlist
bool	sized
char	line[SZ_LINE]
long	i_in_line, ip
long	i
int	ent_num, rs_list()
real	dval
int	ctor()
bool	finished
long	init_size, saved_init_size
int	getline(), rs_readr()

begin

	rentp = NULL

	if ( !IS_INDEFI(rsize1) || !IS_INDEFI(rsize2) ) {
	    if ( !IS_INDEFI(nrows) && !IS_INDEFI(ncols) ) {
		nrows = max( nrows, rsize2 )
		ncols = max( ncols, rsize2 )
	    } else {
		nrows = rsize2
		ncols = rsize1
	    }
	}

	if ( IS_INDEFI(nrows) || IS_INDEFI(ncols) ) {
	    sized = false
	    init_size = INIT_COORD_SIZE
	} else {
	    sized = true
	    init_size  = 2*nrows*ncols
	    saved_init_size = init_size
	}


	# read in the text data into buffer - guess initial size, and
	# expand buffer if required later
	call malloc( temp, init_size, TY_REAL)

	if (coordfile[1] != EOS ) {
	    i_in_line = 1
	    ip = 0
	    fdcoord = open(coordfile, READ_ONLY, TEXT_FILE)
	    finished = false
	    while ( getline(fdcoord, line) != EOF && !finished ) {
		i_in_line = 1
		while ( ctor(line, i_in_line, dval) >= 1 && line[1] != '#' ) {
		    if (ip > init_size) {
			init_size = init_size + INC_COORD_SIZE
			call realloc( temp, init_size, TY_REAL)
		    }
		    Memr[temp + ip ] = dval
		    ip = ip + 1
		}
		if (sized && ip >= init_size ) 
		    finished = true
	    }
	    call close(fdcoord)

	    # set the size as 1*ip if indef sized
	    if ( !sized || ip != saved_init_size) {
		nrows= 1
		ncols = ip/2
	    }
	

	    # copy data to x and y separate buffers	
	    if ( rpin == NULL) {
	        # we need a fake res. pointer for rs_alloc
		call salloc( rentp, LEN_ENTRY_INFO, TY_STRUCT )
		call salloc( RES_XPT( rentp), RES_COLSPERRES + nrows*ncols,
						TY_REAL)
		call salloc( RES_YPT( rentp), RES_COLSPERRES + nrows*ncols,
						TY_REAL)
		call salloc( RES_FPT( rentp), RES_COLSPERRES + nrows*ncols,
						TY_BOOL)
		RES_ENT_NROWS( rentp) = NULL
	    } else {
	        call rs_alloc( rpin, TY_REAL, rentp )
	    }

	    for( i=1; i<= nrows*ncols; i=i+1) {
		Memr[RES_XPT(rentp)+i-1]   = Memr[temp+2*i-2]
	        Memr[RES_YPT(rentp)+i-1]   = Memr[temp+2*i-1]
		if ( IS_INDEFR( Memr[temp+2*i-2]) || IS_INDEFR(Memr[temp+2*i-1]) )
		    Memb[RES_FPT(rentp)+i-1] = true
		else
		    Memb[RES_FPT(rentp)+i-1] = false
	    }

	    call mfree( temp, TY_REAL)

	} else if ( rpin != NULL ) {
	    nrows     = RES_NROWS(rpin)
	    ncols     = RES_NCOLS(rpin)
	    call rs_alloc( rpin, TY_REAL, rentp)
	    rlist     = rs_template( rpin, refentry)
	    ent_num   = rs_list( rlist)
	    if ( rs_readr ( rpin, rentp, EOS, ent_num ) == RES_F_NONEXTENTRY)
		call error( 0, " no matching entry in input reseau file")
	} else 
	    call error (0, " coordinates must be in either coordfile or inres")

end

# XRFGMODEL -- get models to fit to reseau marks

procedure xrfgmodel (modelfile, rm)

char	modelfile[SZ_FNAME]	# i: model file
pointer	rm			# o: pointer to model data
#--

pointer	open()
pointer	fdmodel
pointer	currpt
char	line[SZ_LINE]
long	ip, i_in_line
real	dval
int	ctor()
int	tot_models, nrows, ncols
int	getline()
int	mod_dim, mod_count
bool	in_model

string	bigmod_mod	"Model in model file too large"

begin

	tot_models = 0
	call salloc( rm, (MAX_MODELS + 1)*SZ_MODEL_INFO, TY_REAL)
	
	# read models from text file 
	if (modelfile[1] != EOS) {
	    # read in the text data into buffer - guess initial size, and
	    # expand buffer if required later
	    
	    i_in_line = 1
	    in_model  = false
	    mod_dim   = 1
	    fdmodel = open(modelfile, READ_ONLY, TEXT_FILE)
	    while ( getline(fdmodel, line) != EOF && tot_models <= MAX_MODELS) {
	        i_in_line = 1
	        while ( ctor(line, i_in_line, dval) >= 1 && line[1] != '#' ) {
		    if (!in_model) {
			if (mod_dim == 1) {
			    tot_models = tot_models + 1
			    ncols = dval
			    MODEL_NAXIS1(rm, tot_models) = ncols
			    mod_dim = 2
			} else {
			    nrows = dval
			    MODEL_NAXIS2(rm,tot_models) = nrows
			    mod_dim   = 1
			    in_model  = true
			    mod_count = 1
			    call salloc ( currpt, nrows*ncols, TY_REAL)
			    MODEL_PTR(rm, tot_models) = currpt
			    MODEL_IN_RES(rm, tot_models) = false

			    # check that models are sensible
			    if (nrows > MAX_MODEL_DIM || ncols > MAX_MODEL_DIM )
		    		call error (0, bigmod_mod)
			}
		    } else {
		        Memr[currpt+mod_count-1]=dval
		        ip = ip + 1
			mod_count = mod_count + 1
			if ( mod_count > nrows*ncols )
			    in_model = false
		    }
	        }
	    }
	    call close( fdmodel )

	    MODEL_NUM(rm) = tot_models 
	}

	if ( tot_models <= 0 )
	    call error( 0, "no model data found in modelfile")

end

# XRESFIND -- locate reseau marks given approximate coordinates

procedure xresfind (im, foc_coord, search1, search2, depth, 
			verbose, noindefs,
			rm, nres, x, y, f, xout, yout, fout )		 

pointer	im			# i: input image pointer
pointer	foc_coord		# i: foc coord structure
int	search1, search2	# i: search dimensions axis1,2
real	depth			# i: depth of correlation required
bool	verbose			# i: verbose diagnostic messages
bool	noindefs		# i: do NOT put INDEF in o/p file
pointer	rm			# i: pointer to model info
int	nres			# i: number of reseau
real	x[ARB], y[ARB]		# i: input approx positions
bool	f[ARB]			# i: is approx pos. INDEF
real	xout[ARB], yout[ARB]	# o: output approx pos
bool	fout[ARB]		# o: is found location INDEF
#--

int	maxax1, maxax2
int	i, j
real	mxz1, mxz2
real	xtemp, ytemp
bool	ftemp
pointer	corrpt

begin

	maxax1 = 0
	maxax2 = 0
	do i=1, MODEL_NUM(rm) {
	    maxax1 = max( MODEL_NAXIS1(rm, i), maxax1 )
	    maxax2 = max( MODEL_NAXIS2(rm, i), maxax2 )
	}

	mxz1 = (search1 + maxax1 + 1) / 2
	mxz2 = (search2 + maxax2 + 1) / 2


	call salloc( corrpt, search1*search2, TY_REAL)


	do i = 1, nres {
#####Could sort res locations by y and then use j=index(i)
	    j = i
	    if (verbose) {
		call printf( "  _____________________________________\n")
		call printf( " reseau mark no. %4d, input position ( %g, %g )\n")
		call pargi( i)
		call pargr( x[j])
		call pargr( y[j])
	    }
	    if ( !f[j] && 
		 x[j] > FOC_SAMPBEG(foc_coord) + mxz1 &&
	     	 x[j] < FOC_NPIX1(foc_coord) + 
					FOC_SAMPBEG(foc_coord) -1 - mxz1 &&
	     	 y[j] > FOC_LINEBEG(foc_coord) + mxz2 &&
	     	 y[j] < FOC_NPIX2(foc_coord) + 
					FOC_LINEBEG(foc_coord) -1 - mxz2  )   {

		call xrescorl (im,
			x[j] - FOC_SAMPBEG(foc_coord) + 1.,
			y[j] - FOC_LINEBEG(foc_coord) + 1.,
			rm, search1, search2, depth, 
			maxax1, maxax2, verbose, noindefs, Memr[corrpt],
			xtemp, ytemp, ftemp )
		if ( !IS_INDEFR(xtemp) ) {
		    xout[j] = xtemp + FOC_SAMPBEG(foc_coord) - 1
		    yout[j] = ytemp + FOC_LINEBEG(foc_coord) - 1
		} else {
		    xout[j] = INDEFR
		    yout[j] = INDEFR
		}
		fout[j] = ftemp
	    } else {
		if ( noindefs ) {
		    if (verbose)
			call printf( " keep approximate position unchanged\n")
		    xout[j] = x[j]
		    yout[j] = y[j]
		} else {
		    xout[j] = INDEFR
		    yout[j] = INDEFR
		}
		fout[j] = true
	    }

	    if (verbose) {
		call printf( " new position %g, %g \n")
		call pargr( xout[j])
		call pargr( yout[j])
		if (!fout[j]) {
		    call printf( " displacements: %g, %g \n")
		    call pargr( xout[j] - x[j] )
		    call pargr( yout[j] - y[j] )
		}
	    }

	    call flush ( STDOUT )
	}
	
end

# XRESCORL -- locate one reseau mark given several models

procedure xrescorl (im, x, y, 
			rm, search1, search2, depth, 
			maxax1, maxax2, verbose, noindefs, corr,
			xout, yout, fout )

pointer	im			# i: input image pointer
real	x		# i: approx x coord - rel to start of image
real	y		# i: approx y coord - rel to start of image
pointer	rm			# i: pointer to model info
int	search1, search2	# i: search dimensions axis1,2
real	depth			# i: depth of correlation required
int	maxax1, maxax2		# i: maximum model size axes
bool	verbose			# i: verbose diagnostic messages
bool	noindefs		# i: do NOT put INDEF in o/p file
real	corr[search1, search2]	# i: correlation matrix work area
real	xout, yout	# o: output approx pos - rel to start of image
bool	fout			# o: is found location INDEF
#--

pointer	bufptr, imgs2r()
int	i
int	deltax, xs, xe
int	deltay, ys, ye
real	xref[MAX_MODELS], yref[MAX_MODELS], minref[MAX_MODELS]
int	best_mod

begin

	deltax = (search1 + maxax1 + 1) / 2
	deltay = (search2 + maxax2 + 1) / 2
	xs   = x - deltax
	xe   = xs + 2*deltax - 1
	ys   = y - deltay
	ye   = ys + 2*deltay - 1

	if (xs < 1 || ys < 1 || xe > IM_LEN(im,1) || ye > IM_LEN(im,2)) {
	    fout = true
	    if (noindefs) {
		xout = x 
		yout = y 
	    } else {
		xout = INDEFR
		yout = INDEFR
	    }
	    return
	}

	# read in image section
	bufptr = imgs2r( im, xs, xe, ys, ye)

	# process each model

	for (i=1; i<=MODEL_NUM(rm); i=i+1) {

	    if (verbose) {
		call printf( " processing model %d, ")
		call pargi( i )
	    }
	    call xcormod( Memr[bufptr], 2*deltax, 2*deltay, 	    
	       Memr[MODEL_PTR(rm, i)], MODEL_NAXIS1(rm, i), MODEL_NAXIS2(rm, i),
	       deltax, deltay, search1, search2, depth, 
	       verbose, corr, 
	       xref[i], yref[i], minref[i] )
	    if (verbose && !IS_INDEFR(xref[i]) ) {
	    	call printf( " pos. change: %6.2g, %6.2g\n")
	    	call pargr( -(x- xref[i] - deltax - xs) )
	    	call pargr( -(y- yref[i] - deltay - ys) )
	    }
	}
       
	# choose the best fitting model

	best_mod = 0
	for (i=1; i<=MODEL_NUM(rm); i=i+1) {
	    if ( !IS_INDEFR(xref[i]) && !IS_INDEFR(yref[i]) ) {
		if (best_mod == 0 )
		    best_mod = i
		else if ( minref[i] <= minref[best_mod] )
		    best_mod = i
	    }
	}

	if (best_mod == 0) {
	    xout = INDEFR
	    yout = INDEFR
	    fout = true
	    if (verbose)
	        call printf( " no good fits\n")
	    if ( noindefs ) {
		if (verbose)
		    call printf( " keep approximate position unchanged\n")
		xout = x 
		yout = y 
		fout = true
	    }
	} else {
	    # convert back to absolute coord system
	    xout = xref[best_mod] + deltax + xs
	    yout = yref[best_mod] + deltay + ys
	    fout = false
	    if (verbose) {
	        call printf( " model %d best fit, ")
	        call pargi( best_mod)
	    }
	}
	
end

# XCORMOD -- locate reseau mark given one model

procedure xcormod( in, ax1, ax2, 	    
	 model, max1, max2,
	 deltax, deltay, search1, search2, depth, verbose, corr,
	 xout, yout, minout )

real	in[ax1, ax2]		# i: image section
int	ax1, ax2		# i: image section dimensions
real	model[max1, max2]	# i: model array
int	max1, max2		# i: model array size
int	deltax, deltay		# i: approx centers in section
int	search1, search2	# i: search area size
real	depth			# i: depth required
bool	verbose			# i: print info??
real	corr[ search1, search2]	# o: correlation matrix work area
real	xout, yout		# o: calculated position
real	minout			# o: refined minimum correlation
#--

int	i, j, k, l, ic1, ic2, imin1, imin2
long	nsum
double	sum
real	maxcorr
real	mean
real	fdepth 
real	denom			# check for zero before dividing by this
	
begin

	# calculate scaling factor for model data
	sum  = 0
	nsum = 0
	do i= 1, ax2 {
	    do j = 1, ax1 {
		if ( !IS_INDEFR(in[j, i]) ) {
		    sum = sum + in[j, i]
		    nsum = nsum + 1
		}
	    }
	}

	if (nsum == 0 || sum <= 0.0) {
	    
	    xout   = INDEFR
	    yout   = INDEFR
	    minout = INDEFR
	    return
	}

	mean= sum/(100.0*nsum)

	# perform correlation

	ic1  = deltax - (search1 + 1)/2 - (max1 + 1)/2 + 1
	ic2  = deltay - (search2 + 1)/2 - (max2 + 1)/2 + 1
	nsum = 0
	do i = 1, search2 {
	    do j = 1, search1 {
		sum = 0.0
		do k = 1, max2
		    do l = 1, max1
			sum = sum + 
			      (mean*model[l, k] - in[j+l+ic1, i+k+ic2])**2
		corr[j, i] = sum/(max1*max2)
	    }
	}

	# locate minimum correlation

	imin1 = 1
	imin2 = 1
	maxcorr = 0.0
	do i = 1, search2 {
	    do j = 1, search1 {
		if ( corr[j, i] < corr[imin1, imin2]) {
		    imin1 = j
		    imin2 = i
		}
	        if ( corr[j, i] > maxcorr)
		    maxcorr = corr[j, i]
	    }
	}

	minout = corr[ imin1, imin2]
	fdepth = 100*(1.0-minout/maxcorr)

	if (imin1 == 1 || imin1 == search1 || imin2 == 1 || imin2 == search2) {
	    xout   = INDEFR
	    yout   = INDEFR
	    minout = INDEFR
	    if (verbose) 
		call printf( " correlation minimum at edge of search area\n")

	} else if ( fdepth < depth ) {
	        xout   = INDEFR
	        yout   = INDEFR
	        minout = INDEFR
	        if (verbose) {
		    call printf( " depth %6.2g insufficient\n")
			call pargr( fdepth )
	        }

	} else {

	    # refine minimum position and correlation

	    denom = corr[imin1-1, imin2] + corr[imin1+1, imin2]  -
		      2.0*corr[imin1, imin2]
	    if (denom == 0.) {
		xout = imin1 - int((search1+1)/2)
		if (verbose) 
		    call printf (" correlation is flat in X;\n        ")
	    } else {
		xout = 0.5*( corr[imin1-1, imin2]-corr[imin1+1, imin2] )/	
			denom - int((search1+1)/2) + imin1
	    }

	    if (mod (max1, 2) == 0)
	        xout = xout + 0.5

	    denom = corr[imin1, imin2-1] + corr[imin1, imin2+1]  -
		      2.0*corr[imin1, imin2]
	    if (denom == 0.) {
		yout = imin2 - int((search2+1)/2)
		if (verbose) 
		    call printf (" correlation is flat in Y;\n        ")
	    } else {
		yout = 0.5*( corr[imin1, imin2-1]-corr[imin1, imin2+1] )/	
			denom - int((search2+1)/2) + imin2
	    }

	    if (mod (max2, 2) == 0)
	        yout = yout + 0.5

	    if (verbose) {
		call printf( " depth of mark %6.2g, ")
		call pargr( fdepth)
	    }
	}
end

# XRFRESOUT -- write out reseau data to file

procedure xrfresout( rpin, rpout, rentp, entry, nrows, ncols, foc_coord)

pointer	rpin		# i: res.  input pointer
pointer	rpout		# i: res. output pointer
pointer	rentp		# i: res. entry pointer
char	entry[ARB]	# i: output entry name
int	nrows, ncols	# i: size of grid
pointer	foc_coord	# i: foc coord structure
#--

begin
	# if no entry read from res. file, create entry data
	if ( rentp == NULL)
	    call salloc( rentp, LEN_ENTRY_INFO, TY_STRUCT)

	if ( RES_ENT_NROWS(rentp) == NULL ) {
	    RES_ENT_NROWS(rentp) = nrows
	    RES_ENT_NCOLS(rentp) = ncols
	    RES_ENT_SFIELD1(rentp) = FOC_LINEBEG(foc_coord)
	    RES_ENT_SFIELD2(rentp) = FOC_SAMPBEG(foc_coord)
	    RES_ENT_SFIELD3(rentp) = FOC_NPIX1(foc_coord)
	    RES_ENT_SFIELD4(rentp) = FOC_NPIX2(foc_coord)
	}

	# insert new entry name
	call strcpy( entry, RES_ENT_ENTRY(rentp), SZ_R_ENTRY)

	# insert reseau entry data
	call rs_t_update( rentp, "F", SZ_R_TRACKING)
	call rs_insert( rpin, rpout, rentp )
end

# RF_WELL -- Figure out how well we did in terms of inner and outer marks

procedure rf_well( ncols, nrows, flag, inner, outer)
int	ncols		# i: number of cols in grid
int	nrows		# i: number of rows in grid
bool	flag[ARB]	# i: reseau bad?
int	inner		# o: number of inner marks
int	outer		# o: number of outer marks
#--

int	i, total

begin

	inner = 0
	outer = 0
	total = 0

	for ( i=1; i<=nrows*ncols; i=i+1 )
	    if ( !flag[i] )
		total = total + 1

	if ( nrows != 1) {
	    for  ( i=1; i<=ncols; i=i+1 ) {
	        if ( !flag[i] )
		    outer = outer + 1
	        if ( !flag[ ncols*(nrows-1) + i] )
		    outer = outer + 1
	    }
	}

	if ( ncols != 1) {
	    for  ( i=2; i<=nrows-1; i=i+1 ) {
	        if ( !flag[ ncols*(i-1) + 1] )
		    outer = outer + 1
	        if ( !flag[ ncols*i ] )
		    outer = outer + 1
	    }
	} 
	    

	inner = total - outer

end
