# DARKX -- create dark count image image 

procedure darkx ( input, output, option)

file	input = " " {prompt = "input text file with names if input images"}
file 	output= " " {prompt = "output file containing dark count image"}
string  option= "median" {prompt = "type of filtering", \
		enum="median|average"}
string	title = "dark count" {prompt = "title for output image"}
real    low_reject = 0.0 {prompt="reject these low pixels when calc. average"}
real    high_reject = 0.0 {prompt="reject these high pixels when calc. average"}
bool	verbose  = no	{prompt = "print details of the operations?"}
struct	*inimglist      {prompt = "not user parameter" }
#--

begin

string	intempfile 	= "tmp$dark1x"
string	outtempfile 	= "tmpdarkzzzzzx"	# note that we cannot use tmp$
						# as this causes error in imsum
file	itfile, inimg, otfile, outimg, totfile, toutput, tinput
real	t_low, t_high, exptime
string	toption, ttitle
int	num_images
bool	tverb

	# we must have the images abd cbbdut packages
	if ( !defpac("images") )
	    images >dev$null

	if ( !defpac("ttools") )
	    ttools >dev$null

	cache keypar
	
	num_images = 0

	# make a file with all the file names separately
	itfile 	 = mktemp( (intempfile))
	otfile 	 = mktemp( (outtempfile))
	tinput   = "@"//(input)
	files( (tinput), >>(itfile) )
	toutput = output

	# transfer the user parameters to temporary variables
	toption		= option
	ttitle		= title
	t_low		= low_reject
	t_high		= high_reject
	tverb 		= verbose

	# set the listfiles as the file with separated names
	inimglist 	= (itfile)

	# loop over all the files
	while ( fscan( inimglist,   inimg) != EOF ) {

	    # create temporary intermediate file and add name to list file
	    outimg = mktemp("tmp$dark3x")

	    # perform division by exposure time to get count/sec
	    keypar( inimg, "exptime")
	    exptime = real( keypar.value)
	    imarith (inimg, "/", exptime,outimg,pixtype="real",calctype="real")

	    # add this file name to the list file	    
	    print ( outimg, >>(otfile) )
	    num_images = num_images + 1

	}

	if ( num_images > 1) {
	    # now make the average of the individual dark images
	    totfile = "@"//(otfile)
	    imsum ( totfile, toutput, title=ttitle, hparams="", \
		pixtype="real", calctype="real", option=toption, \
		low_reject=t_low, high_reject=t_high, verbose=tverb )
	    imdelete (totfile, yes, v-)
	} else if ( num_images == 1) {   # but if just one - then copy
	    imcopy ( outimg, toutput, verbose=tverb)
	    imdelete ( outimg, yes, v-)
	} else { 			 # we must have at least one image
	    error ( 1, "no images available")
	}

	delete ( (itfile) )
	delete ( (otfile) )

end
