# FIXNOISEX -- create fix pattern noise image 

procedure fixnoisex ( input, output, smoothing)

file	input = "" {prompt = "input images for generation of fix noise pattern"}
file 	output= "" {prompt = "output images containing patten noise"}

string smoothing= "boxcar" {prompt = "type of smoothing", \
		enum="boxcar|convolve|median|mode"}
int 	xwindow= 13  {prompt = "x side of box for smoothing/processing input ",
			min = 1}
int 	ywindow= 13  {prompt = "y side of box for smoothing/processing input ",
			min = 1}
string 	kernel= "" {prompt = "kernel to be used if convolving"}
string	title = "fix pattern noise" {prompt = "title for output image"}
real    low_reject = 0.0 {prompt="reject these low pixels when calc. average"}
real    high_reject = 0.0 {prompt="reject these high pixels when calc. average"}
string	boundary = "nearest" { prompt = "type of boundary extension",\
				enum="nearest|constant|reflect"}
real	constant = 0.0	{prompt = "constant for constant boundary extension"}
real	zerodiv  = 0.0  {prompt = "replace zero's in smoothed image by this"}
bool	verbose  = no	{prompt = "print details of the operations?"}
struct	*inimglist      {prompt = "not user parameter" }
#--

begin

string	interfile	= "tmp$inter"
string	intempfile 	= "tmp$fixin"
string	outtempfile 	= "fixout"
file	itfile, inimg, otfile, outimg, intermed, totfile, toutput
int	txwindow, tywindow
string	tboundary, tkernel
real	tconstant, tzerodiv, t_low, t_high
string	tsmoothing, ttitle
int	num_images
bool	tverb

	# we must have the images package
	if ( !defpac("images") )
	    images >dev$null

	num_images = 0

	# make a file with all the file names separately
	itfile 	 = mktemp( (intempfile))
	otfile 	 = mktemp( (outtempfile))
	intermed = mktemp( (interfile) )
	files( (input), >>(itfile) )
	toutput = output

	# transfer the user parameters to temporary variables
	tsmoothing	= smoothing
	txwindow	= xwindow
	tywindow  	= ywindow
	if ( tsmoothing == "convolve" )
	    tkernel = kernel
	ttitle		= title
	t_low		= low_reject
	t_high		= high_reject
	tboundary 	= boundary
	tconstant 	= constant
	tzerodiv	= zerodiv
	tverb 		= verbose

	# set imdivide parameters
	imdivide.constant 	= tzerodiv
	imdivide.title 		= "*"
	imdivide.rescale 	= "norescale"
	imdivide.mean 		= "1"
	imdivide.verbose 	= tverb

	# set the listfiles as the file with separated names
	inimglist 	= (itfile)

	# loop over all the files
	while ( fscan( inimglist,   inimg) != EOF ) {

	    # create temporary intermediate file and add name to list file
	    outimg = mktemp("tmp$fff")

	    # perform the operation requested to smooth image
	    if ( tsmoothing == "boxcar") {
		boxcar.boundary = (tboundary)
		boxcar.constant = (tconstant)
	        boxcar( inimg, intermed, txwindow, tywindow)
	    } else if (tsmoothing == "convolve") {
		convolve.boundary = tboundary
		convolve.constant = tconstant
	        convolve( inimg, intermed, tkernel )
	    } else if ( tsmoothing == "median" ) {		
		median.boundary = tboundary
		median.constant = tconstant
	        median( inimg, intermed, txwindow, tywindow )
	    } else if ( tsmoothing == "mode" ) {		
		mode.boundary = tboundary
		mode.constant = tconstant
	        mode( inimg, intermed, txwindow, tywindow  )
	    }

	    # now divide the image into the original
	    imdivide ( inimg, intermed, outimg )

	    # delete the intermediate image
	    imdelete ( intermed, yes, v-)

	    # add this file name to the list file	    
	    print ( outimg, >>(otfile) )
	    num_images = num_images + 1

	}

	if ( num_images > 1) {
	    # now make the average of the individual fix pattern images
	    totfile = "@"//(otfile)
	    imsum ( totfile, toutput, title=ttitle, hparams="", \
		pixtype="real", calctype="real", option="average", \
		low_reject=t_low, high_reject=t_high, verbose=tverb )
	    imdelete (totfile, yes, v-)
	} else if ( num_images == 1) {   # but if just one - then copy
	    imcopy ( outimg, toutput, verbose=tverb)
	    imdelete ( outimg, yes, v-)
	} else { 			 # we must have at least one image
	    error ( 1, "no images available")
	}

	delete ( (itfile) )
	delete ( (otfile) )

end
