# FORMATDEPX -- create FOC format dependent non-uniformity image

procedure formatdepx ( input, f3image, output, illumratio)

file	input = " " {prompt = "input files to make format dep non-unif image "}
file	f3image = " " {prompt = "F3 format images"}
file 	output= " " {prompt = "output files containing fix patten noise image"}
real	illumratio = 1.0 {prompt = "ratio of illum levels large/small format"}
string smoothing= "boxcar" {prompt = "type of smoothing", \
		enum="boxcar|convolve|median|mode|none"}
int 	xwindow= 13  {prompt = "x side of box for smoothing/processing input ",
			min = 1}
int 	ywindow= 13  {prompt = "y side of box for smoothing/processing input ",
			min = 1}
string 	kernel= " " {prompt = "kernel to be used if convolving"}
string	title = "format dependent non-uniformity" {prompt = \
				"title for output image"}
real    low_reject = 0.0 {prompt="reject these low pixels when calc. average"}
real    high_reject = 0.0 {prompt="reject these high pixels when calc. average"}
string	boundary = "nearest" { prompt = "type of boundary extension",\
				enum="nearest|constant|reflect"}
real	constant = 0.0	{prompt = "constant for constant boundary extension"}
real	zerodiv  = 0.0  {prompt = "replace zero's in smoothed image by this"}
bool	verbose  = no	{prompt = "print details of the operations?"}
struct	*inimglist      {prompt = "not user parameter" }
struct	*f3imglist      {prompt = "not user parameter" }
#--

begin

string	interfile	= "tmp$inter"
string	intempfile 	= "tmp$fixin"
string	f3temp		= "tmp$f3im"
string	outtempfile 	= "fixout"
file	itfile, inimg, otfile, outimg, intermed, totfile, toutput, tmpoutput
file	tf3image, sf3image, f3file, savedf3
int	txwindow, tywindow
string	tboundary, tkernel
real	tconstant, tzerodiv, t_low, t_high, tillumratio
string	tsmoothing, ttitle
int	num_images
int	naxis1, naxis2, sampbeg, linebeg
int	sav_naxis1, sav_naxis2, sav_sampbeg, sav_linebeg
string	section
bool	tverb, first, zoomin, zoomf3, sav_zoom
real	cd11in, cd22in, ratin, cd11f3, cd22f3, ratf3

	# we must have the images package
	if ( !defpac("images") )
	    images >dev$null

	# cache imgets so that it is usable in background
	cache imgets

	num_images = 0
	first 	   = yes
	
	# make a file with all the file names separately
	itfile 	 = mktemp( (intempfile))
	otfile 	 = mktemp( (outtempfile))
	f3file	 = mktemp( (f3temp) )
	intermed = mktemp( (interfile) )
	tmpoutput= mktemp( "tmp$tout" )

	files( (input),   >>(itfile) )
	files( (f3image), >>(f3file) )
	toutput 	= output
	tillumratio	= illumratio

	# transfer the user parameters to temporary variables
	tsmoothing	= smoothing
	txwindow	= xwindow
	tywindow  	= ywindow
	if ( tsmoothing == "convolve" )
	    tkernel = kernel
	ttitle		= title
	t_low		= low_reject
	t_high		= high_reject
	tboundary 	= boundary
	tconstant 	= constant
	tzerodiv	= zerodiv
	tverb 		= verbose

	# set imdivide parameters
	imdivide.constant 	= tzerodiv
	imdivide.title 		= "*"
	imdivide.rescale 	= "norescale"
	imdivide.mean 		= "1"
	imdivide.verbose 	= tverb

	# set the listfiles as the file with separated names
	inimglist 	= (itfile)
	f3imglist	= (f3file)

	# loop over all the files
	while ( fscan( inimglist,   inimg) != EOF ) {

	    if ( fscan( f3imglist, tf3image) == EOF) {
		f3imglist = (f3file)
		if ( fscan ( f3imglist, tf3image)  == EOF )
		    error (0, "f3 image list wrong")
	    }

	    # create temporary intermediate file and add name to list file
	    outimg = mktemp("tmp$fff")

	    # check we are not mixing square and rectangular pixels 
	    imgets ( inimg, "cd1_1")      ; cd11in  = int( imgets.value )
	    imgets ( inimg, "cd2_2")      ; cd22in  = int( imgets.value )
	    imgets ( tf3image, "cd1_1")   ; cd11f3  = int( imgets.value )
	    imgets ( tf3image, "cd2_2")   ; cd22f3  = int( imgets.value )
	    if ( cd11in == 2*cd22in )
		zoomin = yes
	    else
		zoomin = no

	    if ( cd11f3 == 2*cd22f3 )
		zoomf3 = yes
	    else
		zoomf3 = no

	    if ( ( zoomin && !zoomf3 ) || ( !zoomin && zoomf3) ) {
		    error ( 0, "canot combine square and rectangular pixels")
	    }

	    # set the image section for f3image:
	    imgets ( inimg, "naxis1")  ; naxis1  = int(imgets.value)
	    imgets ( inimg, "naxis2")  ; naxis2  = int(imgets.value)
	    imgets ( inimg, "sampoff") ; sampbeg = real(imgets.value) + 1
	    imgets ( inimg, "lineoff") ; linebeg = real(imgets.value) + 1

	    # note that SAMPBEG is given in square pixels so we may need 
	    # to correct
	    if ( zoomin )
		sampbeg = ( sampbeg + 1 ) / 2

	    if ( first ) {
		sav_naxis1  = naxis1
		sav_naxis2  = naxis2
		sav_sampbeg = sampbeg
		sav_linebeg = linebeg
		sav_zoom    = zoomin
		first	    = no
	    } else {
		if ( naxis1  != sav_naxis1   ||
		     naxis2  != sav_naxis2   ||
		     sampbeg != sav_sampbeg  ||
		     linebeg != sav_linebeg  ||
		     ( zoomin && !sav_zoom ) ||   
		     ( !zoomin && sav_zoom )     )
		    error ( 0, "input images must have same size, offset")
	    }

	    section = "["//str( sampbeg )//":"\
		//str( sampbeg + naxis1 - 1)//","\
		//str( linebeg )//":"//str( linebeg + naxis2 - 1)//"]"
	    sf3image = tf3image//section

	    # now divide the image into the original
	    imdivide ( inimg, sf3image, intermed )


	    # perform the operation requested to smooth image
	    if ( tsmoothing == "boxcar") {
		boxcar.boundary = (tboundary)
		boxcar.constant = (tconstant)
	        boxcar( intermed, outimg, txwindow, tywindow)
	    } else if (tsmoothing == "convolve") {
		convolve.boundary = tboundary
		convolve.constant = tconstant
	        convolve( intermed, outimg, tkernel )
	    } else if ( tsmoothing == "median" ) {		
		median.boundary = tboundary
		median.constant = tconstant
	        median( intermed, outimg, txwindow, tywindow )
	    } else if ( tsmoothing == "mode" ) {		
		mode.boundary = tboundary
		mode.constant = tconstant
	        mode( intermed, outimg, txwindow, tywindow  )
	    } else if ( tsmoothing == "none" ) {
		imrename ( intermed, outimg, verbose=tverb)
	    }

	    # delete the intermediate image
	    imdelete ( intermed, yes, v-)

	    # add this file name to the list file	    
	    print ( outimg, >>(otfile) )
	    num_images = num_images + 1

	}

	if ( num_images > 1) {
	    # now make the average of the individual fix pattern images
	    totfile = "@"//(otfile)
	    imsum ( totfile, tmpoutput, title=ttitle, hparams="", \
		pixtype="real", calctype="real", option="average", \
		low_reject=t_low, high_reject=t_high, verbose=tverb )
	    imdelete (totfile, yes, v-)
	    imarith ( tmpoutput, "*", tillumratio, toutput, verbose=tverb, \
			title="*", divzero=0.0, hparams="", pixtype="real",\
			calctype="real", noact=no)
	    imdelete ( tmpoutput, v-)
	} else if ( num_images == 1) {   # but if just one - then copy
	    imarith ( outimg, "*", tillumratio, toutput, verbose=tverb, \
			title="*", divzero=0.0, hparams="", pixtype="real",\
			calctype="real", noact=no)
	    imdelete ( outimg, yes, v-)
	} else { 			 # we must have at least one image
	    error ( 1, "no images available")
	}

	delete ( (itfile) )
	delete ( (otfile) )

end
