# FTPLOT -- Plot real and imaginary parts of spatial and transform domain
# data.  Normally, four plots are displayed, the "input" domain on the left and
# the "output" domain on the right.  The real parts is displayed at the top.
# If one of the data files for either domain is absent, the procedure creates
# a "zero" file of appropriate length and plots it.  One or the other domains
# may be missing entirely, in which case the existing domain data are plotted
# with the real part on the left side of the screen.
#
# C. D. Biemesderfer, STScI, 10 Dec 87

procedure ftplot (left, right)

file	left = "" {prompt="> Left side data"}
file	right = "" {prompt="> Right side data"}

struct*	naxfil

string	mode="h"
#------------------------------------------------------------------------------
begin

string	ld, rd		# left data, right data
string	rl, il, rr, ir	# real  left, imag  left, real  right, imag  right
string	ul, ll, ur, lr	# upper left, lower left, upper right, lower right
string	ityp, naxstr, mosin, nfnam
int	naxfoo, nax
bool	lpresent=yes
bool	rpresent=yes

if ( ! (defpac ("plot") && defpac ("images")) ) {
    print ("The plot and images packages must be loaded.")
    bye()
}

# Buffer input names so user is only prompted once for each
ld = left
rd = right

# Assemble actual data file names according to convention
rl = ld // "r"
il = ld // "i"
rr = rd // "r"
ir = rd // "i"

# Make up temp names for throwaway vdm files
ul = mktemp("ftpl") // ".vdm"
ll = mktemp("ftpl") // ".vdm"
ur = mktemp("ftpl") // ".vdm"
lr = mktemp("ftpl") // ".vdm"

# Get default image type
ityp = envget("imtype")
#ityp = "imh"
nfnam = "uparm$nax"

# Check for data file existence, graph file and trap vdm.  If a particular file
# doesn't exist, make a file with the correct number of pixels that contains
# zeroes and graph it instead.

if (access(rl//"."//ityp))	# real input data (upper left)
    graph (rl, >G (ul))
else {
    if (!access(il//"."//ityp))
	lpresent=no	# error (0, "No original domain data")
    else {
	naxfil = nfnam
	images.hselect (il, "naxis1", yes, >nfnam)
	naxfoo = fscan (naxfil, naxstr)
	nax = int(naxstr)
	testfunc ("constant", const=0.0, npoints=nax, | graph >G (ul))
    }
}

if (access(il//"."//ityp))	# imaginary input data (lower left)
    graph (il, >G (ll))
else {
    if (!access(rl//"."//ityp))
	lpresent=no	# error (0, "No original domain data")
    else {
	naxfil = nfnam
	images.hselect (rl, "naxis1", yes, >nfnam)
	naxfoo = fscan (naxfil, naxstr)
	nax = int(naxstr)
	testfunc ("constant", const=0.0, npoints=nax, | graph >G (ll))
    }
}

if (access(rr//"."//ityp))	# real output data (upper right)
    graph (rr, >G (ur))
else {
    if (!access(ir//"."//ityp)) {
	rpresent=no	# error (0, "No transform domain data")
    } else {
	naxfil = nfnam
	images.hselect (ir, "naxis1", yes, >nfnam)
	naxfoo = fscan (naxfil, naxstr)
	nax = int(naxstr)
	testfunc ("constant", const=0.0, npoints=nax, | graph >G (ur))
    }
}

if (access(ir//"."//ityp))	# imaginary output data (lower right)
    graph (ir, >G (lr))
else {
    if (!access(rr//"."//ityp)) {
	rpresent=no	# error (0, "No transform domain data")
    } else {
	naxfil = nfnam
	images.hselect (rr, "naxis1", yes, >nfnam)
	naxfoo = fscan (naxfil, naxstr)
	nax = int(naxstr)
	testfunc ("constant", const=0.0, npoints=nax, | graph >G (lr))
    }
}

# Mosaic metacode
if (lpresent && rpresent) {
    mosin = ul // "," // ll // "," // ur // "," // lr
    gkimosaic (mosin, nx=2, ny=2)
} else if (lpresent) {
    mosin = ul // "," // ll
    gkimosaic (mosin, nx=2, ny=1)
} else if (rpresent) {
    mosin = ur // "," // lr
    gkimosaic (mosin, nx=2, ny=1)
} else
    error (0, "No data found")

# Discard vdm files
delete (mosin)

if (access(nfnam))
    delete (nfnam)

end
