        SUBROUTINE VASNPT (
*
*  inputs
*
     :                  OFILE, TARGET, APERT, COUNT, DCOUNT, FLUX, ASEN,
     :                  TEMP, EPOCH, NPTS, TNAME,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.9.1.3
*
*  Module Name: abssenv
*
*  Keyphrase:
*  ----------
*  Write the result of VASEN to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VASNPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'TRGTNAME'                    O      target name
*  'APER_NAME'                   O      aperture name
*  'COUNT'                       O      observed count (rate)
*  'COUNT_ERR'                   O      error of COUNT
*  'REF_FLUX'                    O      reference flux density
*  'SENSITIVITY'                 O      absolute sensitivity
*  (temp_key)                    O      temperature as specified in TNAME
*  'EPOCH'                       O      average epoch
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       08-30-86     J.-C. HSU        design and coding
*     2       10-05-87     J.-C. HSU        F77 standard
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --file name of output table
        CHARACTER*(*)           OFILE,
*                                       --target name
     :                          TARGET(1),
*                                       --aperture name
     :                          APERT(1),
*                                       --temperature column name
     :                          TNAME
*                                       --absolute sensitivity
        REAL                    ASEN(1),
*                                       --count (rate) and its standard deviation
     :                          COUNT(1), DCOUNT(1),
*                                       --reference flux
     :                          FLUX(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --number of input data points
        INTEGER                 NPTS
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, J, NPUT,
*                                       --number of existed rows of output
*                                       --table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(20),
*                                       --data type
     :                          DTYPE(20),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    CBUFF(20)
*                                       --column format
        CHARACTER*8             COLFMT(20)
*                                       --output SDAS table names
        CHARACTER*16            CNAME(20),
*                                       --column units
     :                          UNIT(20)
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
*                                       --message destination and priority
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify column names
*
        NCOLS = 8
*
        CNAME(1) = 'TRGTNAME'
        CNAME(2) = 'APER_NAME'
        CNAME(3) = 'COUNT'
        CNAME(4) = 'COUNT_ERR'
        CNAME(5) = 'REF_FLUX'
        CNAME(6) = 'SENSITIVITY'
        CNAME(7) = TNAME
        CNAME(8) = 'EPOCH'
*
*  data types of output columns
*
        DTYPE(1) = -20
        DTYPE(2) = -10
        DO 10 I = 3, 7
            DTYPE(I) = TYREAL
   10   CONTINUE
        DTYPE(8) = TYDOUB
*
*  specify units and display formats of output columns
*
        DO 20 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   20   CONTINUE
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up output table'
            GO TO 999
        END IF
*
*  append each observation to output table as one new row
*
        DO 40 I = 1, NPTS
*
*  specify buffer contents
*
            CBUFF(1) = COUNT(I)
            CBUFF(2) = DCOUNT(I)
            CBUFF(3) = FLUX(I)
            CBUFF(4) = ASEN(I)
            CBUFF(5) = TEMP(I)
*
*  write each column
*
            CALL UTRPTT (TP, COLIDN(1), 1, NROWS+I, TARGET(I),  STAT(1))
            CALL UTRPTT (TP, COLIDN(2), 1, NROWS+I, APERT(I),   STAT(2))
            CALL UTRPTR (TP, COLIDN(3), 5, NROWS+I, CBUFF,   STAT(3))
            CALL UTRPTD (TP, COLIDN(8), 1, NROWS+I, EPOCH(I),   STAT(4))
            NPUT = 4
*
            DO 30 J = 1, NPUT
                IF (STAT(J) .NE. OK) THEN
                    STATUS = ERRNUM(1)
                    CONTXT = 'cannot write a row to output table'
                    GO TO 999
                END IF
   30       CONTINUE
   40   CONTINUE
*
*  close output table
*
        CALL UTTCLO (TP, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VASNPT: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
