        SUBROUTINE VAP2PT (
*
*  inputs
*
     :                  OFILE, A, B, THETA, V2TRGT, V3TRGT,
     :                  NPTS, NPEAK, LEVEL,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.11.2.2.2
*
*  Module Name: imgscale
*
*  Keyphrase:
*  ----------
*  Write the result of phase II aperture location to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VAP2PT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'X_SCALE'                     O      plate scale of X axis relative to V2
*  'Y_SCALE'                     O      plate scale of Y axis relative to V3
*  'ROTATE_ANGLE'                O      rotation angle of X-Y
*  'TARGET_V2'                   O      V2 coordinate of the target
*  'TARGET_V3'                   O      V3 coordinate of the target
*  'LOW_LIMIT'                   O      fraction of the highest count, points
*                                       below this count level is rejected
*  'NPOINTS'                     O      number of points used to determine
*                                       the plate scales
*  'NPEAK'                       O      number of highest points selected in
*                                       each input file
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       02-20-87     J.-C. HSU        design and coding
*     1       11-10-87     J.-C. HSU        F77 SDAS
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --output table name
        CHARACTER*(*)           OFILE
*                                       --number of points used to determine
*                                       --the plate scales
        INTEGER                 NPTS,
*                                       --number of highest points selected in
*                                       --each input file
     :                          NPEAK
*                                       --transformation coefficients
        REAL                    A, B, THETA, V2TRGT, V3TRGT,
*                                       --fraction of the highest count, points
*                                       --below this count level is rejected
     :                          LEVEL
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --output table column content
        REAL                    CBUFF(20)
*                                       --loop index
        INTEGER                 I,
*                                       --pointer to table descripter
     :                          TP,
*                                       --number of existed rows of output table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS,
*                                       --column data type
     :                          DTYPE(20),
*                                       --column identifiers
     :                          COLIDN(20),
*                                       --error status
     :                          STAT(10), STATOK
*                                       --column format
        CHARACTER*8             COLFMT(20)
*                                       --output table column names
        CHARACTER*16            CNAME(20),
*                                       --column units
     :                          UNIT(20)
*                                       --error message context
        CHARACTER*130           CONTXT
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify output buffer contents
*
        CBUFF(1) = A
        CBUFF(2) = B
        CBUFF(3) = THETA
        CBUFF(4) = V2TRGT
        CBUFF(5) = V3TRGT
        CBUFF(6) = LEVEL
*
*  specify column names
*
        CNAME(1) = 'X_SCALE'
        CNAME(2) = 'Y_SCALE'
        CNAME(3) = 'ROTATE_ANGLE'
        CNAME(4) = 'TARGET_V2'
        CNAME(5) = 'TARGET_V3'
        CNAME(6) = 'LOW_LIMIT'
*
        CNAME(7) = 'NPOINTS'
        CNAME(8) = 'NPEAK'
*
*  define column units, formats, and data types
*
        NCOLS = 8
*
        DO 10 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
            DTYPE(I) = TYREAL
   10   CONTINUE
        DTYPE(NCOLS-1) = TYINT
        DTYPE(NCOLS) = TYINT
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. 0) THEN
            CONTXT = 'cannot set up output table'
            GO TO 999
        END IF
*
*  write result to output table
*
        CALL UTRPTR (TP, COLIDN, NCOLS-2, NROWS+1, CBUFF,     STAT(1))
        CALL UTRPTI (TP, COLIDN(NCOLS-1), 1, NROWS+1, NPTS,   STAT(2))
        CALL UTRPTI (TP, COLIDN(NCOLS), 1, NROWS+1, NPEAK,    STAT(3))
*
        DO 60 I = 1, 3
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(2)
                CONTXT = 'cannot write a row to output table'
                GO TO 999
            END IF
   60   CONTINUE
*
        CALL UTTCLO (TP,   STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   CALL UMSPUT ('VAP2PT: ' // CONTXT, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
