        SUBROUTINE VAP3L
*
*  Module number: 15.11.2.3.2
*
*  Module name: lgaperloc
*
*  Keyphrase:
*  ----------
*  Aperture location calibration Phase III of a large aperture.
*
*  Description:
*  ------------
*
*  FORTRAN name: VAP3L.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       VAP3LG, VEDG3L, VAP3LP
*  SDAS:
*       UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       01-20-87     J.-C. HSU        design and coding
*     2       11-10-87     J.-C. HSU        F77 SDAS
*
*-------------------------------------------------------------------------------
*
*== local:
*                                       --total number of data points of the
*                                       --input time series
        INTEGER                 NPTS
*                                       --input arrays of count rate, its standard
*                                       --deviation, V2 and V3 coordinates
        REAL                    COUNT(2500), DCOUNT(2500),
     :                          V2(2500), V3(2500),
*                                       --coordinates of the aperture center
*                                       --and their standard errors
     :                          V2CEN, V3CEN, DV2CEN, DV3CEN,
*                                       --ration between count levels of the
*                                       --edge and the maximum
     :                          LEVEL
*                                       --epoch of observation
        DOUBLE PRECISION        EPOCH(2500),
*                                       --epoch range
     :                          EPOMIN, EPOMAX
*                                       --error status
        INTEGER                 STATUS
*                                       --output file name
        CHARACTER*128           OFILE
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  input parameters and data
*
        CALL VAP3LG (
     :                  V2, V3, COUNT, DCOUNT, EPOCH, NPTS, LEVEL,
     :                  OFILE, STATUS)
*
        IF (STATUS .NE. OK) THEN
            GO TO 1000
        END IF
*
*  find out the coordinate of aperture center
*
        CALL VEDG3L (NPTS, V2, V3, COUNT, DCOUNT, EPOCH, LEVEL,
     :                  V2CEN, V3CEN, DV2CEN, DV3CEN, EPOMIN, EPOMAX,
     :                  STATUS)
*
        IF (STATUS .NE. OK) THEN
            GO TO 1000
        END IF
*
*  write coordinates of aperutre center and their standard errors to a table
*
        CALL VAP3LP (OFILE, V2CEN, V3CEN, DV2CEN, DV3CEN,       STATUS)
*
        IF (STATUS .NE. OK) THEN
            GO TO 1000
        END IF
*
        STATUS = OK
        GO TO 1000
*
 1000   RETURN
        END
