        SUBROUTINE CDPCOR (
*
*  inputs
*
     :                  XYZ,
*
*  outputs
*
     :                  R, PHI, THETA)
*
*  Module number:
*
*  Module name:
*
*  Keyphrase:
*  ----------
*  convert rectangular coordinates to polar coordinates
*
*  Description:
*  ------------
*  convert rectangular coordinates, X, Y, Z, to polar coordinates, R (radius),
*  PHI (azimuth angle, i. e. angle on the XY plane, from X axis), and THETA
*  (polar angle, from XY plane).  All angles are in decimal degrees.  THETA
*  is from -90 to +90 and PHI from 0 to 360.
*
*  FORTRAN name: CDPCOR.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       None
*  SDAS:
*       None
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       02-28-88     J.-C. HSU        coding
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --rectangular coordinates
        DOUBLE PRECISION        XYZ(3)
*
*== output:
*                                       --polar coordinates
        DOUBLE PRECISION        R, PHI, THETA
*
*== local:
*
        DOUBLE PRECISION        W, X, Y, Z, RXY,
*                                       --degrees per radian
     :                          RADN
*
*------------------------------------------------------------------------------
*
*  degrees of one radian
*
        RADN = 45.D0 / ATAN(1.D0)
*
        X = ABS(XYZ(1))
        Y = ABS(XYZ(2))
        Z = ABS(XYZ(3))
        W = X + Y + Z
*
*  if radius is zero
*
        IF (W .EQ. 0.D0) THEN
            R = 0.D0
            PHI = 0.D0
            THETA = 0.D0
            RETURN
        END IF
*
*  calculate radius, the algorithm here is to avoid over/under flow
*
        R = W * SQRT (1.D0 - 2.D0 * ((X/W) * (Y/W) + (Y/W) * (Z/W) +
     :                               (Z/W) * (X/W)))
*
        PHI = ATAN2 (XYZ(2)/R, XYZ(1)/R) * RADN
*
*  make the range of PHI from 0 to 360
*
        IF (PHI .LT. 0.D0) PHI = PHI + 360.D0
*
*  use ATAN2 instead of ASIN to calculate theta for better accuracy
*
        W = X + Y
*
        IF (W .EQ. 0.D0) THEN
            RXY = 0.D0
        ELSE
            RXY = W * SQRT (1.D0 - 2.D0 * ((X/W) * (Y/W)))
        END IF
*
        THETA = ATAN2 (XYZ(3)/R, RXY/R) * RADN
*
        RETURN
        END
