        SUBROUTINE VSMSJD (
*
*  inputs
*
     :                    YEAR, DAY, HOUR, MIN, SEC,
*
*  output
*
     :                    MJD)
*
*  Module Number:
*
*  Module Name:
*
*  Keyphrase:
*  ----------
*  Convert SMS-format time to Julian Date
*
*  Description:
*  ------------
*  Convert SMS-format, i.e. year and number of days since Jan 0, hour, minute,
*  and second to Julian Day Number.
*
*  uses algorithm in H. F. Fliegel & T. C. Van Flandern (1968), Comm. A.C.M.,
*  vol 11, p 657;
*  and Van Flandern & Pulkkinen (1979 Nov), Ap.J. Suppl., vol 41, 391-411.
*
*  FORTRAN Name: VSMSJD.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDGSRD, CDGCFD
*  SDAS:
*       UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       02-20-89      J.-C. Hsu       Coding, adapted from VCALJD.FOR
*
*-------------------------------------------------------------------------------
*
*  Use the following integer arithmetic expression
*
*  JD = (regular month)Day - 0.5 + 367 * Year - 7 * [Year + (Month + 9)/12]/4
*        - 3 * {[Year + (Month - 9)/7]/100 + 1}/4 + 275 * Month/9 + 1721029
*
*------------------------------------------------------------------------------
*
*== inputs
*                                       --Input year and month
        INTEGER                 YEAR,
*                                       --Input day
     :                          DAY,
*                                       --input hour, minute
     :                          HOUR, MIN
*                                       --input second
        REAL                    SEC
*
*== outputs
*                                       --Output Modified Julian day number
        DOUBLE PRECISION        MJD
*
*== local
*                                       --local integer expresion
        INTEGER                 INTEXP
*
* ------------------------------------------------------------------------
*
        INTEXP = 367 * YEAR - 7 * YEAR / 4
     :         -   3 * ((YEAR - 1) / 100 + 1) / 4 + 1721059 - 2400001
*
        MJD = (DBLE (HOUR) + (DBLE(MIN) + (DBLE(SEC) / 60.D0)) / 60.D0)
     :          / 24.D0
        MJD = DBLE(INTEXP + DAY) + MJD
*
        RETURN
        END
