        SUBROUTINE VPVFBN (
*
*  inputs
*
     :                          CNT, CNTSD, APPA, TEMP, EPOCH, NPTS,
*
*  inputs/outputs
*
     :                          APERT, TARGET,
*
*  outputs
*
     :                          POL, PA, INTEN, P0, PA0, TMPAVE, EPAVE,
     :                          NROWS, STATUS)
*
*  Module Number: 15.12.4.1
*
*  Module Name: polverify
*
*  Keyphrase:
*  ----------
*  Calculate polarization and position angle at each aperture.
*
*  Description:
*  ------------
*  Sort input data into different polarizer apertures, and calculate
*  polarization and position angle of a target at a given aperture.  The
*  calculation includes the corrections of polarization efficiency and
*  position angle offset.
*  Maximum number of standard targets is 250 (i. e. total number of entries in
*  the standard target table can not be more than 1000, since each target has
*  four entries corresponding to each filter).
*
*  FORTRAN Name: VPVFBN.FOR
*
*  Keywords of Accessed Files and Tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDPTRG, CDPEF, CDPPA, CDROLL, CDP3AN
*  SDAS:
*       UMSPUT
*  OTHERS:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       07-15-86     J.-C. Hsu        Design and coding
*     2       10-05-87     J.-C. Hsu        F77 standard
*     3       12-01-89     J.-C. Hsu        Error propagation
*-------------------------------------------------------------------------------
*
*== input:
*                                       --count rate and its standard deviation
        REAL                    CNT(1), CNTSD(1),
*                                       --position angle of aperture used
     :                          APPA(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --number of input data points
        INTEGER                 NPTS
*
*== inputs/outputs
*                                       --aperture name
        CHARACTER*(*)           APERT(1),
*                                       --target name
     :                          TARGET(1)
*
*== output:
*                                       --intensity, polarization and its
*                                       --position angle
        REAL                    INTEN(1), POL(1), PA(1),
*                                       --standard polarization and position
*                                       --angle for a specific aperture
     :                          P0(1), PA0(1),
*                                       --average temperature
     :                          TMPAVE(1)
*                                       --average epoch
        DOUBLE PRECISION        EPAVE(1)
*                                       --number of rows to be put in output table
        INTEGER                 NROWS,
*                                       --error status
     :                          STATUS
*
*== local:
*                                       --loop indices
        INTEGER                 I, J, K, M, N,
*                                       --number of entries in standard
*                                       --polarization target table, polarization
*                                       --efficiency table, and position angle
*                                       --table
     :                          NPOL, NPEFF, NPA,
*                                       --number of different apertures
     :                          NAPER,
*                                       --number of roll angles
     :                          NANGLE(20),
*                                       --number of observation at a specific
*                                       --aperture
     :                          OBAPER(20),
*                                       --number of observation at a
*                                       --specific roll angle of one aperture
     :                          OBANG(20,3),
*                                       --aperture index
     :                          INDEX1,
*                                       --roll angle index
     :                          INDEX2,
*                                       --number of observations index
     :                          INDEX3,
     :                          INDEX(20, 3, 2000),
*                                       --roll angle indices
     :                          ROLLID(3)
*                                       --Stokes parameters
        REAL                    Q, U,
*                                       --polarization and position angle read
*                                       --from standard target table
     :                          REFP(1000), REFPA(1000),
*                                       --errors of the above
     :                          DREFP(1000), DREFPA(1000),
*                                       --polarization efficiency
     :                          PEFF(20), DPEFF(20), EFF,
*                                       --position angle offset
     :                          PAOFST(4), OFFSET,
*                                       --position angle of the transmission axis
     :                          TRANS,
*                                       --sum of count rate, weights
     :                          SUM(3), SUMWT(3),
*                                       --weight
     :                          WT,
*                                       --roll angles
     :                          ROLL(3), APPAX(20, 3), THETA1, THETA2
        INTEGER                 STAT(10), STATOK
*                                       --filter name
        CHARACTER*4             FILTER, FILTPA(4),
*                                       --filter name of the standard stars
     :                          REFFLT(1000)
        CHARACTER*5             CHAR5
*                                       --aperture name
        CHARACTER*10            APERX(20), APEREF(20)
*                                       --target name
        CHARACTER*20            OBJ(20)
*                                       --reference target name
        CHARACTER*20            REFOBJ(5,1000)
*                                       --error message
        CHARACTER*130           CONTXT
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*-------------------------------------------------------------------------------
*
*  initialize
*
        NROWS = 0
        DO 20 J = 1, 20
            NANGLE(J) = 0
            OBAPER(J) = 0
            TMPAVE(J) = 0.
            EPAVE(J) = 0.D0
            DO 10 K = 1, 3
                OBANG(J, K) = 0
   10       CONTINUE
   20   CONTINUE
*
*  sort input data points according to their aperture:
*  index1 (and j) is the aperture index, index2 (and k) is the roll angle index
*
        DO 70 I = 1, NPTS
*
*  initialize for the first data point
*
            IF (I .EQ. 1) THEN
                NAPER = 1
                APERX(NAPER) = APERT(I)
            END IF
*
*  match with known apertures
*
            DO 30 J = 1, NAPER
                IF (APERT(I) .EQ. APERX(J)) THEN
                    INDEX1 = J
                    GO TO 40
                END IF
   30       CONTINUE
*
*  if no matching aperture, increment the aperture number counter
*
            NAPER = NAPER + 1
            APERX(NAPER) = APERT(I)
            INDEX1 = NAPER
*
*  increment number of observations for the aperture
*
   40       OBAPER(INDEX1) = OBAPER(INDEX1) + 1
*
*  assign the target name as the reference object name for the new aperture
*
            IF (OBAPER(INDEX1) .EQ. 1) THEN
                OBJ(INDEX1) = TARGET(I)
            END IF
*
*  check if the target name is the same for the same aperture, if not,
*  skip the data point and write message
*
            IF (TARGET(I) .NE. OBJ(INDEX1)) THEN
                WRITE (CHAR5, '(I5)') I
                CONTXT = 'different targets for the same aperture at '
     :                          // 'entry #' // CHAR5
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
                OBAPER(J) = OBAPER(J) - 1
            ELSE
*
*  sort input data points according to their roll angle
*
*  initialize for the first entry of the specified roll angle
*
                IF (NANGLE(INDEX1) .EQ. 0) THEN
                    NANGLE(INDEX1) = 1
                    APPAX(INDEX1, NANGLE(INDEX1)) = APPA(I)
                END IF
*
*  match with know roll angles
*
                DO 50 K = 1, NANGLE(INDEX1)
                    IF (APPA(I) .EQ. APPAX(INDEX1, K)) THEN
                        INDEX2 = K
                        GO TO 60
                    END IF
   50           CONTINUE
*
*  if no matching roll angle, increment the roll angle number counter
*
                NANGLE(INDEX1) = NANGLE(INDEX1) + 1
                APPAX(INDEX1, NANGLE(INDEX1)) = APPA(I)
                INDEX2 = NANGLE(INDEX1)
*
*  check if within maximum number (=3) of roll angles:
*
   60           IF (INDEX2 .GT. 3) THEN
                    CONTXT = APERT(I) // 'has more than 3 roll angles'
                    CALL UMSPUT (CONTXT, DEST, PRIO,    STATOK)
                ELSE
                    OBANG(INDEX1, INDEX2) = OBANG(INDEX1, INDEX2) + 1
                    INDEX3 = OBANG(INDEX1, INDEX2)
                    INDEX(INDEX1, INDEX2, INDEX3) = I
                END IF
            END IF
   70   CONTINUE
*
*  obtain reference data from standard polarization target table
*
        CALL CDPTRG (    REFOBJ, REFFLT, REFP, DREFP, REFPA, DREFPA, 
     :                NPOL, STATUS)
        IF (STATUS .NE. OK   .OR.   NPOL .GT. 1000) THEN
            CONTXT = 'error reading standard polarization target table '
     :                  // 'or too many standard targets (>1000)'
            GO TO 999
        END IF
*
*  read polarization efficiency data
*
        CALL CDPEF(    APEREF, PEFF, DPEFF, NPEFF, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot read polarization efficiency table'
            GO TO 999
        END IF
*
*  read position angle offset data
*
        CALL CDPPA (    FILTPA, PAOFST, NPA, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot read position angle offset table'
            GO TO 999
        END IF
*
*  process each aperture
*
        DO 300 J = 1, NAPER
*
*  identify filter name (assume filter name has ten characters) corresponding
*  to the aperture
*
            DO 80 I = 1, 8
                IF (APERX(J)(I:I) .EQ. 'F') GO TO 90
   80       CONTINUE
*
            STATUS = ERRNUM(1)
            CONTXT = 'illegal aperture name ' // APERX(J)
            GO TO 300
*
   90       FILTER = APERX(J)(I:I+3)
*
*  look for polarizer transmission axis angle
*
            DO 100 I = 1, 10
                IF (APERX(J)(I:I) .EQ. 'P') THEN
                    GO TO 110
                END IF
  100       CONTINUE
*
  110       IF (APERX(J)(I+1:I+1) .EQ. '0') THEN
                TRANS = 0.
            ELSE IF (APERX(J)(I+1:I+2) .EQ. '45') THEN
                TRANS = 45.
            ELSE IF (APERX(J)(I+1:I+2) .EQ. '90') THEN
                TRANS = 90.
            ELSE IF (APERX(J)(I+1:I+3) .EQ. '135') THEN
                TRANS = 135.
            ELSE
                CONTXT = 'illegal aperture name ' // APERX(J)
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
                GO TO 300
            END IF
*
*  match with standard polarization target data
*
            DO 130 I = 1, NPOL
                IF (FILTER .EQ. REFFLT(I)) THEN
                    DO 120 N = 1, 5
                        IF (OBJ(J) .EQ. REFOBJ(N, I)) THEN
                            P0(J) = REFP(I)
                            PA0(J) = REFPA(I)
                            GO TO 140
                        END IF
  120               CONTINUE
                END IF
  130       CONTINUE
*
*  if no matching standard star data, skip, and write warning message
*
            CONTXT = 'no reference polarization for ' // OBJ(J) //
     :                  ' at aperture ' // APERX(J)
            CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
            P0(J) = -1.E-10
            PA0(J) = -1.E-10
*
*  if data only obtained in less than 3 roll angles, skip, and write
*  nasty message
*
  140       IF (NANGLE(J) .LT. 3) THEN
                CONTXT = APERX(J) // 'has less than three roll angles'
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
                GO TO 300
            END IF
*
*  match with position angle offset calibration table
*
            DO 150 I = 1, NPA
                IF (FILTER .EQ. FILTPA(I)) THEN
                    OFFSET = PAOFST(I)
                    GO TO 160
                END IF
  150       CONTINUE
*
*  if no matching position angle offset, skip, and write warning message
*
            CONTXT = 'no position angle offset for ' // FILTER
            CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
            GO TO 300
  160       CONTINUE
*
*  match with polarization efficiency
*
            DO 170 I = 1, NPEFF
                IF (APERX(J) .EQ. APEREF(I)) THEN
                    EFF = PEFF(I)
                    GO TO 180
                END IF
  170       CONTINUE
*
*  if no matching polarization efficiency, skip, and write warning message
*
            CONTXT = 'no polarization efficiency for ' // APERX(J)
            CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
            GO TO 300
  180       CONTINUE
*
*  process each roll angle, if there are more than 3 roll angles, only
*  the first 3 will be processed
*
            DO 200 K = 1, 3
                SUM(K) = 0
                SUMWT(K) = 0
                DO 190 M = 1, OBANG(J, K)
                    WT = 1. / CNTSD(INDEX(J, K, M))**2
                    SUM(K) = CNT(INDEX(J, K, M)) * WT + SUM(K)
                    SUMWT(K) = WT + SUMWT(K)
                    TMPAVE(J) = TMPAVE(J) + TEMP(INDEX(J, K, M))
                    EPAVE(J) = EPAVE(J) + EPOCH(INDEX(J, K, M))
  190           CONTINUE
*
*  if no observation in the roll angle bin, put error message
*
                SUM(K) = SUM(K) / SUMWT(K)
                ROLL(K) = APPAX(J, K)
  200       CONTINUE
*
*  determine separations among the three roll angles
*
            CALL CDROLL (ROLL,
     :                   ROLLID, THETA1, THETA2, STAT(1))
            IF (STAT(1) .NE. OK) THEN
                CONTXT = APERX(J) // 'has inadequate roll angles'
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
                GO TO 300
            END IF
*
*  calculate Stokes parameters from measurements made at the three roll angles
*
            CALL CDP3AN (SUM(ROLLID(1)), SUM(ROLLID(2)), SUM(ROLLID(3)),
     :                   THETA1, THETA2,
     :                   INTEN(J), Q, U, STAT(2))
            IF (STAT(2) .NE. OK) THEN
                CONTXT = 'error calculating Stokes parameters for '
     :                            // 'aperture ' // APERX(J)
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
                GO TO 300
            END IF
*
*  mean temperature and epoch
*
            NROWS = NROWS + 1
            TMPAVE(NROWS) = TMPAVE(J) / REAL(OBAPER(J))
            EPAVE(NROWS) = EPAVE(J) / DBLE(OBAPER(J))
*
            INTEN(NROWS) = INTEN(J)
            P0(NROWS) = P0(J)
            PA0(NROWS) = PA0(J)
            APERT(NROWS) = APERX(J)
            TARGET(NROWS) = OBJ(J)
*
*  calculate polarization and position angle
*
            POL(NROWS) = SQRT (Q**2 + U**2) / EFF
            PA(NROWS) = 0.5 * ATAN2(U, Q) * 57.29577951
*
*  reference polarization is in PER CENTS
*
            POL(NROWS) = POL(NROWS) * 100.0
*
*  position angle = observed angle + position angle offset + aperture position 
*  angle of zero roll + position angle of the transmission axis
*
            PA(NROWS) = PA(NROWS) + OFFSET + ROLL(ROLLID(1)) + TRANS
*
*  restrict position angle to be between 0 and +180. degrees
*
  210       IF (PA(NROWS) .GT. 180.) THEN
                PA(NROWS) = PA(NROWS) - 180.
                GO TO 210
            END IF
*
  220       IF (PA(NROWS) .LT. 0.) THEN
                PA(NROWS) = PA(NROWS) + 180.
                GO TO 220
            END IF
*
  300   CONTINUE
*
        STATUS = OK
        GO TO 1000
*
  999   CALL UMSPUT ('VPVFBN: ' // CONTXT, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
