        SUBROUTINE VEPPT (
*
*  inputs
*
     :                  OFILE, ORDER, NPTS, WFLAG, TYPE,
     :                  COEFF, CHISQ, BASEKY, TEMP0, TIME0,
     :                  TTMIN, TTMAX, EMIN, EMAX,
     :                  DETID, APERT, VOLT, GAIN, THRESH,
     :                  NATTRB, ATTRB,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.1.0.2.2
*
*  Module Name: polyepoch
*
*  Keyphrase:
*  ----------
*  Write the result of VEPOCH to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VEPPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*  'OUTTABLE'                    O      SDAS table containing result
*
*       output table column names:
*
*  'A00', 'A01', ... 'A10', 'A11'... 'A33'
*                                O      polynomial coefficients
*  'CHISQi'                      O      chi squared of the fit
*  'WEIGHT'                      O      weight scheme
*  'BASE_TEMP'                   O      base temperature
*  'TEMPMIN'                     O      lower limit of temperature
*  'TEMPMAX'                     O      upper limit of temperature
*  'BASE_TIME'                   O      base epoch
*  'EPOCHMIN'                    O      lower limit of epoch
*  'EPOCHMAX'                    O      upper limit of epoch
*  'ORDER'                       O      order of the fit
*  'NPOINTS'                     O      number of data points
*  'TYPE'                        O      data type
*  (attribi)                     O      attributes copied from the input table
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       10-15-85     J.-C. HSU        design and coding
*     2       07-15-85     J.-C. HSU        F77 standard
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --VMS file name of 'OUTTABLE'
        CHARACTER*(*)           OFILE,
*                                       --data type
     :                          TYPE,
*                                       --base value column name
     :                          BASEKY,
*                                       --attributes
     :                          ATTRB(1)
*                                       --weighting scheme
        REAL                    WFLAG,
*                                       --polynomial coefficients
     :                          COEFF(1),
*                                       --minimum and maximum temperature
     :                          TTMIN, TTMAX,
*                                       --base temperature
     :                          TEMP0
*                                       --minimum, maximum and base epoch
        DOUBLE PRECISION        EMIN, EMAX, TIME0,
*                                       --chi-squared
     :                          CHISQ(1)
*                                       --number of input data points
        INTEGER                 NPTS,
*                                       --order of polynomial fit
     :                          ORDER,
*                                       --detector ID
     :                          DETID,
*                                       --number of attributes
     :                          NATTRB
*                                       --high volatge, gain setting,
*                                       --discriminator threshold setting
        REAL                    VOLT, GAIN, THRESH
*                                       --aperture name
        CHARACTER*10            APERT
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, J, K, L, NPUT,
*                                       --number of existed rows of output
*                                       --SDAS table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS, NC, NCO,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(100),
*                                       --data type
     :                          DTYPE(100),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    CBUFF(100)
        DOUBLE PRECISION        CBUFF2(20)
        CHARACTER*1             CHARJ, CHARK
*                                       --column format
        CHARACTER*8             COLFMT(100)
*                                       --output SDAS table names
        CHARACTER*16            CNAME(100),
*                                       --column units
     :                          UNIT(100)
*                                       --error message context
        CHARACTER*130           CONTXT
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify column names
*
        CNAME(1) = BASEKY
*
        DO 20 J = 1, ORDER+1
            WRITE (CHARJ, '(I1)') J-1
            DO 10 K = 1, ORDER+1
                WRITE (CHARK, '(I1)') K-1
                I = (ORDER+1) * (J-1) + K
                CNAME(I+1) = 'A' // CHARJ // CHARK
   10       CONTINUE
            CNAME((ORDER+1)**2+J+1) = 'CHISQ' // CHARJ
   20   CONTINUE
*
        NC = (ORDER+1) * (ORDER+2) + 1
        CNAME(NC+1) = 'WEIGHT'
        CNAME(NC+2) = 'BASE_TEMP'
        CNAME(NC+3) = 'TEMPMIN'
        CNAME(NC+4) = 'TEMPMAX'
        CNAME(NC+5) = 'BASE_TIME'
        CNAME(NC+6) = 'EPOCHMIN'
        CNAME(NC+7) = 'EPOCHMAX'
*
*  column data type
*
        DO 30 I = 1, NC+4
            DTYPE(I) = TYREAL
   30   CONTINUE
*
        DO 40 I = NC+5, NC+7
            DTYPE(I) = TYDOUB
   40   CONTINUE
*
        CNAME(NC+8) = 'ORDER'
        CNAME(NC+9) = 'NPOINTS'
        CNAME(NC+10) = 'TYPE'
        DTYPE(NC+8) = TYINT
        DTYPE(NC+9) = TYINT
        DTYPE(NC+10) = -7
*
        NCO = NC + 10
*
*  identify attributes
*
        IF (NATTRB .GT. 0) THEN
            DO 50 I = 1, NATTRB
                IF (ATTRB(I) .EQ. 'DETECTOB' .OR. ATTRB(I) .EQ.
     :                  'DET_NUM') THEN
                    CNAME(NCO+I) = 'DET_NUM'
                    DTYPE(NCO+I) = TYINT
                ELSE IF (ATTRB(I) .EQ. 'APERTOBJ' .OR. ATTRB(I) .EQ.
     :                  'APER_NAME') THEN
                    CNAME(NCO+I) = 'APER_NAME'
                    DTYPE(NCO+I) = -10
                ELSE IF (ATTRB(I) .EQ. 'VOLTAGE') THEN
                    CNAME(NCO+I) = 'VOLTAGE'
                    DTYPE(NCO+I) = TYREAL
                ELSE IF (ATTRB(I) .EQ. 'VGAIND') THEN
                    CNAME(NCO+I) = 'VGAIND'
                    DTYPE(NCO+I) = TYREAL
                ELSE IF (ATTRB(I) .EQ. 'THRESH') THEN
                    CNAME(NCO+I) = 'THRESH'
                    DTYPE(NCO+I) = TYREAL
                END IF
   50       CONTINUE
        END IF
*
*  find out the number of rows the SDAS table already has and append
*  the result of this routine as one new row at the bottom
*
        NCOLS = NCO + NATTRB
*
        DO 60 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   60   CONTINUE
*
*  specify buffer contents
*
        CBUFF(1) = COEFF(1)
        DO 80 J = 1, ORDER+1
            DO 70 K = 1, ORDER+1
                L = (J - 1) * (ORDER + 1) + K
                CBUFF(L+1) = COEFF(L) / CBUFF(1)
   70       CONTINUE
            CBUFF((ORDER+1)**2+J+1) = SNGL(CHISQ(J))
   80   CONTINUE
        CBUFF(NC+1) = WFLAG
        CBUFF(NC+2) = TEMP0
        CBUFF(NC+3) = TTMIN
        CBUFF(NC+4) = TTMAX
        CBUFF2(1) = TIME0
        CBUFF2(2) = EMIN
        CBUFF2(3) = EMAX
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up output table'
            GO TO 999
        END IF
*
*  write results to the output table
*
        CALL UTRPTR (TP, COLIDN(1), NC+4, NROWS+1, CBUFF, STAT(1))
        CALL UTRPTD (TP, COLIDN(NC+5), 3, NROWS+1, CBUFF2, STAT(2))
        CALL UTRPTI (TP, COLIDN(NC+8), 1, NROWS+1, ORDER, STAT(3))
        CALL UTRPTI (TP, COLIDN(NC+9), 1, NROWS+1, NPTS, STAT(4))
        CALL UTRPTT (TP, COLIDN(NC+10), 1, NROWS+1, TYPE, STAT(5))
        NPUT = 5
*
*  write attributes
*
        IF (NATTRB .NE. 0) THEN
            DO 90 I = 1, NATTRB
                IF (ATTRB(I) .EQ. 'DETECTOB' .OR. ATTRB(I) .EQ.
     :                  'DET_NUM') THEN
                    CALL UTRPTI (TP, COLIDN(NC+I), 1, NROWS+1, DETID,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'APERTOBJ' .OR. ATTRB(I) .EQ.
     :                  'APER_NAME') THEN
                    CALL UTRPTT (TP, COLIDN(NC+I), 1, NROWS+1, APERT,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'VOLTAGE') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, VOLT,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'VGAIND') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, GAIN,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'THRESH') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, THRESH,
     :                          STAT(NPUT+I))
                END IF
   90       CONTINUE
        END IF
*
        DO 100 I = 1, NPUT+NATTRB
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(1)
                CONTXT = 'cannot write a row to output table'
                GO TO 999
            END IF
  100   CONTINUE
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   CALL UMSPUT ('VEPPT: ' // CONTXT, DEST, PRIO,        STATOK)
*
 1000   RETURN
        END
