        SUBROUTINE VGTCTP (
*
*  inputs
*
     :                          ROOT, 
*
*  outputs
*
     :                          AARTBL, HVTBL, GNTBL, PATBL, SNTBL,
     :                          DKTBL, CVTBL, DTTBL, DKATBL, STATUS)
*
*  Module Number:
*
*  Module Name: 
*
*  Keyphrase:
*  ----------
*  get the calibration table names from the CL parameter file
*
*  Description:
*  ------------
*
*  FORTRAN Name: VGTCTP.FOR
*
*  Keywords of Accessed Files and Tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       keywords from CL parameter file
*
*  'ccp0'                        I      aperture area table name
*  'ccp1'                        I      high voltage factor calibartion table name
*  'ccp2'                        I      analog gain factor calibartion table name
*  'ccp3'                        I      pre-amplifier noise calibration table name
*  'ccp4'                        I      relative sensitivity calibartion table name
*  'ccp5'                        I      dark signal calibartion table name
*  'ccp7'                        I      CVC offset calibartion table name
*  'ccp8'                        I      digital linearity (deadtime) calibartion table name
*  'ccp9'                        I      translation table from regular aperture
*                                       name to "dark" aperture name
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       None
*  SDAS:
*       UCLGST, UMSPUT
*  OTHERS:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       08-11-89     J.-C. Hsu        Design and coding
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --root name of the input file
        CHARACTER*(*)           ROOT
*
*== output:
*                                       --file name of the dark signal table
        CHARACTER*(*)           DKTBL,
*                                       --file name of the pre-amp noise table
     :                          PATBL,
*                                       --file name of the high voltage factor table
     :                          HVTBL,
*                                       --file name of the relative sensitivity
*                                       --table
     :                          SNTBL,
*                                       --file name of the gain factor table
     :                          GNTBL,
*                                       --file name of the CVC offset table
     :                          CVTBL,
*                                       --file name of the deadtime table
     :                          DTTBL,
*                                       --file name of the dark aperture translation
*                                       --table
     :                          DKATBL,
*                                       --file name of the aperture size table
     :                          AARTBL
*
*== local:
*                                       --return status
        INTEGER                 STATUS,
*                                       --return status
     :                          STAT(30), STATOK,
*                                       --loop indices
     :                          I
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*-------------------------------------------------------------------------------
*
*  read the calibration table names from the CL parameter file
*
        CALL UCLGST ('ccp0',       AARTBL, STAT(1))
        CALL UCLGST ('ccp1',       HVTBL, STAT(2))
        CALL UCLGST ('ccp2',       GNTBL, STAT(3))
        CALL UCLGST ('ccp3',       PATBL, STAT(4))
        CALL UCLGST ('ccp4',       SNTBL, STAT(5))
        CALL UCLGST ('ccp5',       DKTBL, STAT(6))
        CALL UCLGST ('ccp7',       CVTBL, STAT(7))
        CALL UCLGST ('ccp8',       DTTBL, STAT(8))
        CALL UCLGST ('ccp9',       DKATBL, STAT(9))
*
        DO 10 I = 1, 9
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(1)
                CONTXT = 'cannot read CL parameter(s)' 
                GO TO 999
            END IF
   10   CONTINUE
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VGTCTP: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
