        SUBROUTINE VTMPPT (
*
*  inputs
*
     :                  OFILE, ORDER, NPTS, WFLAG, TYPE,
     :                  COEFF, CHISQ,
     :                  XREF, XMIN, XMAX, EPMIN, EPMAX, EPAVE,
     :                  DETID, APERT, VOLT, GAIN, THRESH,
     :                  NATTRB, ATTRB, XREFKY, XMINKY, XMAXKY,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.1.0.1.2
*
*  Module Name: polyfit
*
*  Keyphrase:
*  ----------
*  Write the result of VTEMP to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VTMPPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'A0', 'A1', 'A2', 'A3'...     O      polynomial coefficients
*  'CHISQ'                       O      chi squared of the fit
*  'WEIGHT'                      O      weight scheme
*  (xrefname)                    O      reference value of x as passed from 'XREFNAME'
*  (xminname)                    O      lower limit of x as passed from 'XNAMEMIN'
*  (xmaxname)                    O      upper limit of x as passed from 'XMAXNAME'
*  'EPOCHMIN'                    O      lower limit of epoch
*  'EPOCHMAX'                    O      upper limit of epoch
*  'EPOCHAVE'                    O      average epoch
*  'ORDER'                       O      order of the fit
*  'NPOINTS'                     O      number of data points
*  'TYPE'                        O      data type
*  (attribi)                     O      attributes copied from the input table
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       10-15-85     J.-C. HSU        design and coding
*     2       07-15-87     J.-C. HSU        F77 standard
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --VMS file name of 'OUTTABLE'
        CHARACTER*(*)           OFILE,
*                                       --data type
     :                          TYPE,
*                                       --attributes
     :                          ATTRB(1),
*                                       --column names of minimum and maximum X
     :                          XMINKY, XMAXKY,
*                                       --column name of reference X
     :                          XREFKY
*                                       --weighting scheme
        REAL                    WFLAG,
*                                       --minimum and maximum X
     :                          XMIN, XMAX,
*                                       --reference X value
     :                          XREF
*                                       --minimum, maximum and mean epoch
        DOUBLE PRECISION        EPMIN, EPMAX, EPAVE,
*                                       --polynomial coefficients
     :                          COEFF(0:1),
*                                       --chi-squared
     :                          CHISQ
*                                       --number of input data points
        INTEGER                 NPTS,
*                                       --order of polynomial fit
     :                          ORDER,
*                                       --detector ID
     :                          DETID,
*                                       --number of attributes
     :                          NATTRB
*                                       --high volatge, gain setting,
*                                       --discriminator threshold setting
        REAL                    VOLT, GAIN, THRESH
*                                       --aperture name
        CHARACTER*10            APERT
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, NPUT,
*                                       --number of existed rows of output
*                                       --SDAS table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS, NC,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(100),
*                                       --data type
     :                          DTYPE(100),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    CBUFF(100)
        DOUBLE PRECISION        CBUFF2(20)
        CHARACTER*1             CHAR1
*                                       --column format
        CHARACTER*8             COLFMT(100)
*                                       --output SDAS table names
        CHARACTER*16            CNAME(100),
*                                       --column units
     :                          UNIT(100)
*                                       --error message context
        CHARACTER*130           CONTXT
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify column names
*
        DO 10 I = 1, ORDER+1
            WRITE (CHAR1, '(I1)') I-1
            CNAME(I) = 'A' // CHAR1
   10   CONTINUE
        CNAME(ORDER+2) = 'CHISQ'
        CNAME(ORDER+3) = 'WEIGHT'
        CNAME(ORDER+4) = XREFKY
        CNAME(ORDER+5) = XMINKY
        CNAME(ORDER+6) = XMAXKY
        CNAME(ORDER+7) = 'EPOCHMIN'
        CNAME(ORDER+8) = 'EPOCHMAX'
        CNAME(ORDER+9) = 'EPOCHAVE'
*
*  column data type
*
        DO 20 I = 1, ORDER+6
            DTYPE(I) = TYREAL
   20   CONTINUE
*
        DO 30 I = ORDER+7, ORDER+9
            DTYPE(I) = TYDOUB
   30   CONTINUE
*
        CNAME(ORDER+10) = 'ORDER'
        CNAME(ORDER+11) = 'NPOINTS'
        CNAME(ORDER+12) = 'TYPE'
        DTYPE(ORDER+10) = TYINT
        DTYPE(ORDER+11) = TYINT
        DTYPE(ORDER+12) = -7
*
        NC = ORDER + 12
*
*  identify attributes
*
        IF (NATTRB .GT. 0) THEN
            DO 40 I = 1, NATTRB
                IF (ATTRB(I) .EQ. 'DETECTOB' .OR. ATTRB(I) .EQ.
     :                  'DET_NUM') THEN
                    CNAME(NC+I) = 'DET_NUM'
                    DTYPE(NC+I) = TYINT
                ELSE IF (ATTRB(I) .EQ. 'APERTOBJ' .OR. ATTRB(I) .EQ.
     :                  'APER_NAME') THEN
                    CNAME(NC+I) = 'APER_NAME'
                    DTYPE(NC+I) = -10
                ELSE IF (ATTRB(I) .EQ. 'VOLTAGE') THEN
                    CNAME(NC+I) = 'VOLTAGE'
                    DTYPE(NC+I) = TYREAL
                ELSE IF (ATTRB(I) .EQ. 'VGAIND') THEN
                    CNAME(NC+I) = 'VGAIND'
                    DTYPE(NC+I) = TYREAL
                ELSE IF (ATTRB(I) .EQ. 'THRESH') THEN
                    CNAME(NC+I) = 'THRESH'
                    DTYPE(NC+I) = TYREAL
                END IF
   40       CONTINUE
        END IF
*
*  find out the number of rows the SDAS table already has and append
*  the result of this routine as one new row at the bottom
*
        NCOLS = NC + NATTRB
*
        DO 50 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   50   CONTINUE
*
*  specify buffer contents
*
        DO 60 I = 1, ORDER+1
            CBUFF(I) = SNGL(COEFF(I-1))
   60   CONTINUE
        CBUFF(ORDER+2) = SNGL(CHISQ)
        CBUFF(ORDER+3) = WFLAG
        CBUFF(ORDER+4) = XREF
        CBUFF(ORDER+5) = XMIN
        CBUFF(ORDER+6) = XMAX
        CBUFF2(1) = EPMIN
        CBUFF2(2) = EPMAX
        CBUFF2(3) = EPAVE
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up output table'
            GO TO 999
        END IF
*
*  write results to the output table
*
        CALL UTRPTR (TP, COLIDN(1), ORDER+6, NROWS+1, CBUFF, STAT(1))
        CALL UTRPTD (TP, COLIDN(ORDER+7), 3, NROWS+1, CBUFF2, STAT(2))
        CALL UTRPTI (TP, COLIDN(ORDER+10), 1, NROWS+1, ORDER, STAT(3))
        CALL UTRPTI (TP, COLIDN(ORDER+11), 1, NROWS+1, NPTS, STAT(4))
        CALL UTRPTT (TP, COLIDN(ORDER+12), 1, NROWS+1, TYPE, STAT(5))
        NPUT = 5
*
*  write attributes
*
        IF (NATTRB .NE. 0) THEN
            DO 70 I = 1, NATTRB
                IF (ATTRB(I) .EQ. 'DETECTOB' .OR. ATTRB(I) .EQ.
     :                  'DET_NUM') THEN
                    CALL UTRPTI (TP, COLIDN(NC+I), 1, NROWS+1, DETID,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'APERTOBJ' .OR. ATTRB(I) .EQ.
     :                  'APER_NAME') THEN
                    CALL UTRPTT (TP, COLIDN(NC+I), 1, NROWS+1, APERT,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'VOLTAGE') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, VOLT,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'VGAIND') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, GAIN,
     :                          STAT(NPUT+I))
                ELSE IF (ATTRB(I) .EQ. 'THRESH') THEN
                    CALL UTRPTR (TP, COLIDN(NC+I), 1, NROWS+1, THRESH,
     :                          STAT(NPUT+I))
                END IF
   70       CONTINUE
        END IF
*
        DO 80 I = 1, NPUT+NATTRB
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(1)
                CONTXT = 'cannot write a row to output table'
                GO TO 999
            END IF
   80   CONTINUE
*
        CALL UTTCLO (TP, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   CALL UMSPUT ('VTMPPT: ' // CONTXT, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
