        SUBROUTINE VRSNGT (
*
*  outputs
*
     :                  COUNT, DCOUNT, TEMP, EPOCH, APERT, TARGET,
     :                  PSFLAG, NPTS, OFILE, TYPE, TNAME, STATUS)
*
*  Module number: 15.8.1
*
*  Module name: relsen
*
*  Keyphrase:
*  ----------
*  Input parameters and data needed for relative sensitivity calibration
*
*  Description:
*  ------------
*
*  FORTRAN name: VRSNGT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       CL parameters:
*
*  'intable'                     I      input table name
*  'outtable'                    I      output table name
*  'temp_key'                    I      keyword name of the temperature
*
*       input table column names:
*
*  (temp_key)                    I      temperature, specified in 'temp_key'
*  'APERTOBJ'                    I      aperture name
*  'TRGTNAME'                    I      target name
*  'PTSRCFLG'                    I      point source flag
*  'EPOCH'                       I      epoch
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTIN, VRSNCR
*  SDAS:
*       UCLGSR, UCLGST, UTRGTR, UTRGTD, UTRGTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       05-30-86     J.-C. HSU        design and coding
*     2       10-05-87     J.-C. HSU        F77 standard
*     3       10-17-89     J.-C. HSU        error propagation
*-------------------------------------------------------------------------------
*
*== output:
*                                       --count rate and its standard deviation
        REAL                    COUNT(1), DCOUNT(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --aperture name
        CHARACTER*(*)           APERT(1),
*                                       --target names
     :                          TARGET,
*                                       --point source flag
     :                          PSFLAG,
*                                       --output file name
     :                          OFILE,
*                                       --data type
     :                          TYPE,
*                                       --temperature name
     :                          TNAME
*                                       --total number of valid input data points
        INTEGER                 NPTS,
*                                       --error status
     :                          STATUS
*
*== local:
*                                       --number of input data points
        INTEGER                 NROWS,
*                                       --pointer of table descripter and column
*                                       --identifier
     :                          TP, COLIDN(20),
*                                       --loop indices, number of input columns
     :                          I, J, NCOLS,
*                                       --status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    COLBUF(20)
        CHARACTER*1             FLAG
*                                       --column names
        CHARACTER*16            COLNAM(20)
        CHARACTER*20            TRGT
*                                       --null flag in UTRGTR
        LOGICAL                 NULMSK(20)
        CHARACTER*5             CHAR5
*                                       --input file name
        CHARACTER*128           IFILE
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  input CL parameters:
*
        CALL UCLGST ('intable',         IFILE, STAT(1))
        CALL UCLGST ('outtable',        OFILE, STAT(2))
        CALL UCLGST ('type',            TYPE, STAT(3))
        CALL UCLGST ('temp_key',        TNAME, STAT(4))
*
        DO 10 I = 1, 4
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(1)
                CONTXT = 'can not get CL parameter(s)'
                GO TO 999
            END IF
   10   CONTINUE
*
*  define input column names
*
        COLNAM(1) = TNAME
        COLNAM(2) = 'EPOCH'
        COLNAM(3) = 'PTSRCFLG'
        COLNAM(4) = 'APERTOBJ'
        COLNAM(5) = 'TRGTNAME'
*
*  analog data only
*
        CALL UUUPPC (TYPE,    TYPE)
        IF (TYPE(1:1) .EQ. 'A') THEN
            COLNAM(6) = 'AOBJ_C'
            COLNAM(7) = 'AOBJ_ERR_C'
            NCOLS = 7
*
*  digital data only
*
        ELSE IF (TYPE(1:1) .EQ. 'D') THEN
            COLNAM(6) = 'DOBJ_C'
            COLNAM(7) = 'DOBJ_ERR_C'
            NCOLS = 7
        ELSE
            STATUS = ERRNUM(1)
            CONTXT = 'illegal data type keyword ' // TYPE
            GO TO 999
        END IF
*
*  calculate corrected count rates
*
        CALL VRSNCR (IFILE, TYPE,    STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot calculate corrected count rates'
            GO TO 999
        END IF
*
*  set up the input table
*
        CALL CDTIN (IFILE, COLNAM, NCOLS,    TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up input table'
            GO TO 999
        END IF
*
*  read data from the input table, row by row
*
        NPTS = 0
        DO 50 I = 1, NROWS
            CALL UTRGTR (TP, COLIDN(1), 1, I,     COLBUF(1), NULMSK(1),
     :                   STAT(1))
            CALL UTRGTD (TP, COLIDN(2), 1, I,     EPOCH(I), NULMSK(2),
     :                   STAT(2))
            CALL UTRGTT (TP, COLIDN(3), 1, I,     FLAG, NULMSK(3),
     :                   STAT(3))
            CALL UTRGTT (TP, COLIDN(4), 1, I,     APERT(I), NULMSK(4),
     :                   STAT(4))
            CALL UTRGTT (TP, COLIDN(5), 1, I,     TRGT, NULMSK(5),
     :                   STAT(5))
            CALL UTRGTR (TP, COLIDN(6), 2, I,     COLBUF(2), NULMSK(6),
     :                   STAT(6))
*
*  read the point source flag and target name from the first row
*
            IF (I .EQ. 1) THEN
                PSFLAG = FLAG
                TARGET = TRGT
            END IF
*
            STATOK = OK
            DO 20 J = 1, 6
                IF (STAT(J) .NE. OK) THEN
                    STATOK = ERRNUM(1)
                    GO TO 40
                END IF
   20       CONTINUE
*
            DO 30 J = 1, NCOLS
                IF (NULMSK(J)) THEN
                    STATOK = ERRNUM(2)
                    GO TO 40
                END IF
   30       CONTINUE
*
*  if error in UTRGTR or count rate is non-positive, put an error message and
*  do NOT increment the data array index
*
   40       CONTINUE
            IF (STATOK .NE. OK   .OR.   COLBUF(2) .LE. 0.) THEN
                WRITE(CHAR5, '(I5)') I
                CONTXT = 'can not read input data or non-positive '
     :                          // 'data at row #' // CHAR5
                CALL UMSPUT (CONTXT, DEST, PRIO,   STATOK)
*
*  data must have the same point source flag and the same target
*
            ELSE IF (FLAG .NE. PSFLAG .OR. TRGT .NE. TARGET) THEN
                CONTXT = 'different point source flag or target '
     :                          // 'at row #' // CHAR5
                CALL UMSPUT (CONTXT, DEST, PRIO,        STATOK)
            ELSE
                NPTS = NPTS + 1
                TEMP(NPTS) = COLBUF(1)
                EPOCH(NPTS) = EPOCH(I)
                APERT(NPTS) = APERT(I)
                COUNT(NPTS) = COLBUF(2)
                DCOUNT(NPTS) = COLBUF(3)
            END IF
   50   CONTINUE
*
*  close the input table
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close input table'
            GO TO 999
        END IF
*
*  check if there is any valid data point
*
        IF (NPTS .LE. 0) THEN
            STATUS = ERRNUM(3)
            CONTXT = 'no valid input data'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VRSNGT: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,    STATOK)
*
 1000   RETURN
        END
