        SUBROUTINE VRSNPT (
*
*  inputs
*
     :                  OFILE, APERT, TRGT, PSFLAG, RSEN, DRSEN, TEMP,
     :                  EPOCH, INDEX, NPTS, TYPE, TNAME,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.8.2
*
*  Module Name: relsen
*
*  Keyphrase:
*  ----------
*  Write the result of relative sensitivity calibration to the output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VRSNPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'APER_NAME'                   O      aperture name
*  'TRGTNAME'                    O      trarget name
*  'SENSITIV'                    O      relative sensitivity
*  'SENSITIV_ERR'                O      standard deviation of relative sensitivity
*  (temp_key)                    O      temperature as specified in TNAME
*  'EPOCH'                       O      average epoch
*  'TYPE'                        O      data type
*  'PTSRCFLG'                    O      point source flag
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       05-30-86     J.-C. HSU        design and coding
*     2       10-05-87     J.-C. HSU        F77 standard
*     3       10-18-89     J.-C. HSU        error propagation
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --output table name
        CHARACTER*(*)           OFILE,
*                                       --aperture name
     :                          APERT(1),
*                                       --target name
     :                          TRGT,
*                                       --point source flag
     :                          PSFLAG,
*                                       --data type
     :                          TYPE,
*                                       --temperature column name
     :                          TNAME
*                                       --relative sensitivity and its error
        REAL                    RSEN(1), DRSEN(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --number of input data points
        INTEGER                 NPTS,
*                                       --array index
     :                          INDEX(1)
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, J, NPUT,
*                                       --number of existed rows of output table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(20),
*                                       --data type
     :                          DTYPE(20),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    CBUFF(20)
*                                       --column format
        CHARACTER*8             COLFMT(20)
*                                       --column names
        CHARACTER*16            CNAME(20),
*                                       --column units
     :                          UNIT(20)
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  define output column names
*
        CNAME(1) = 'APER_NAME'
        CNAME(2) = 'TRGTNAME'
        CNAME(3) = 'SENSITIV'
        CNAME(4) = 'SENSITIV_ERR'
        CNAME(5) = TNAME
        CNAME(6) = 'EPOCH'
        CNAME(7) = 'TYPE'
        CNAME(8) = 'PTSRCFLG'
        NCOLS = 8
*
*  define output column data types 
*
        DTYPE(1) = -10
        DTYPE(2) = -20
        DO 10 I = 3, 5
            DTYPE(I) = TYREAL
   10   CONTINUE
        DTYPE(6) = TYDOUB
        DTYPE(7) = -7
        DTYPE(8) = -1
*
*  define output column units and display formats 
*
        DO 20 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   20   CONTINUE
*
*  set up the output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,   TP,
     :                  COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up output table'
            GO TO 999
        END IF
*
*  append each observation to the output table as one new row
*
        DO 40 I = 1, NPTS
*
*  assign buffer contents
*
            CBUFF(1) = RSEN(I)
            CBUFF(2) = DRSEN(I)
            CBUFF(3) = TEMP(INDEX(I))
*
            CALL UTRPTT (TP, COLIDN(1), 1, NROWS+I, APERT(INDEX(I)),
     :                          STAT(1))
            CALL UTRPTT (TP, COLIDN(2), 1, NROWS+I, TRGT,    STAT(2))
            CALL UTRPTR (TP, COLIDN(3), 3, NROWS+I, CBUFF,   STAT(3))
            CALL UTRPTD (TP, COLIDN(6), 1, NROWS+I, EPOCH(INDEX(I)),
     :                          STAT(4))
            CALL UTRPTT (TP, COLIDN(7), 1, NROWS+I, TYPE,    STAT(5))
            CALL UTRPTT (TP, COLIDN(8), 1, NROWS+I, PSFLAG,  STAT(6))
            NPUT = 6
*
            DO 30 J = 1, NPUT
                IF (STAT(J) .NE. OK) THEN
                    STATUS = ERRNUM(1)
                    CONTXT = 'cannot write a row to output table'
                    GO TO 999
                END IF
   30       CONTINUE
   40   CONTINUE
*
*  close the output table
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VRSNPT: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
