        SUBROUTINE VSCTPT (
*
*  inputs
*
     :                  OFILE, RATIO, DRATIO, DISTD, PA, APER, TARGET,
     :                  TEMP, EPOCH, NPTS,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.14.2
*
*  Module Name: scatterv
*
*  Keyphrase:
*  ----------
*  Write the result of VSCATT to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VSCTPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'APER_NAME'                   O      aperture name
*  'ANGULAR_DIST'                O      angular distance from the target
*  'POSITION_ANG'                O      position angle from the target
*  'FLUX_RATIO'                  O      ratio of the observed flux to the target
*  'RATIO_ERR'                   O      standard deviation of flux ratio
*  'TEMPERATURE'                 O      temperature of observation
*  'EPOCH'                       O      epoch of observation
*  'TRGTNAME'                    O      target name
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       06-26-87     J.-C. HSU        design and coding
*     2       11-20-87     J.-C. HSU        F77 SDAS
*-------------------------------------------------------------------------------
*
*== input:
*                                       --output table name
        CHARACTER*(*)           OFILE,
*                                       --aperture name and target name
     :                          APER(1), TARGET(1)
*                                       --flux ratios and their standard dev
        REAL                    RATIO(1), DRATIO(1),
*                                       --position angle
     :                          PA(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --angular distance and epoch
        DOUBLE PRECISION        DISTD(1), EPOCH(1)
*                                       --number of input data points
        INTEGER                 NPTS
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, J,
*                                       --number of existed rows of output table
     :                          NROWS,
*                                       --number of columns in output table
     :                          NCOLS,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(20),
*                                       --data type
     :                          DTYPE(20),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --output SDAS table names
        CHARACTER*16            CNAME(20),
*                                       --column units
     :                          UNIT(20)
*                                       --column format
        CHARACTER*8             COLFMT(20)
*                                       --error message context
        CHARACTER*130           CONTXT
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify column names
*
        CNAME(1) = 'APER_NAME'
        CNAME(2) = 'ANGULAR_DIST'
        CNAME(3) = 'POSITION_ANG'
        CNAME(4) = 'FLUX_RATIO'
        CNAME(5) = 'RATIO_ERR'
        CNAME(6) = 'TEMPERATURE'
        CNAME(7) = 'EPOCH'
        CNAME(8) = 'TRGTNAME'
*
*  column data type
*
        DTYPE(1) = -10
        DTYPE(2) = TYDOUB
        DTYPE(3) = TYREAL
        DTYPE(4) = TYREAL
        DTYPE(5) = TYREAL
        DTYPE(6) = TYREAL
        DTYPE(7) = TYDOUB
        DTYPE(8) = -20
*
        NCOLS = 8
*
        DO 10 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   10   CONTINUE
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot set up output table'
            GO TO 999
        END IF
*
*  put the result into output table
*
        DO 40 I = 1, NPTS
            CALL UTRPTT (TP, COLIDN(1), 1, NROWS+1, APER(I),    STAT(1))
            CALL UTRPTD (TP, COLIDN(2), 1, NROWS+1, DISTD(I),   STAT(2))
            CALL UTRPTR (TP, COLIDN(3), 1, NROWS+1, PA(I),      STAT(3))
            CALL UTRPTR (TP, COLIDN(4), 1, NROWS+1, RATIO(I),   STAT(4))
            CALL UTRPTR (TP, COLIDN(5), 1, NROWS+1, DRATIO(I),  STAT(5))
            CALL UTRPTR (TP, COLIDN(6), 1, NROWS+1, TEMP(I),    STAT(6))
            CALL UTRPTD (TP, COLIDN(7), 1, NROWS+1, EPOCH(I),   STAT(7))
            CALL UTRPTT (TP, COLIDN(8), 1, NROWS+1, TARGET(I),  STAT(8))
*
            STATUS = 0
            DO 20 J = 1, NCOLS
                IF (STAT(I) .NE. OK) THEN
                    STATUS = ERRNUM(1)
                    GO TO 30
                END IF
   20       CONTINUE
*
   30       IF (STATUS .NE. OK) THEN
                CONTXT = 'cannot write a row to output table'
                CALL UMSPUT (CONTXT, DEST, PRIO,        STATOK)
            ELSE
                NROWS = NROWS + 1
            END IF
   40   CONTINUE
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   CALL UMSPUT ('VSCTPT: ' // CONTXT, DEST, PRIO,        STATOK)
*
 1000   RETURN
        END
