        SUBROUTINE VAP3SP (
*
*  inputs
*
     :                  OFILE,
     :                  COEFF, MATRIX, CHISQ, NCOEFF, NVAR, DIM,
     :                  SCHEME, TOLERN, FRAC, ITER, NPTS,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.11.2.3.1.3
*
*  Module Name: smaperloc
*
*  Keyphrase:
*  ----------
*  Write the result of VAP3S to output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VAP3SP.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'V2'                          O      V2 coordinate of the aperture center
*  'V3'                          O      V3 coordinate of the aperture center
*  'RADIUS'                      O      radius of the aperture
*  'SIGMA_V2'                    O      standard error of V2
*  'SIGMA_V3'                    O      standard error of V3
*  'SIGMA_RADIUS'                O      standard error of RADIUS
*  'CORR_V2_V3'                  O      correlation between V2 and V3
*  'CORR_V2_R'                   O      correlation between V2 and RADIUS
*  'CORR_V3_R'                   O      correlation between V3 and RADIUS
*  'V2_WEIGHT'                   O      weight scheme of V2
*  'V3_WEIGHT'                   O      weight scheme of V3
*  'CHISQ'                       O      chi-squared
*  'TOLERANCE'                   O      convergence of sigma squared
*  'FRACTION'                    O      specified fraction of coefficient
*                                       modification
*  'NPOINTS'                     O      number of data points
*  'ITERMAX'                     O      number of iterations of least square fit
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       02-20-87     J.-C. HSU        design and coding
*     2       11-16-87     J.-C. HSU        F77 SDAS
*
*-------------------------------------------------------------------------------
*
*== input:
*                                       --output table name
        CHARACTER*(*)           OFILE
*                                       --number of coefficients and variables
        INTEGER                 NCOEFF, NVAR,
*                                       --number of input data points
     :                          NPTS,
*                                       --number of least square iterations
     :                          ITER,
*                                       --dimension of the covariance matrix as
*                                       --declared in the calling routine
     :                          DIM
*                                       --fitting coefficients, covariance
*                                       --matrix, and chi-squared
        DOUBLE PRECISION        COEFF(1), MATRIX(DIM, DIM), CHISQ
*                                       --weighting schemes
        REAL                    SCHEME(1),
*                                       --tolerance of sigma-squared difference
     :                          TOLERN,
*                                       --fraction of coefficient modification
     :                          FRAC
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --output table content
        REAL                    CBUFF(240)
*                                       --loop index
        INTEGER                 I, J, K,
*                                       --pointer to table descripter
     :                          TP,
*                                       --number of existed rows of output table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS,
*                                       --column data type
     :                          DTYPE(240),
*                                       --column identifiers
     :                          COLIDN(240),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --output table column names
        CHARACTER*16            CNAME(240),
*                                       --column units
     :                          UNIT(240)
*                                       --column format
        CHARACTER*8             COLFMT(240)
*                                       --error message context
        CHARACTER*130           CONTXT
*
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  specify output buffer contents
*
*  coefficients and their standard errors
*
        DO 10 I = 1, NCOEFF
            CBUFF(I) = REAL(COEFF(I))
            CBUFF(NCOEFF+I) = REAL(SQRT(MATRIX(I,I)))
   10   CONTINUE
*
*  correlations between coefficients
*
        K = 2 * NCOEFF
        IF (NCOEFF .NE. 1) THEN
            DO 30 I = 1, NCOEFF-1
                DO 20 J = I+1, NCOEFF
                    K = K + 1
                    CBUFF(K) = REAL(MATRIX(I, J) /
     :                          SQRT(MATRIX(I, I) * MATRIX(J, J)))
   20           CONTINUE
   30       CONTINUE
        END IF
*
*  others
*
        DO 40 I = 1, NVAR
            CBUFF(K+I) = SCHEME(I)
   40   CONTINUE
        CBUFF(K+NVAR+1) = REAL(CHISQ)
        CBUFF(K+NVAR+2) = TOLERN
        CBUFF(K+NVAR+3) = FRAC
*
*  specify column names
*
*  coefficients
*
        CNAME(1) = 'V2'
        CNAME(2) = 'V3'
        CNAME(3) = 'RADIUS'
*
*  standard error of coefficients
*
        CNAME(NCOEFF+1) = 'SIGMA_V2'
        CNAME(NCOEFF+2) = 'SIGMA_V3'
        CNAME(NCOEFF+3) = 'SIGMA_RADIUS'
*
*  correlation between coefficients
*
        CNAME(NCOEFF*2+1) = 'CORR_V2_V3'
        CNAME(NCOEFF*2+2) = 'CORR_V2_R'
        CNAME(NCOEFF*2+3) = 'CORR_V3_R'
*
*  others
*
        CNAME(K+1) = 'V2_WEIGHT'
        CNAME(K+2) = 'V3_WEIGHT'
        CNAME(K+NVAR+1) = 'CHISQ'
        CNAME(K+NVAR+2) = 'TOLERANCE'
        CNAME(K+NVAR+3) = 'FRACTION'
        CNAME(K+NVAR+4) = 'NPOINTS'
        CNAME(K+NVAR+5) = 'ITERMAX'
*
*  define column units, formats, and data types
*
        NCOLS = K + 5 + NVAR
*
        DO 50 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
            DTYPE(I) = TYREAL
   50   CONTINUE
        DTYPE(NCOLS-1) = TYINT
        DTYPE(NCOLS) = TYINT
*
*  set up output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,
     :                  TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. 0) THEN
            CONTXT = 'cannot set up output table'
            GO TO 999
        END IF
*
*  write result to output table
*
        CALL UTRPTR (TP, COLIDN, NCOLS-2, NROWS+1, CBUFF,     STAT(1))
        CALL UTRPTI (TP, COLIDN(NCOLS-1), 1, NROWS+1, NPTS,   STAT(2))
        CALL UTRPTI (TP, COLIDN(NCOLS), 1, NROWS+1, ITER,     STAT(3))
*
        DO 60 I = 1, 3
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(2)
                CONTXT = 'cannot write a row to output table'
                GO TO 999
            END IF
   60   CONTINUE
*
        CALL UTTCLO (TP,   STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   CALL UMSPUT ('VAP3SP: ' // CONTXT, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
