        SUBROUTINE VHVFGT (
*
*  outputs
*
     :                  COUNT, DCOUNT, TEMP, HV, EPOCH, DETID, TYPE,
     :                  APERT, TARGET, NPTS, OFILE, TNAME, STATUS)
*
*  Module number: 15.9.2.1.1
*
*  Module name: voltfac
*
*  Keyphrase:
*  ----------
*  Input parameters and data needed for high voltage factor calibration
*
*  Description:
*  ------------
*
*  FORTRAN name: VHVFGT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       CL parameters:
*
*  'intable'                     I      input table name
*  'outtable'                    I      output table name
*  'temp_key'                    I      keyword name of the temperature
*  'type'                        I      data type
*
*       input table column names:
*
*  (temp_key)                    I      temperature, specified in 'temp_key'
*  'VOLTAGE'                     I      high voltage setting
*  'DETECTOB'                    I      object detector ID
*  'EPOCH'                       I      epoch
*  'APERTOBJ'                    I      aperture used for the object
*  'TRGTNAME'                    I      target name
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTIN, VHVFCR
*  SDAS:
*       UUUPPC, UCLGSR, UCLGST, UTRGTR, UTRGTD, UTRGTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       12-20-85     J.-C. HSU        design and coding
*     2       09-15-87     J.-C. HSU        F77 standard
*     3       10-15-89     J.-C. HSU        error propagation
*-------------------------------------------------------------------------------
*
*== output:
*                                       --count rate and its standard deviation
        REAL                    COUNT(1), DCOUNT(1),
*                                       --temperature
     :                          TEMP(1),
*                                       --high voltage setting
     :                          HV(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --data type
        CHARACTER*(*)           TYPE,
*                                       --output file name
     :                          OFILE,
*                                       --target name
     :                          TARGET,
*                                       --aperture name
     :                          APERT,
*                                       --temperature name
     :                          TNAME
*                                       --total number of valid input data points
        INTEGER                 NPTS,
*                                       --detector ID
     :                          DETID, DET,
*                                       --error status
     :                          STATUS
*
*== local:
*                                       --number of input data points
        INTEGER                 NROWS,
*                                       --pointer of table descripter and column
*                                       --identifier
     :                          TP, COLIDN(20),
*                                       --loop indices, number of input columns
     :                          I, J, NCOLS,
*                                       --status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    COLBUF(20)
*                                       --null flag in UTRGTR
        LOGICAL                 NULMSK(20)
        CHARACTER*5             CHAR5
        CHARACTER*10            APER
        CHARACTER*20            TRGT
*                                       --column names
        CHARACTER*16            COLNAM(20)
*                                       --input file name
        CHARACTER*128           IFILE
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  input CL parameters:
*
        CALL UCLGST ('intable',         IFILE, STAT(1))
        CALL UCLGST ('outtable',        OFILE, STAT(2))
        CALL UCLGST ('type',            TYPE, STAT(3))
        CALL UCLGST ('temp_key',        TNAME, STAT(4))
*
        DO 10 I = 1, 4
            IF (STAT(I) .NE. OK) THEN
                STATUS = ERRNUM(1)
                CONTXT = 'can not get CL parameter(s)'
                GO TO 999
            END IF
   10   CONTINUE
*
*  define input table column names
*
        COLNAM(1) = TNAME
        COLNAM(2) = 'VOLTAGE'
        COLNAM(3) = 'EPOCH'
        COLNAM(4) = 'DETECTOB'
        COLNAM(5) = 'APERTOBJ'
        COLNAM(6) = 'TRGTNAME'
*
*  analog data only
*
        CALL UUUPPC (TYPE,    TYPE)
        IF (TYPE(1:1) .EQ. 'A') THEN
            COLNAM(7) = 'AOBJ_C'
            COLNAM(8) = 'AOBJ_ERR_C'
            NCOLS = 8
*
*  digital data only
*
        ELSE IF (TYPE(1:1) .EQ. 'D') THEN
            COLNAM(7) = 'DOBJ_C'
            COLNAM(8) = 'DOBJ_ERR_C'
            NCOLS = 8
        ELSE
            STATUS = ERRNUM(1)
            CONTXT = 'illegal data type keyword ' // TYPE
            GO TO 999
        END IF
*
*  calculate corrected count rates
*
        CALL VHVFCR (IFILE, TYPE,    STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot calculate corrected count rates'
            GO TO 999
        END IF
*
*  set up input table
*
        CALL CDTIN (IFILE, COLNAM, NCOLS,    TP, COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up input table'
            GO TO 999
        END IF
*
*  read data from input table
*
        NPTS = 0
        DO 50 I = 1, NROWS
            CALL UTRGTR (TP, COLIDN(1), 2, I,   COLBUF(1), NULMSK(1),
     :                   STAT(1))
            CALL UTRGTD (TP, COLIDN(3), 1, I,   EPOCH(I), NULMSK(3),
     :                   STAT(2))
            CALL UTRGTI (TP, COLIDN(4), 1, I,   DET, NULMSK(4),
     :                   STAT(3))
            CALL UTRGTT (TP, COLIDN(5), 1, I,   APER, NULMSK(5),
     :                   STAT(4))
            CALL UTRGTT (TP, COLIDN(6), 1, I,   TRGT, NULMSK(6),
     :                   STAT(5))
            CALL UTRGTR (TP, COLIDN(7), 2, I,   COLBUF(3), NULMSK(7),
     :                   STAT(6))
*
*  read detector ID, aperture name, and target name from first row
*
            IF (I .EQ. 1) THEN
                DETID = DET
                APERT = APER
                TARGET = TRGT
            END IF
*
            STATOK = OK
            DO 20 J = 1, 6
                IF (STAT(J) .NE. OK) THEN
                    STATOK = ERRNUM(1)
                    GO TO 40
                END IF
   20       CONTINUE
*
            DO 30 J = 1, NCOLS
                IF (NULMSK(J)) THEN
                    STATOK = ERRNUM(2)
                    GO TO 40
                END IF
   30       CONTINUE
*
*  if error in UTRGTR or count rate is non-positive, put an error message and
*  do NOT increment the data array index
*
   40       CONTINUE
            WRITE(CHAR5, '(I5)') I
            IF (STATOK .NE. OK   .OR.   COLBUF(3) .LE. 0.) THEN
                CONTXT = 'can not read input data or non-positive '
     :                          // 'data at row #' // CHAR5
                CALL UMSPUT (CONTXT, DEST, PRIO,        STATOK)
*
*  must have the same aperture, same detector, and the same target
*
            ELSE IF (DET .NE. DETID .OR. APER .NE. APERT .OR.
     :          TRGT .NE. TARGET) THEN
                CONTXT = 'different detector or aperture or target '
     :                          // 'at row #' // CHAR5
                CALL UMSPUT (CONTXT, DEST, PRIO,        STATOK)
            ELSE
                NPTS = NPTS + 1
                TEMP(NPTS) = COLBUF(1)
                HV(NPTS) = COLBUF(2)
                EPOCH(NPTS) = EPOCH(I)
                COUNT(NPTS) = COLBUF(3)
                DCOUNT(NPTS) = COLBUF(4)
            END IF
   50   CONTINUE
*
*  close input table
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close input table'
            GO TO 999
        END IF
*
*  check if there is any valid data point
*
        IF (NPTS .LE. 0) THEN
            STATUS = ERRNUM(3)
            CONTXT = 'no valid input data'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VHVFGT: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,     STATOK)
*
 1000   RETURN
        END
