        SUBROUTINE VHVFPT (
*
*  inputs
*
     :                  OFILE, HV, HVFAC, DHVFAC, TEMP, EPOCH, INDEX,
     :                  NPTS, TYPE, TNAME, DETID, APERT, TARGET,
*
*  outputs
*
     :                  STATUS)
*
*  Module Number: 15.9.2.1.2
*
*  Module Name: voltfac
*
*  Keyphrase:
*  ----------
*  Write the result of high voltage factor calibration to the output table
*
*  Description:
*  ------------
*
*  FORTRAN name: VHVFPT.FOR
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*  Name                         I/O     Description / Comments
*
*       output table column names:
*
*  'TYPE'                        O      data type
*  'DET_NUM'                     O      detector ID
*  'VOLTAGE'                     O      high voltage setting
*  'HVFACT'                      O      high voltage factor
*  'HVFACT_ERR'                  O      error of high voltage factor
*  (temp_key)                    O      temperature as specified in TNAME
*  'EPOCH'                       O      average epoch
*  'APER_NAME'                   O      aperture name
*  'TRGTNAME'                    O      target name
*
*  Subroutines Called:
*  -------------------
*  CDBS:
*       CDTOUT
*  SDAS:
*       UTRPTR, UTRPTI, UTRPTD, UTRPTT, UTTCLO, UMSPUT
*  Others:
*       None
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       12-20-85     J.-C. HSU        design and coding
*     2       09-15-87     J.-C. HSU        F77 standard
*     3       09-15-89     J.-C. HSU        error propagation
*-------------------------------------------------------------------------------
*
*== input:
*                                       --output table name
        CHARACTER*(*)           OFILE,
*                                       --temperature column name
     :                          TNAME,
*                                       --data type
     :                          TYPE,
*                                       --target name
     :                          TARGET,
*                                       --aperture name
     :                          APERT
*                                       --high voltage factor and its standard
*                                       --deviation
        REAL                    HVFAC(1), DHVFAC(1),
*                                       --high voltage setting
     :                          HV(1),
*                                       --temperature
     :                          TEMP(1)
*                                       --epoch
        DOUBLE PRECISION        EPOCH(1)
*                                       --number of input data points
        INTEGER                 NPTS,
*                                       --array index
     :                          INDEX(1),
*                                       --detector ID
     :                          DETID
*
*== output:
*                                       --error status
        INTEGER                 STATUS
*
*== local:
*                                       --loop index
        INTEGER                 I, J, NPUT,
*                                       --number of existed rows of the output
*                                       --table
     :                          NROWS,
*                                       --number of columns
     :                          NCOLS,
*                                       --pointer to table descripter
     :                          TP,
*                                       --column identifiers
     :                          COLIDN(20),
*                                       --data type
     :                          DTYPE(20),
*                                       --error status
     :                          STAT(20), STATOK
*                                       --data buffer
        REAL                    CBUFF(20)
*                                       --column format
        CHARACTER*8             COLFMT(20)
*                                       --output table names
        CHARACTER*16            CNAME(20),
*                                       --column units
     :                          UNIT(20)
*                                       --error message context
        CHARACTER*130           CONTXT, MESS
*==========================begin iraf77.inc (without INTEGER*2)=================
*     Include file for the iraf77 FORTRAN interface to the IRAF VOS
*     Get IRAF common into main program
*
      LOGICAL MEMB(1)
      INTEGER MEMI(1)
      INTEGER MEML(1)
      REAL MEMR(1)
      DOUBLE PRECISION MEMD(1)
      COMPLEX MEMX(1)
      EQUIVALENCE (MEMB, MEMI, MEML, MEMR, MEMD, MEMX)
      COMMON /MEM/ MEMD
*
*     File I/O access modes
*
      INTEGER   RDONLY
      PARAMETER (RDONLY = 1)
      INTEGER   RDWRIT
      PARAMETER (RDWRIT = 2)
      INTEGER   WRONLY
      PARAMETER (WRONLY = 3)
      INTEGER   APPEND
      PARAMETER (APPEND = 4)
      INTEGER   NEWFIL
      PARAMETER (NEWFIL = 5)
      INTEGER   TMPFIL
      PARAMETER (TMPFIL = 6)
      INTEGER   NEWCPY
      PARAMETER (NEWCPY = 7)
      INTEGER   NEWIMG
      PARAMETER (NEWIMG = 5)
*
*     codes for data types
*
      INTEGER   TYBOOL
      PARAMETER (TYBOOL = 1)
      INTEGER   TYCHAR
      PARAMETER (TYCHAR = 2)
      INTEGER   TYSHOR
      PARAMETER (TYSHOR = 3)
      INTEGER   TYINT
      PARAMETER (TYINT = 4)
      INTEGER   TYLONG
      PARAMETER (TYLONG = 5)
      INTEGER   TYREAL
      PARAMETER (TYREAL = 6)
      INTEGER   TYDOUB
      PARAMETER (TYDOUB = 7)
      INTEGER   TYCPLX
      PARAMETER (TYCPLX = 8)
      INTEGER   TYUSHT
      PARAMETER (TYUSHT = 11)
      INTEGER   TYUBYT
      PARAMETER (TYUBYT = 12)
*
*     TYTEXT is a special code for the iraf77 interface; it is not in the VOS
*
      INTEGER TYTEXT
      PARAMETER (TYTEXT = 13)
*========================end iraf77.inc=========================================
*=========================begin hsp.inc=========================================
*                                       --status return code
        INTEGER         OK, ERRNUM(20)
*                                       --message destination and priority
        INTEGER         DEST, PRIO
        DATA OK /0/
        DATA ERRNUM /701, 702, 703, 704, 705, 706, 707, 708, 709, 710,
     :               711, 712, 713, 714, 715, 716, 717, 718, 719, 720/
        DATA DEST, PRIO /1, 0/
*=========================end hsp.inc===========================================
*------------------------------------------------------------------------------
*
*  define output column names
*
        CNAME(1) = 'TYPE'
        CNAME(2) = 'DET_NUM'
        CNAME(3) = 'VOLTAGE'
        CNAME(4) = 'HVFACT'
        CNAME(5) = 'HVFACT_ERR'
        CNAME(6) = TNAME
        CNAME(7) = 'EPOCH'
        CNAME(8) = 'APER_NAME'
        CNAME(9) = 'TRGTNAME'
*
        NCOLS = 9
*
*  define output column data types, units, and display formats
*
        DTYPE(1) = -7
        DTYPE(2) = TYINT
        DO 10 I = 3, 6
            DTYPE(I) = TYREAL
   10   CONTINUE
        DTYPE(7) = TYDOUB
        DTYPE(8) = -10
        DTYPE(9) = -20
*
        DO 20 I = 1, NCOLS
            UNIT(I) = ' '
            COLFMT(I) = ' '
   20   CONTINUE
*
*  set up the output table
*
        CALL CDTOUT (OFILE, CNAME, UNIT, COLFMT, DTYPE, NCOLS,    TP, 
     :                  COLIDN, NROWS, STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'error setting up output table'
            GO TO 999
        END IF
*
*  write the result of each observation to the output table
*
        DO 40 I = 1, NPTS
*
*  specify output column buffer contents
*
            CBUFF(1) = HV(INDEX(I))
            CBUFF(2) = HVFAC(I)
            CBUFF(3) = DHVFAC(I)
            CBUFF(4) = TEMP(INDEX(I))
*
            CALL UTRPTT (TP, COLIDN(1), 1, NROWS+I, TYPE,   STAT(1))
            CALL UTRPTI (TP, COLIDN(2), 1, NROWS+I, DETID,   STAT(2))
            CALL UTRPTR (TP, COLIDN(3), 4, NROWS+I, CBUFF,   STAT(3))
            CALL UTRPTD (TP, COLIDN(7), 1, NROWS+I, EPOCH(INDEX(I)),
     :                   STAT(4))
            CALL UTRPTT (TP, COLIDN(8), 1, NROWS+I, APERT,
     :                   STAT(5))
            CALL UTRPTT (TP, COLIDN(9), 1, NROWS+I, TARGET,
     :                   STAT(6))
            NPUT = 6
*
            DO 30 J = 1, NPUT
                IF (STAT(J) .NE. OK) THEN
                    STATUS = ERRNUM(1)
                    CONTXT = 'cannot write a row to output table'
                    GO TO 999
                END IF
   30       CONTINUE
   40   CONTINUE
*
*  close the output table
*
        CALL UTTCLO (TP,        STATUS)
        IF (STATUS .NE. OK) THEN
            CONTXT = 'cannot close output table'
            GO TO 999
        END IF
*
        STATUS = OK
        GO TO 1000
*
*  write error message
*
  999   MESS = 'VHVFPT: ' // CONTXT
        CALL UMSPUT (MESS, DEST, PRIO,   STATOK)
*
 1000   RETURN
        END
