# MVREFER -- Move reference files between machines

procedure mvrefer (list)

string	list		  {prompt = "List of files to transfer"}
string	direction = "vs"  {enum = "vs|sv", prompt = "Direction to move files"}
bool	verbose = yes	  {prompt = "Print operations performed"}
struct	*lptr		  # list directed parameter

begin
	bool	verb		# verbose message string
	string	direct		# direction string
	string	type		# file type to be transferred
	string	dir1		# input directory
	string	dir2		# output directory
	string	pattern		# file name pattern
	string	temp		# temporary file name

	# Make sure tools package is loaded

	if (! defpac ("tools"))
	    error (1, "Tools package must be loaded before using task")

	# Check list to make sure it exists

	if (! access(list))
	    error (1, "List not found")

	# Set input parameters to temporary variables

	lptr = list
	direct = direction
	verb = verbose

	# Loop over each line in list

	while (fscan (lptr, type, dir1, dir2, pattern) != EOF) {

	    # Skip blank lines

	    if (nscan() == 0)
		next

	    # Create temporary file and put file names to be transferred in it

	    temp = mktemp ("tmp$ddf")
	    ddiff (dir1, dir2, pattern, maxdepth=1, abs+, >temp)

	    # Convert files and copy to their destination

	    if (type == "I") {
		if (direct == "vs") {
		    vax2sun ("@"+temp, dir2, verbose=verb)
		} else {
		    sun2vax ("@"+temp, dir2, verbose=verb)
		}
	    } else if (type == "T") {
		tconvert ("@"+temp, dir2, direct=direct, verbose=verb)
	    } else {
		print ("Unknown type ignored:", type)
	    }

	    # Delete list of file names

	    delete (temp)
	}

end
