include <imhdr.h>
include	<time.h>
define	SZ_HIST	SZ_TIME+4*SZ_FNAME+100		# size of the history log

#  FPHOT_DO -- Perform the filter photometry reduction
#
#  Description:
#  ------------
#
#  Date		Author			Description
#  ----		------			-----------
#  11-Apr-1986	D. Ball			Original module
#  04-May-1990  J.-C. Hsu		rewrite in SPP
#------------------------------------------------------------------------------

procedure fphot_do (fin1, finmask1, fin2, finmask2, fout, foutmask, 
				nfin1, nfin2, mask1, mask2, option, coeff, 
			  	datafill)

pointer	fin1, finmask1, fin2, finmask2, fout, foutmask 	
			# input: file template pointers
int	nfin1, nfin2	# input: number of files in the input template
bool	mask1, mask2	# input: mask file existence flags
int	option		# input: option of operation
real	coeff[3]	# input: transformation coefficients
real	datafill	# input: fill value for bad output data pixel
 
bool	mag1, mag2	# magnitude flags
pointer	ipin1, ipin2, ipinmask1, ipinmask2, ipout, ipoutmask
pointer	arrin1, arrin2, arrmask1, arrmask2, arrout, arroutmask
			# addresses of data
char	ifile1[SZ_FNAME], ifile2[SZ_FNAME], ofile[SZ_FNAME]
char	imask1[SZ_FNAME], imask2[SZ_FNAME], omask[SZ_FNAME]
char	bunit[SZ_BUNIT]
int	npix			# number of points of the input file
int	nchar, j
pointer ip, sp
char	text[SZ_HIST]
	
pointer	immap()
pointer	imgl1r()
pointer	impl1r() 
int	strmatch()
int	imtgetim()
long	clktime()
#==============================================================================
begin

	# loop all input files 
	do j = 1, nfin1 {

	    call smark (sp)

	    # read the next file name in the template list
	    nchar = imtgetim (fin1, ifile1, SZ_FNAME) 
            nchar = imtgetim (fout, ofile, SZ_FNAME) 
            nchar = imtgetim (foutmask, omask, SZ_FNAME) 

	    # open the first input file
	    ipin1 = immap (ifile1, READ_ONLY, 0)		
	    npix = IM_LEN(ipin1, 1)

	    # input file must be 1-D data and real data type
	    if (IM_NDIM(ipin1) != 1) 		
		call error (1, "first input file is not 1-D data")
	    if (IM_PIXTYPE(ipin1) != TY_REAL)
		call error (1, "first input file is not real data type")
	    arrin1 = imgl1r (ipin1)

	    # read bunit of the first file
	    call imgstr (ipin1, "BUNIT", bunit, SZ_BUNIT)
	    mag1 = (strmatch(bunit, "MAG") != 0)

	    # open second input file, if any.  check dimension, data type, and
	    # size
	    if (nfin2 > 0) {
                nchar = imtgetim (fin2, ifile2, SZ_FNAME) 
	        ipin2 = immap (ifile2, READ_ONLY, 0)		
		if (IM_LEN(ipin2, 1) != npix)
		    call error (1, "mismatch of file size: infile1 and infile2")
	        if (IM_NDIM(ipin2) != 1) 		
		    call error (1, "second input file is not 1-D data")
	    	if (IM_PIXTYPE(ipin2) != TY_REAL)
		    call error (1, "second input file is not real data type")
		arrin2 = imgl1r (ipin2)

	        # read bunit of the second file
	        call imgstr (ipin2, "BUNIT", bunit, SZ_BUNIT)
	    	mag2 = (strmatch(bunit, "MAG") != 0)
	    } else
		call salloc (arrin2, 1, TY_REAL)

	    # open first input mask, if any.  check dimension, data type, and
	    # size
	    if (mask1) {
                nchar = imtgetim (finmask1, imask1, SZ_FNAME) 
	        ipinmask1 = immap (imask1, READ_ONLY, 0)		
		if (IM_LEN(ipinmask1, 1) != npix)
		    call error (1, "mismatch of file size: infile1 and inmask1")
	        if (IM_NDIM(ipinmask1) != 1) 		
		    call error (1, "first input mask is not 1-D data")
	    	if (IM_PIXTYPE(ipinmask1) != TY_REAL)
		    call error (1, "first input mask is not real data type")
		arrmask1 = imgl1r (ipinmask1)
	    } else
		call salloc (arrmask1, 1, TY_REAL)

	    # open second input mask, if any.  check dimension, data type, and
	    # size
	    if (mask2) {
                nchar = imtgetim (finmask2, imask2, SZ_FNAME) 
	        ipinmask2 = immap (imask2, READ_ONLY, 0)		
		if (IM_LEN(ipinmask2, 1) != npix)
		    call error (1, "mismatch of file size: infile1 and inmask2")
	        if (IM_NDIM(ipinmask2) != 1) 		
		    call error (1, "first input mask is not 1-D data")
	    	if (IM_PIXTYPE(ipinmask2) != TY_REAL)
		    call error (1, "first input mask is not real data type")
		arrmask2 = imgl1r (ipinmask2)
	    } else
		call salloc (arrmask2, 1, TY_REAL)

	    # initialize the output file and mask
	    ipout = immap (ofile, NEW_FILE, 0)	
	    IM_LEN(ipout, 1) = npix
	    IM_PIXTYPE(ipout) = TY_REAL
	    ipoutmask = immap (omask, NEW_FILE, 0)	
	    IM_LEN(ipoutmask, 1) = npix
	    IM_PIXTYPE(ipoutmask) = TY_REAL

	    # allocate output array space
	    call malloc (arrout, npix, TY_REAL)
	    call malloc (arroutmask, npix, TY_REAL)

	    # calculate magnitudes or colors
	    call mag_color (Memr[arrin1], Memr[arrin2], Memr[arrmask1], 
			    Memr[arrmask2], Memr[arrout], Memr[arroutmask],
			    option, coeff, npix, nfin2, mask1, mask2, 
			    mag1, mag2, datafill)

	    # put the result to the output files and close them
	    ip = impl1r (ipout)
	    call amovr (Memr[arrout], Memr[ip], npix)
	    ip = impl1r (ipoutmask)
	    call amovr (Memr[arroutmask], Memr[ip], npix)

	    # update header keyword(s) and history
	    call imastr (ipout, "BUNIT", "MAG", SZ_BUNIT)

	    call cnvtime (clktime(0), text, SZ_HIST)
	    call strcat (" created by the task FILTERPHOT, from the file: ",
			 text, SZ_HIST)
	    call strcat (ifile1, text, SZ_HIST)
	    if (nfin2 > 0) {
	    	call strcat (", and the file: ", text, SZ_HIST)
	    	call strcat (ifile2, text, SZ_HIST)
	    }
	    call imputh (ipout, "HISTORY", text)
	    call imputh (ipoutmask, "HISTORY", text)

	    # close files
	    call imunmap (ipin1)
	    call imunmap (ipout)
	    call imunmap (ipoutmask)
	    if (nfin2 > 0)
		call imunmap (ipin2)
	    if (mask1)
		call imunmap (ipinmask1)
	    if (mask2)
		call imunmap (ipinmask2)

	    # print out message of which files been created
	    call printf ("filterphot: output file %s is created\n")
	    	call pargstr (ofile)
	    call printf ("filterphot: output mask %s is created\n")
	    	call pargstr (omask)

	    call mfree (arrout, TY_REAL)
	    call mfree (arroutmask, TY_REAL)
	    call sfree (sp)
	}

end
