include	"filterphot.h"
define	SZ_OPTION	8

#  FPHOT_GET -- Read parameters for the task filterphot.
#
#  Description:
#  ------------
#  Read CL parameters and do necessary checking and conversions.
#  
#  Input CL parameters:
#  -----------------
#
#  "infile1"		First input science data file template name
#  "inmask1"		First input science mask file template name (optional)
#  "infile2"		Second input science data file template name
#  "inmask2"		Second input science mask file template name (optional)
#  "outfile"		Output science data file template name
#  "outmask"		Output science mask file template name
#  "option"		option of operation
#  "coeff1"		First transformation coefficient
#  "coeff2"		Second transformation coefficient
#  "coeff3"		Third transformation coefficient
#  "datafill"		Fill value in the output data file for invalid pixel
#
#  Date		Author			Description
#  ----		------			-----------
#  04-May-1990  J.-C. Hsu		rewrite in SPP
#------------------------------------------------------------------------------

procedure fphot_get (fin1, finmask1, fin2, finmask2, fout, foutmask, 
				nfin1, nfin2, mask1, mask2, option, coeff, 
			  	datafill)

pointer	fin1, finmask1, fin2, finmask2, fout, foutmask	
				# output: file template pointers
int	nfin1, nfin2		# output: numbers of input files 
bool	mask1, mask2		# output: are there input mask files?
int	option			# output: operation to be performed
real	coeff[3]		# output: transformation coefficients
real	datafill		# output: fill value for the invalid pixels in
				#         the output data file

int	nfinmask1, nfinmask2	# number of input mask files
int	nfout
char	opt[SZ_OPTION]		# operation to be performed
	
pointer	imtopenp() 
int	imtlen(), strmatch()
real	clgetr()
#==============================================================================
begin

	# open file templates and find out how many files are in the templates
	fin1 = imtopenp ("infile1")
	finmask1 = imtopenp ("inmask1")
	fin2 = imtopenp ("infile2")
	finmask2 = imtopenp ("inmask2")
	fout = imtopenp ("outfile")
	foutmask = imtopenp ("outmask")

	# get the operation option
	call clgstr ("option", opt, SZ_OPTION)
	if (strmatch (opt, "instmag") != 0)
	    option = INSTMAG
	else if (strmatch (opt, "instcol") != 0)
	    option = INSTCOL
	else if (strmatch (opt, "stdmag") != 0)
	    option = STDMAG
	else if (strmatch (opt, "stdcol") != 0)
	    option = STDCOL
	else {
	    call eprintf ("illegal option: %s")
	    call pargstr (opt)
	    call error (1, "")
	}

	nfin1 = imtlen (fin1)
	nfin2 = imtlen (fin2)
	nfout = imtlen (fout)
	nfinmask1 = imtlen (finmask1)
	nfinmask2 = imtlen (finmask2)

	# for the first option, we don't need the second file
	if (option == INSTMAG) {
	    nfin2 = 0
	    nfinmask2 = 0
	}

	# check input parameters, the following files can not be empty
	if (nfin1 < 1)
	    call error (1, "blank first input file template")

	# if input mask template is not specified, set mask flag to false
	mask1 = (nfinmask1 > 0)
	mask2 = (nfinmask2 > 0)

	# check the numbers of files in each template are consistent/reasonable
	if (nfin1 != nfout)
	    call error (1, "mismatch no. of files: input and output")
	if (imtlen(foutmask) != nfout)
	    call error (1, "mismatch no. of files: output and output mask")
	if (mask1) {
	    if (nfinmask1 != nfin1)
	        call error (1, "mismatch no. of files: input and input mask")
	}

	# existence of the second input file
	if (nfin2 > 1) {
	    if (nfin2 != nfin1)
		call error (1, "mismatch no. of files: first and second input")
	    if (mask2) {
	    	if (nfinmask2 != nfin2)
	            call error (1, 
				  "mismatch no. of files: input and input mask")
	    }
	}
	    
	# read other CL parameters
	coeff[1] = clgetr ("coeff1")
	coeff[2] = clgetr ("coeff2")
	coeff[3] = clgetr ("coeff3")
        datafill = clgetr ("datafill")
end
