include	"filterphot.h"

#  MAG_COLOR -- Calculate magnitudes or colors 
#
#  Description:
#  ------------
#
#  Date		Author			Description
#  ----		------			-----------
#  11-Apr-1986	D. Ball			Original module
#  04-May-1990  J.-C. Hsu		rewrite in SPP
#------------------------------------------------------------------------------

procedure mag_color (input1, input2, inmask1, inmask2, output, outmask,
			option, coeff, npix, nfin2, mask1, mask2, mag1, mag2, 
			datafill)

real	input1[ARB], input2[ARB], inmask1[ARB], inmask2[ARB]
real	output[ARB], outmask[ARB]
			# input: data arrays
int	option		# input: option of operation
real	coeff[3]	# input: transformation coefficients
int	npix		# input: number of points of the input files
int	nfin2		# input: number of files in the input template
bool	mask1, mask2	# input: mask file existence flags
bool	mag1, mag2	# input: magnitude flags
real	datafill	# input: fill value for bad output data pixel

int	j
real	temp1, temp2	
#==============================================================================
begin

	switch (option) {

	# calculate instrumental magnitudes
	case INSTMAG:
	    do j = 1, npix {

		# if the input data mask is not OK, output mask is not OK either
		if (mask1) {
		    if (inmask1[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 

		# if the input is already in magnitude, only need to shift
		# the zero point
		if (mag1)
		    temp1 = input1[j]
		else {

		    # skip the data which have non-positive count rates
		    if (input1[j] <= 0.) {
			output[j] = datafill
			outmask[j] = DQM_BADVAL
			next
		    }
		    temp1 = -2.5 * log10(input1[j])
		}
		output[j] = temp1 + coeff[1]
		outmask[j] = DQM_OKVAL
	    }

	# calculate instrumental colors
	case INSTCOL:
	
	    # must have two input files
	    if (nfin2 < 1)
		call error (1, "instrumental color needs two input files")
	    do j = 1, npix {

		# if any the input data mask is not OK, output mask is 
		# not OK either
		if (mask1) { 
		    if (inmask1[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 
		if (mask2) {
		    if (inmask2[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 

		# if the input is already in magnitude, only need to shift
		# the zero point
		if (mag1)
		    temp1 = input1[j]
		else {

		    # skip the data which have non-positive count rates
		    if (input1[j] <= 0.) {
			output[j] = datafill
			outmask[j] = DQM_BADVAL
			next
		    }
		    temp1 = -2.5 * log10(input1[j])
		}
		if (mag2)
		    temp2 = input2[j]
		else {

		    # skip the data which have non-positive count rates
		    if (input2[j] <= 0.) {
			output[j] = datafill
			outmask[j] = DQM_BADVAL
			next
		    }
		    temp2 = -2.5 * log10(input2[j])
		}
		output[j] = temp1 - temp2 + coeff[1]
		outmask[j] = DQM_OKVAL
	    }

	# calculate standard magnitudes
	case STDMAG:
	
	    # second file must be colors (in magnitudes)
	    if (nfin2 > 0 && !(mag2))
		call error (1, 
		        "2nd file must be colors when calculating standard mag")
	    do j = 1, npix {

		# if any the input data mask is not OK, output mask is 
		# not OK either
		if (mask1) {
		    if (inmask1[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 
		if (mask2) {
		    if (inmask2[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 

		# if the input is already in magnitude, don't take the log
		if (mag1)
		    temp1 = input1[j]
		else {

		    # skip the data which have non-positive count rates
		    if (input1[j] <= 0.) {
			output[j] = datafill
			outmask[j] = DQM_BADVAL
			next
		    }
		    temp1 = -2.5 * log10(input1[j])
		}
		if (nfin2 > 0)
		    temp2 = input2[j]
		else
		    temp2 = 0.
		output[j] = coeff[1] + coeff[2] * temp1 + coeff[3] * temp2
		outmask[j] = DQM_OKVAL
	    }

	# calculate standard colors
	case STDCOL:
	
	    # both input files must be colors (in magnitudes)
	    if (!(mag1) || (nfin2 > 0 && !(mag2)))
		call error (1, 
		   "input files must be colors when calculating standard color")
	    do j = 1, npix {

		# if any the input data mask is not OK, output mask is 
		# not OK either
		if (mask1) {
		    if (inmask1[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 
		if (mask2) {
		    if (inmask2[j] != DQM_OKVAL) {
		    	output[j] = datafill
		    	outmask[j] = DQM_BADVAL
			next
		    }
		} 
		temp1 = input1[j]
		if (nfin2 > 0)
		    temp2 = input2[j]
		else
		    temp2 = 0.
		output[j] = coeff[1] + coeff[2] * temp1 + coeff[3] * temp2
		outmask[j] = DQM_OKVAL
	    }
	}
end
