# SORT_PHASE -- sort the phase array into ascending numerical order and at the
# same time make the corresponding rearrangement of the input array and the 
# input mask array.
#-----------------------------------------------------------------------------

procedure sort_phase (phase, file, mask, maskflag, npix)

double	phase[npix]		# input/output: phase array to be sorted
real	file[npix]		# input/output: data array
real	mask[npix]		# input/output: mask array
bool	maskflag		# input: is there mask file?
int	npix			# input: number of input points

pointer	ipindex, ipwork, ipworkd	# index array and work space array
pointer	sp
int	i
#============================================================================
begin
	call smark (sp)
	call salloc (ipindex, npix, TY_INT)
	call salloc (ipwork, npix, TY_REAL)
	call salloc (ipworkd, npix, TY_DOUBLE)

	# sort the phase array and get the index array at the same time
	call sortd_index (phase, Memi[ipindex], npix)

	# move data to a work area and then move the sorted array back to the 
	# data array, ditto for phase and mask array if there is any.
	call amovr (file, Memr[ipwork], npix)
	do i = 1, npix
	    file[i] = Memr[ipwork-1+Memi[ipindex-1+i]]

	call amovd (phase, Memd[ipworkd], npix)
	do i = 1, npix
	    phase[i] = Memd[ipworkd-1+Memi[ipindex-1+i]]

	if (maskflag) {
	    call amovr (mask, Memr[ipwork], npix)
	    do i = 1, npix
	    	mask[i] = Memr[ipwork-1+Memi[ipindex-1+i]]
	}

	call sfree (sp)
end
