        SUBROUTINE YCCOR(FLUX,NS,WIDTH,TEMPLT,TSIZE,XSTEPS,CHAN1,
     *                  NLINES,APPROX,EXACT,INTE)
*
* Module Number: 14.8.2.2
*
* Module Name: YCCOR
*
* Keyphrase:
* ----------
*       Find line locations by cross correlation
*
* Description:
* ------------
*       This routine finds spectral line positions with approximate
*       positions stored in approx.  Cross Correlation with a template
*       is used about a specified search width.
*
* Fortran Name: YCCOR.for
*
* Keywords of accessed files and tables:
* --------------------------------------
*       none
*
* Subroutines Called:
* -------------------
*       none
*
* History:
* --------
* version         date            Author          Description
*    1          3/20/87         D. Lindler      Designed and coded
*    2          Jan 87          D. Lindler      New sdas standards
*    3          Aug 88          D. Lindler      Added template
*-------------------------------------------------------------------------
C
C INPUT PARAMTERS
C
        DOUBLE PRECISION FLUX(1)
C                                    ---> FLUX ARRAY OF NS POINTS
        INTEGER NS
C                                    ---> NUMBER OF POINTS IN FLUX
        INTEGER WIDTH
C                                    ---> WIDTH OF SEARCH AREA
        DOUBLE PRECISION TEMPLT(1)
C                                    ---> INPUT TEMPLATE
        INTEGER TSIZE
C                                    ---> WIDTH OF CORRELATION TEMPLATE
        INTEGER XSTEPS
C                                    ---> NUMBER OF XSTEPS
        INTEGER CHAN1
C                                    ---> FIRST CHANNEL
        INTEGER NLINES
C                                    ---> NUMBER OF LINES TO SEARCH FOR
        DOUBLE PRECISION APPROX(1)
C                                    ---> ARRAY OF APPROXIMATE LINE POSITIONS
C
C OUTPUT PARAMTERS
C
        DOUBLE PRECISION EXACT(1)
C                                    ---> ARRAY OF FOUND POSITIONS
        DOUBLE PRECISION INTE(1)
C                                    ---> PEAK COUNTS IN LINE
C
C LOCAL VARIABLES
C
        INTEGER AREA
C                                    ---> WIDTH OF AREA OF FLUX VALUES NEEDED TO
C                                               SEARCH FOR A LINE
        INTEGER INTPOS
C                                    ---> APPROX POSITION CONVERTED TO INTEGER
        INTEGER ISTART
C                                    ---> STARTING POSITION IN FLUX ARRAY FOR SEARCH
        DOUBLE PRECISION RESULT(100)
C                                    ---> ARRAY OF CORRELATION RESULTS
        DOUBLE PRECISION SUMSQ
C                                    ---> SUM OF SQUARES OF DIFFERENCES
        DOUBLE PRECISION MAXVAL
C                                    ---> MAXIMUM VALUE IN FLUX ARRAY
        DOUBLE PRECISION MINVAL
C                                    ---> MINIMUM IN CORR. MATRIX
        INTEGER MINPOS
C                                    ---> POSITION OF MINIMUM
        INTEGER LINE,J,K,I
C                                    ---> INDICES
        DOUBLE PRECISION C1,C2,C3,XPOS,DIFF
        REAL    VAL
        INTEGER HFAREA,IEND,I1,I2
C----------------------------------------------------------
C
C COMPUTE SIZE OF AREA OF FLUX VALUES NEEDED TO FIND A LINE
C
        AREA=TSIZE+WIDTH-1
        HFAREA=AREA/2
C                                    ---> ONE HALF OF AREA
C
C PROCESS EACH LINE
C
        DO 200 LINE=1,NLINES
           EXACT(LINE)=0.0
C                                    --->MAX COUNT FOR LINE
           INTPOS=APPROX(LINE)*XSTEPS+1.5
C                                    ---> CHANGE TO INDEX IN FLUX
           ISTART=INTPOS-HFAREA
C                                    ---> STARTING POSITION IN FLUX ARRAY
           IEND=ISTART+AREA
C                                    ---> ENDING POSITION
C
C TEST IF LINE IS TOO CLOSE TO EDGE OF ARRAY
C
           IF( (ISTART.LT.1) .OR. (IEND.GT.NS) ) GO TO 100
C
C GET MAXIMUM OF FLUX ARRAY WITHIN SEARCH AREA (USED TO SCALE TEMPLT)
C
           MAXVAL=0.0
           I1=INTPOS-WIDTH/2
           I2=INTPOS+WIDTH/2
           DO 40 I=I1,I2
              IF( FLUX(I).GT.MAXVAL ) MAXVAL=FLUX(I)
40         CONTINUE
C
C COMPUTE CORRELATION MATRIX OF SIZE WIDTH
C
           DO 50 J=1,WIDTH
              SUMSQ=0.0
C                                    ---> INIT. SUM OF SQUARES
              DO 20 K=1,TSIZE
C                                    ---> ACCUM. SQUARES OVER TEMPLATE SIZE
                   VAL=FLUX(ISTART+J+K-2)
                   DIFF=MAXVAL*TEMPLT(K)-VAL
                   SUMSQ=SUMSQ+DIFF*DIFF
20            CONTINUE
              RESULT(J)=SUMSQ
C                                    ---> SAVE SUM OF SQUARES IN CORR. MAT.
50         CONTINUE
C
C FIND MINIMUM IN CORRELATION MATRIX
C
           MINPOS=0
           MINVAL=1.0E25
           DO 60 J=1,WIDTH
                  IF(RESULT(J).LT.MINVAL) THEN
                        MINVAL=RESULT(J)
                        MINPOS=J
                        ENDIF
60         CONTINUE
C
C CHECK IF FOUND POSITION IS ON EDGE OF CORRELATION MATRIX
C
           IF((MINPOS.EQ.1) .OR. (MINPOS.EQ.WIDTH)) GO TO 100
C
C REFINE POSITION WITH QUADRATIC FIT AT MINIMUM
C
           C1=RESULT(MINPOS-1)
           C2=RESULT(MINPOS)
           C3=RESULT(MINPOS+1)
           XPOS= (C1-C2)/(C1+C3-2*C2)-0.5 + MINPOS + TSIZE/2+ ISTART-1
C
C CHECK IF FOUND POSITION IS TOO FAR FROM APPROX POSITION
C
C
C NEW LINE POSITION FOUND
C
           XPOS=(XPOS-1.0)/XSTEPS
C                                    ---> CHANGE TO SAMPLE NUMBER
           EXACT(LINE)=XPOS
C
100        INTE(LINE)=MAXVAL
200     CONTINUE
C
        RETURN
        END
