# darksum -- Produce tables and plots to summary dark calibration observations.

procedure darksum (input)

file    input	        {prompt="Observation rootnames to be summarized"}

file    summary_dir     {"",prompt="Directory to place summary tables and files"}
file    group_stats	{"group_stats.tab",prompt="Statistics of individual groups"}
file    obs_stats       {"obs_stats.tab",prompt="Statistics of observations"}
file    total_stats     {"total_stats.tab",prompt="Statistics of related observations"}
file    events          {"events.tab",prompt="Table containing particle hits"}
file    noisy           {"noisy.tab",prompt="Table containing noisy diodes"}
file    dark            {"dark.tab",prompt="Table of average dark spectra"}
real    pthresh         {indef,min=0.,prompt="Particle event threshold"}
real    pnsig           {4.,min=0.,prompt="Number of sigmas above average for hit"}
real    dthresh         {indef,min=0.,prompt="Noisy diode threshold"}
real    dnsig           {4.,min=0.,prompt="Number of sigmas above average for noisy"}
bool    plotpos         {yes,prompt="Plot HST position at time of observation?"}
string  device          {"stdgraph",prompt="Device to plot to"}
string  marker          {"box",prompt="Marker type (see help for hstpos)"}
real    szmarker        {0.01,min=0.,max=1.,prompt="Marker size in NDC"}
bool    append          {no,prompt="Append to a previous plot?"}
bool    verbose         {no,prompt="Describe the progress of the script?"}

struct  *listptr        {mode="h"}

begin
        
        # Declarations
        int     i
        int     n_related               # Number of related groups.
        int     row                     # Current row of the related table.

        string  full_dark               # Directory + dark table.
        string  full_events             # Directory + event table.
        string  full_noisy              # Directory + noisy table.
        string  full_obs_stats          # Directory + name of obs_stats.
        string  full_group_stats        # Directory + name of group_stats.
        string  full_total_stats        # Directory + name of total_stats.
        string  prootname               # Rootname of data to examine.
        string  obs_dir                 # Directory of observations.
        string  obs_ext                 # Extension of observations.
        string  related                 # Related filename.
        string  time_key                # Time header keyword.
        string  tmp_cdfile              # Temporary column description file.
        string  tmp_obs_pos             # Table contain positions per obs.
        string  tmp_obs_stats           # Table contain stats per obs.
        string  tmp_group_pos            # Table contain positions per group.
        string  tmp_group_stats          # table contain stats per group.
        string  tmp_string_1            # Temporary string.
        string  tmp_string_2            # Temporary string.
        string  tmp_table_1             # Temporary table.
        string  tmp_table_2             # temporary table.
        string  tmp_table_3             # Temporary table.
        string  tmp_total_pos           # Table contain pos per related.
        string  tmp_total_stats         # Table contain stats per related.

        # Make sure the necessary packages are loaded.
        tmp_string_1 = ""
	if (!defpac ("images"))
	    tmp_string_1 = tmp_string_1//" images"
	if (!defpac ("tools"))
	    tmp_string_1 = tmp_string_1//" tools"
	if (!defpac ("z_calib"))
	    tmp_string_1 = tmp_string_1//" z_calib"
	if (!defpac ("playpen"))
	    tmp_string_1 = tmp_string_1//" playpen"
	if (!defpac ("stplot"))
	    tmp_string_1 = tmp_string_1//" stplot"
	if (!defpac ("ttools"))
	    tmp_string_1 = tmp_string_1//" ttools"
        if (strlen (tmp_string_1) > 0)
            error (1, "Please load packages:"//tmp_string_1)

        # Make sure the major parameters are read.
        prootname = input
        full_group_stats = summary_dir//group_stats
        full_obs_stats = summary_dir//obs_stats
        full_total_stats = summary_dir//total_stats
        full_events = summary_dir//events
        full_noisy = summary_dir//noisy
        full_dark = summary_dir//dark
        
        # Define the temporary files.
        tmp_cdfile      = "Ds-tmp-cdfile.txt"
        tmp_obs_pos     = "Ds-tmp-obs-pos.tab"
        tmp_obs_stats   = "Ds-tmp-obs-stats.tab"
        tmp_group_pos    = "Ds-tmp-group-pos.tab"
        tmp_group_stats  = "Ds-tmp-group-stats.tab"
        tmp_table_1     = "Ds-tmp-table-1.tab"
        tmp_table_2     = "Ds-tmp-table-2.tab"
        tmp_table_3     = "Ds-tmp-table-3.tab"
        tmp_total_pos   = "Ds-tmp-total-pos.tab"
        tmp_total_stats = "Ds-tmp-total-stats.tab"

        # Determine whether the data set is an old style headers or
        # new style headers.
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        files (prootname, sort=yes, > tmp_table_1)
        listptr = tmp_table_1
        i = fscan (listptr, tmp_string_1)
        listptr=""
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        hedit (tmp_string_1, "expstart", ".", show=yes, > tmp_table_1)
        tmp_string_2 = ""
        listptr = tmp_table_1
        i = fscan (listptr, tmp_string_2)
        listptr=""
        if (strlen (tmp_string_2) > 0) {
            time_key = "expstart"
        } else {
            time_key = "pkttime"
            print ("darksum: Warning! Old data headers - some time fields will be blank")
        }

        # Using the filename above, get directory and extension information.
        fparse (tmp_string_1, verbose=no)
        obs_dir = fparse.directory
        obs_ext = fparse.extension
        
        # Calculate the statistics.
        if (verbose)
            print ("darksum: calculating statistics")
        delete (tmp_group_stats, verify=no, allversions=yes, >& "dev$null")
        delete (tmp_obs_stats, verify=no, allversions=yes, >& "dev$null")
        delete (tmp_total_stats, verify=no, allversions=yes, >& "dev$null")
        darkstat (prootname, tmp_group_stats, tmp_obs_stats, tmp_total_stats,
                  full_events, full_noisy, full_dark, verbose=yes, 
                  pthresh=pthresh, pnsig=pnsig, dthresh=dthresh,
                  dnsig=dnsig, dnum=indef)
	
        # Produce a table with the observation positions in it.
        if (verbose)
            print ("darksum: finding observation positions.")
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        delete (tmp_group_pos, verify=no, allversions=yes, >& "dev$null")
        hstpos (prootname, orbit="", timekey=time_key,
                position=tmp_group_pos, plot=no, allgroups=yes)

        # Create a table with the DATE-OBS and TIME-OBS keywords.
        delete (tmp_cdfile, verify=no, allversions=yes, >& "dev$null")
        delete (tmp_table_3, verify=no, allversion=yes, >& "dev$null")
        delete (tmp_table_2, verify=no, allversion=yes, >& "dev$null")
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        files (prootname, sort=yes, > tmp_table_3)
        listptr = tmp_table_3
        while (fscan (listptr, tmp_string_1) != EOF)
            grlist (tmp_string_1, "-", >> tmp_table_1)
        print ("TIME-OBS CH*8", > tmp_cdfile)
        print ("DATE-OBS CH*8", >> tmp_cdfile)
        keyselect ("@"//tmp_table_1, tmp_table_2,
                   "TIME-OBS=TIME-OBS DATE-OBS=DATE-OBS", expr="",
                   cdfile=tmp_cdfile, >& "dev$null")

        # Complete the position table.
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        tmerge (tmp_group_pos//","//tmp_table_2,
                tmp_table_1, option="merge", allcols=yes, tbltype="row",
                extracol=0)
        delete (tmp_group_pos, verify=no, allversions=yes, >& "dev$null")
        rename (tmp_table_1, tmp_group_pos, field="root")

        # Create the finished group summary.
        delete (tmp_table_1, verify=no, allversions=yes, >& "dev$null")
        delete (tmp_table_2, verify=no, allversions=yes, >& "dev$null")
        tproject (tmp_group_pos, tmp_table_2,
                  "TIME,LONGITUDE,LATITUDE,DATE-OBS,TIME-OBS", uniq=no)
        tmerge (tmp_group_stats//","//tmp_table_2,
                tmp_table_1, option="merge", allcols=yes, tbltype="row",
                extracol=0)
        delete (tmp_group_stats, verify=no, allversions=yes, >& "dev$null")
        rename (tmp_table_1, tmp_group_stats, field="root")

        # Now loop through all the related groups, producing plots, printing
        # summaries, and creating the obs_stats and total_stats files.
        tinfo (tmp_total_stats, ttout=no)
        n_related = tinfo.nrows
        for (row = 1; row <= n_related; row = row + 1) {

            # Get the related observation name.
            tabpar (tmp_total_stats, "ROOTNAME", row)
            related = tabpar.value

            # If plotting positions, do it now.
            if (plotpos) {
                if (verbose)
                    print ("darksum: plotting the positions for observations ", related)
                
                delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
                delete (tmp_table_2, verify=no,allversions=yes,>&"dev$null")
                tselect (tmp_obs_stats, tmp_table_1,
                         "rootname ?= '"//related//"?*'")
                tinfo (tmp_table_1, ttout=no)
                for (i=1; i <= tinfo.nrows; i = i +1) {
                    tabpar (tmp_table_1, "ROOTNAME", i)
                    print (obs_dir//tabpar.value//obs_ext, >>tmp_table_2)
                }
                hstpos ("@"//tmp_table_2, orbit="", timekey=time_key,
                        position="", model=7,
                        plot=yes, allgroups=no, device=device, marker=marker,
                        szmarker=szmarker, append=append, geomag=yes,
                        n_latitudes=2, latint=20.)

                # Get the proposal id.
                tabpar (tmp_table_1, "ROOTNAME", 1)
                keypar (obs_dir//tabpar.value//obs_ext, "PROPOSID")

                # Make the title.
                tmp_string_1 = "Proposal ID: "//keypar.value//" Observations: "//related
                delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
                print ("location 0 1 0 1; limits 0 1 0 1; lweight 1.5",
                       > tmp_table_1)
                print ("justify 5; move .5 .9; expand 1.5", >> tmp_table_1)
                print ("label \"\\\\i "//tmp_string_1//"\"", >>tmp_table_1)
                igi (device=device, metacode="", append=yes, debug=no,
                     < tmp_table_1)
                gflush
                
            }

            # Extract just this related set from the group stats.
            delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
            tselect (tmp_group_stats, tmp_table_1,
                     expr="rootname ?= '"//related//"*'")
            
            # Print the statistics for just this related set.
            if (verbose)
                print ("darksum: creating text file of group statistics of ", related)
            tmp_string_1 = summary_dir//"summary_"//related//".txt"
            tprint (tmp_table_1, prparam=no, prdata=yes, pwidth=132,
                    plength=0, showrow=no, showhdr=yes,
                    columns="ROOTNAME,MEAN,SIGMA,MAX,TIME,LONGITUDE,LATITUDE,TIME-OBS,DATE-OBS",
                    rows="-", option="plain", align=yes, sp_col="",
                    lgroup=0, > tmp_string_1)
            
            # Extract just the observation positions from all the positions.
            delete (tmp_table_2, verify=no, allversions=yes, >& "dev$null")
            tselect (tmp_group_pos, tmp_table_2,
                     expr="rootname ?= '"//related//"[a-zA-Z0-9.]*\[1\]$'")
            
            # Extract just a position for the whole related group.
            delete (tmp_table_3, verify=no, allversions=yes, >& "dev$null")
            tselect (tmp_table_2, tmp_table_3, expr="row() == 1")
            
            # If this is the first time through, just rename the temporary
            # tables, else merge them.
            if (row == 1) {
                delete (tmp_obs_pos,verify=no,allversions=yes, >& "dev$null")
                delete (tmp_total_pos,verify=no,allversions=yes, >&"dev$null")
                rename (tmp_table_2, tmp_obs_pos, field="root")
                rename (tmp_table_3, tmp_total_pos, field="root")
            } else {
                delete (tmp_table_1, verify=no, allversions=yes, >&"dev$null")
                tmerge (tmp_obs_pos//","//tmp_table_2, tmp_table_1,
                        option="append", allcols=yes, tbltype="row",
                         extracol=0)
                delete (tmp_obs_pos,verify=no,allversions=yes, >& "dev$null")
                rename (tmp_table_1, tmp_obs_pos, field="root")
                
                tmerge (tmp_total_pos//","//tmp_table_3, tmp_table_1,
                        option="append", allcols=yes, tbltype="row",
                         extracol=0)
                delete (tmp_total_pos,verify=no,allversions=yes, >&"dev$null")
                rename (tmp_table_1, tmp_total_pos, field="root")
            }
        }

        # Append/create the accumulative group statistics table.
        if (access (full_group_stats)) {
            if (verbose)
                print ("darksum: appending to the group_stats table")
            delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
            tmerge (full_group_stats//","//tmp_group_stats, tmp_table_1,
                    option="append", allcols=yes, tbltype="row", 
                    extracol=0)
            delete (full_group_stats, verify=no,allversions=yes,>&"dev$null")
            rename (tmp_table_1, full_group_stats, field="root")
        } else {
            if (verbose)
                print ("darksum: creating new group_stats table")
            rename (tmp_group_stats, full_group_stats, field="root")
        }

        # Create the final observation statistics table.
        delete (tmp_table_2, verify=no,allversions=yes,>&"dev$null")
        delete (tmp_table_3, verify=no,allversions=yes,>&"dev$null")
        tproject (tmp_obs_pos, tmp_table_2,
                  "TIME,LONGITUDE,LATITUDE,DATE-OBS,TIME-OBS", uniq=no)
        tmerge (tmp_obs_stats//","//tmp_table_2, tmp_table_3,
                option="merge", allcols=yes, tbltype="row", extracol=0)
        if (access (full_obs_stats)) {
            if (verbose)
                print ("darksum: appending to the obs_stats table")
            delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
            tmerge (full_obs_stats//","//tmp_table_3, tmp_table_1,
                    option="append", allcols=yes, tbltype="row", allrow=100,
                    extracol=0)
            delete (full_obs_stats, verify=no,allversions=yes,>&"dev$null")
            rename (tmp_table_1, full_obs_stats, field="root")
        } else {
            if (verbose)
                print ("darksum: creating new obs_stats table")
            rename (tmp_table_3, full_obs_stats, field="root")
        }
	
        # Now append/create the total statistics table.
        delete (tmp_table_2, verify=no,allversions=yes,>&"dev$null")
        delete (tmp_table_3, verify=no,allversions=yes,>&"dev$null")
        tproject (tmp_total_pos, tmp_table_2,
                  "TIME,LONGITUDE,LATITUDE,DATE-OBS,TIME-OBS", uniq=no)
        tmerge (tmp_total_stats//","//tmp_table_2, tmp_table_3,
                option="merge", allcols=yes, tbltype="row", extracol=0)
        if (access (full_total_stats)) {
            if (verbose)
                print ("darksum: appending to the total_stats table")
            delete (tmp_table_1, verify=no,allversions=yes,>&"dev$null")
            tmerge (full_total_stats//","//tmp_table_3, tmp_table_1,
                    option="append", allcols=yes, tbltype="row",
		    extracol=0)
            delete (full_total_stats, verify=no,allversions=yes, >&"dev$null")
            rename (tmp_table_1, full_total_stats, field="root")
        } else {
            if (verbose)
                print ("darksum: creating new total_stats table")
            rename (tmp_table_3, full_total_stats, field="root")
        }

        # Delete any extraneous temporary files.
        delete ("Ds-*", verify=no, allversions=yes, >& "dev$null")
        
end
