        SUBROUTINE ZDTFNC(X,T,Y,DYDT,MA)
*
*  Module number:  13.6.2
*
*  Module name: ZDTFNC
*
*  Keyphrase:
*  ----------
*       Compute deadtime function
*  Description:
*  ------------
*       This routine computes the deadtime function and its
*       derivative with respect to the time constant.
*       The deadtime function used is:
*               y = (1 - exp(-tx))/t
*
*  FORTRAN name: zdtfnc
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*       none
*
*  Subroutines Called:
*  -------------------
*       none
*
*  History:
*  --------
*  Version      Date        Author          Description
*     1       Sept 88   D. Lindler      Designed and coded
*-------------------------------------------------------------------------------
*
* INPUT PARAMETERS
*
*       X - expected count rate (real*8)
*       T - deadtime constant (real*8)
*       MA - number of coef. in the equation (must be set to 1) (integer)
*
* OUTPUT PARAMETERS
*
*       Y - observed count rate
*       DYDT - direvative of Y w.r.t. T
*
*-----------------------------------------------------------------------------
        DOUBLE PRECISION X,Y,T,DYDT
        INTEGER MA
C
C LOCAL VARIABLES
C
        DOUBLE PRECISION E
C
C COMPUTE Y
C
        E = DEXP(-T*X)
        Y = (1-E)/T
C
C COMPUTE DERIVATIVE
C
        DYDT = (X*E - Y)/T
        RETURN
        END
