#---------------------------------------------------------------------------
# pha_diode -- Produce summary graphs for all diodes.
#---------------------------------------------------------------------------
procedure pha_all

file    difphd          {"",prompt="Differential pulse height distribution image"}
file    fitphd          {"",prompt="Fitted pulse height distribution image"}
file    table           {"",prompt="Threshold statistics table"}
real    t0              {0,prompt="Initial threshold level in input"}
real    delt            {0,prompt="Step size in theshold levels in input"}
string  device          {"stdgraph",prompt="Graphics output device"}
real    left            {0.,min=0.,max=1.,prompt="Left edge of viewport"}
real    right           {1.,min=0.,max=1.,prompt="Right edge of viewport"}
real    bottom          {0.,min=0.,max=1.,prompt="Bottom edge of viewport"}
real    top             {1.,min=0.,max=1.,prompt="Top edge of viewport"}
bool    verbose         {yes,prompt="Write out status?"}

string  Version         {"6Aug93",prompt="Date of Installation"}

begin
        # Declarations
        real    pane_max_x      # Maximum extent of x panes.
        real    pane_max_y      # Maximum extend of y panes.
        real    pane_off_x      # Initial start of x panes.
        real    pane_off_y      # Initial start of y panes.
        real    pane_size_x     # Size of a pane in x direction.
        real    pane_size_y     # Size of a pane in y direction.
        real    x, y            # Generic.
        
        int     detector        # Detector.
        int     diode           # Current diode number working on.
        int     group           # Which group we are in.
        int     i, j            # Generic.
        int     n_diodes        # Total number of diodes to plot.
        int     n_thresh        # Number of thresholds.
        int     n_xpane         # Number of graphs in x direction.
        int     n_ypane         # Number of graphs in y direction.
        int     xpane           # Current x pane.
        int     ypane           # Current y pane.
        
        file    igi_script      # The igi script to execute.
        file    thresholds      # List of thresholds.
        file    tmp_table_1     # Table sorted on diodes.
        
        string  pdiodes         # The diodes to print up.
        string  rootname        # Rootname of observation.
        string  ts1             # Generic

        # Define how many graphs appear on a page and the extent
        n_xpane = 4
        n_ypane = 4
        pane_off_x = .05
        pane_off_y = .03
        pane_max_x = .95
        pane_max_y = .9
        pane_size_x = (pane_max_x - pane_off_x) / n_xpane
        pane_size_y = (pane_max_y - pane_off_y) / n_ypane
        
        # Set temporary file names
        igi_script = mktemp ("tmp$pha_igi")
        thresholds = mktemp ("tmp$pha_thresh")
        tmp_table_1 = mktemp ("tmp$pha_table")//".tab"
        
        # Make sure the correct packages are loaded.
        ts1 = ""
	if (!defpac ("z_calib"))
	    ts1 = ts1//"z_calib "
        if (!defpac ("stplot"))
            ts1 = ts1//"stplot "
        if (!defpac ("ttools"))
            ts1 = ts1//"ttools "
	if (!defpac ("images"))
	    ts1 = ts1//"images "
        if (strlen (ts1) > 0)
            error (1, "Please load packages: "//ts1)

        # Take values from the phaz task if not defined here.
        if (verbose)
            print ("pha_all: Retrieving parameters")
        if (strlen (difphd) == 0) {
            ts1 = phaz.difphd.p_mode
            phaz.difphd.p_mode = "h"
            difphd = phaz.difphd
            phaz.difphd.p_mode = ts1
        }
        if (strlen (fitphd) == 0) {
            ts1 = phaz.fitphd.p_mode
            phaz.fitphd.p_mode = "h"
            fitphd = phaz.fitphd
            phaz.fitphd.p_mode = ts1
        }
        if (strlen (table) == 0) {
            ts1 = phaz.table.p_mode
            phaz.table.p_mode = "h"
            table = phaz.table
            phaz.table.p_mode = ts1
        }
        if (t0 == 0.0) {
            ts1 = phaz.t0.p_mode
            phaz.t0.p_mode = "h"
            t0 = phaz.t0
            phaz.t0.p_mode = ts1
        }
        if (delt == 0.0) {
            ts1 = phaz.delt.p_mode
            phaz.delt.p_mode = "h"
            delt = phaz.delt
            phaz.delt.p_mode = ts1
        }

        # Get the detector number, rootname
        keypar (difphd, "detector")
        detector = int (keypar.value)
        keypar (difphd, "rootname")
        rootname = keypar.value

        # Create a data file with the list of threshold values.
        if (verbose)
            print ("pha_all: Creating threshold data")
        delete (thresholds, go_ahead=yes, verify=no, >& "dev$null")
        keypar (difphd, "i_naxis1")
        n_thresh = int (keypar.value)
        for ( i = 0; i < n_thresh; i = i + 1 )
            print ((i*delt)+t0, >> thresholds)

        # Sort the table on diodes.
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")
        tcopy (table, tmp_table_1, verbose=no)
        tsort (tmp_table_1, columns="DIODE", ascend=yes, casesens=yes)

        # Plot up all the diode information.
        keypar (difphd, "i_naxis2")
        n_diodes = int (keypar.value)
        xpane = 0
        ypane = 0
        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        for (diode = 1; diode <= n_diodes; diode = diode + 1) {
            if (verbose)
                print ("pha_all: Diode "//diode)

            # Increase pane count.
            xpane = xpane + 1
            if (xpane > n_xpane) {
                ypane = ypane - 1
                xpane = 1
            }

            # If done with a page, flush it out and start again.
            if (ypane == 0 ) {
                if (verbose)
                    print ("pha_all: Starting new page")

                # Start new pane count.
                xpane = 1
                ypane = n_ypane

                # If an igi script exists already, send it out and
		# start another.
                if (access (igi_script)) {
                    if (verbose)
                        print ("pha_all: Executing igi script "//igi_script)
                    print ("end", >> igi_script)
                    flprcache
                    igi (initcmd="", device=device, metacode="", append=no,
                         debug=no,cursor="", < igi_script)

                }
                
                # Start a new script.
                delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
                print ("data "//thresholds//"; xcolumn 1", >> igi_script)
                print ("lweight 2.0", >> igi_script)
                print ("justify 5", >> igi_script)
                print ("location .1 .95 .1 .95", >> igi_script)
                print ("ptype 1 3", >> igi_script)
                print ("expand 1.5", >> igi_script)
                print ("vmove .5 .95", >> igi_script)
                print ("label ID = "//rootname//" Detector = "//detector,
                       >> igi_script)
                j = t0
                i = t0 + (delt * (n_thresh - 1))
                print ("lweight 1.0; expand .7; vmove .5 .03", >> igi_script )
                print ("label Thresholds from "//j//" to "//i, >> igi_script)
                
                # Macro to plot differential graph.
                print ("define difplot", >> igi_script)
                print ("limits 0 1 0 1", >> igi_script)
                print ("move 0 0", >> igi_script)
                print ("draw 0 1", >> igi_script)
                print ("draw 1 1", >> igi_script)
                print ("draw 1 0", >> igi_script)
                print ("draw 0 0", >> igi_script)
                print ("limits", >> igi_script)
                print ("points", >> igi_script)
                print ("end", >> igi_script)
                
                # Plot the fit graph overlay and threshold.
                print ("define fitplot", >> igi_script)
                print ("lweight 1.5", >> igi_script)
                print ("connect", >> igi_script)
                print ("ltype 2", >> igi_script)
                print ("lweight 1.", >> igi_script)
                print ("move &1 &2", >> igi_script)
                print ("draw &1 &3", >> igi_script)
                print ("ltype 0", >> igi_script)
                print ("limits 0 1 0 1", >> igi_script)
                print ("expand .6", >> igi_script)
                print ("move .9 .9", >> igi_script)
                print ("label &4", >> igi_script)
                print ("expand .5", >> igi_script)
                print ("move .07 1.05", >> igi_script)
                print ("label &3", >> igi_script)
                print ("expand 1.", >> igi_script)
                print ("end", >> igi_script)
                
            }

            # Decide on which pane this one will go in.
            x = pane_off_x + ((xpane - 1) * pane_size_x)
            y = pane_off_y + ((ypane - 1) * pane_size_y)
            print ("vpage "//x//" "//x+pane_size_x//" "//y//" "//y+pane_size_y,
                   >> igi_script)

            # Draw the differential graph.
            print ("ysection "//difphd//"[*,"//diode//"]", >> igi_script )
            print ("difplot", >> igi_script)

            # Draw the fit and threshold overlay.
            print ("ysection "//fitphd//"[*,"//diode//"]", >> igi_script )
            minmax (difphd//"[*,"//diode//"]", force=yes, update=no, verbose=no)
            tabpar (tmp_table_1, "THRESH", diode)
            ts1 = "fitplot "//int (tabpar.value)//" "//int(minmax.minval)
            ts1 = ts1//" "//int(minmax.maxval)//" "//diode
            print (ts1, >> igi_script)
        }

        # Make sure the script is out.
        if (verbose)
            print ("pha_all: Executing "//igi_script)
        print ("end", >> igi_script)
        flprcache
        igi (initcmd="", device=device, metacode="", append=no,
             debug=no,cursor="", < igi_script, >& "dev$null")

        # That's all folks.
        if (verbose)
            print ("pha_all: That's all folks")
        gflush
        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        delete (thresholds, go_ahead=yes, verify=no, >& "dev$null")
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")

end
#---------------------------------------------------------------------------
# End of pha_all
#---------------------------------------------------------------------------
