#---------------------------------------------------------------------------
# pha_center -- Graph histogram of all PHA centers.
#---------------------------------------------------------------------------
procedure pha_center

file    table           {prompt="PHA results table"}
string  device          {"stdgraph",prompt="Graphics output device"}
real    left            {0.,min=0.,max=1.,prompt="Left edge of viewport"}
real    right           {1.,min=0.,max=1.,prompt="Right edge of viewport"}
real    bottom          {0.,min=0.,max=1.,prompt="Bottom edge of viewport"}
real    top             {1.,min=0.,max=1.,prompt="Top edge of viewport"}
bool    verbose         {no,prompt="Print status messages?"}

string  Version         {"11Dec92",prompt="Date of Installation"}

begin
        # Declarations
        string  title           # Title of graph
        string  ts1             # Generic.

        file    igi_script      # Igi script.
        file    tmp_table_1     # Temporary histogram table.

        # Set temporary file names.
        igi_script = mktemp ("tmp$pha_igi")
        tmp_table_1 = mktemp("tmp$pha_table_1")//".tab"

        # Make sure the correct packages are loaded.
        ts1 = ""
	if (!defpac ("z_calib"))
	    ts1 = ts1//"z_calib "
        if (!defpac ("stplot"))
            ts1 = ts1//"stplot "
        if (!defpac ("ttools"))
            ts1 = ts1//"ttools "
        if (strlen (ts1) > 0)
            error (1, "Please load packages: "//ts1)

        # Retrieve the parameters.
        if (table == "") {
            ts1 = phaz.table.p_mode
            phaz.table.p_mode = "h"
            table = phaz.table
            phaz.table.p_mode = ts1
        }
        if (verbose)
            print ("table = "//table)

        # Get header paramters.
        keypar (table, "rootname")
        title = "ID = "//keypar.value
        keypar (table, "detector")
        title = title//" Detector = "//keypar.value
        
        # Now produce the histogram.
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")
        thistogram (table, tmp_table_1, "CENTER", nbins=100, lowval=60.,
                    highval = 110., dx=INDEF, clow=INDEF, chigh=INDEF,
		    rows="-", outcolx="center", outcoly="number")
        
        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        print ("justify 5; lweight 1.5; expand 1.5; limits 0 1 0 1",
               >> igi_script)
        print ("move .5 1.", >> igi_script)
        print ("label SIGNAL PEAK (Center of Gaussian)", >> igi_script)
        print ("move .5 .95", >> igi_script)
        print ("label "//title, >> igi_script)
        print ("location .1 .9 .1 .88", >> igi_script)
        print ("data "//tmp_table_1, >> igi_script)
        print ("expand 1.0; xcolumn center; ycolumn number; limits",
               >> igi_script)
        print ("margin; box; step", >> igi_script )
        print ("xlabel Threshold Units", >> igi_script)
        print ("ylabel Number", >> igi_script)
        print ("end", >> igi_script)
        igi (initcmd="", device=device, metacode="", append=no,
             debug=no,cursor="", < igi_script)

        # That's all folks.
        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")

end
#---------------------------------------------------------------------------
# End of pha_center
#---------------------------------------------------------------------------
