#---------------------------------------------------------------------------
# pha_sig2rms -- Produce histogram of signal to noise ratios of PHA data.
#---------------------------------------------------------------------------
procedure pha_rms

file    table           {prompt="PHA results table"}
string  device          {"stdgraph",prompt="Graphics output device"}
real    left            {0.,min=0.,max=1.,prompt="Left edge of viewport"}
real    right           {1.,min=0.,max=1.,prompt="Right edge of viewport"}
real    bottom          {0.,min=0.,max=1.,prompt="Bottom edge of viewport"}
real    top             {1.,min=0.,max=1.,prompt="Top edge of viewport"}
bool    verbose         {no,prompt="Write out status?"}

string  Version         {"11Dec92",prompt="Date of Installation"}

begin
        # Declarations
        file    igi_script      # Igi script.
        file    sig2rms         # Signal to Noise ratio table.
        file    tmp_table_1     # Temporary histogram table.

        string  title           # Title of graph.
        string  ts1             # Generic.

        # Set temporary file names.
        igi_script = mktemp ("tmp$pha_igi")
        sig2rms = mktemp ("tmp$pha_sig2rms")
        tmp_table_1 = mktemp("tmp$pha_table_1")//".tab"

        # Make sure the correct packages are loaded.
        ts1 = ""
        if (!defpac ("stplot"))
            ts1 = ts1//"stplot "
        if (!defpac ("ttools"))
            ts1 = ts1//"ttools "
	if (!defpac ("z_calib"))
	    ts1 = ts1//"z_calib "
        if (strlen (ts1) > 0)
            error (1, "Please load packages: "//ts1)

        # Retrieve the parameters.
        if (table == "") {
            ts1 = phaz.table.p_mode
            phaz.table.p_mode = "h"
            table = phaz.table
            phaz.table.p_mode = ts1
        }
        if (verbose)
            print ("table = "//table)

        # Get the title.
        keypar (table, "rootname")
        title = "ID = "//keypar.value
        keypar (table, "detector")
        title = title//" Detector = "//keypar.value
        
        # Calculate the signal to noise ratio.
        delete (sig2rms, go_ahead=yes, verify=no, >& "dev$null")
        tcopy (table, sig2rms, verbose=verbose)
        tcalc (sig2rms, "SIG2RMS", "CENTER / SIGMA", datatype="real")

        # Now produce the histogram.
        thistogram (sig2rms, tmp_table_1, "SIG2RMS", nbins=100, lowval=6.,
                    highval=22., rows="-", dx=INDEF, clow=INDEF,
		    chigh=INDEF, outcolx="sig2rms", outcoly="number")

        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        print ("justify 5; lweight 1.5; expand 1.5; limits 0 1 0 1",
               >> igi_script)
        print ("move .5 1.", >> igi_script)
        print ("label SIG/NOISE (Gaussian center/sigma)", >> igi_script)
        print ("move .5 .95", >> igi_script)
        print ("label "//title, >> igi_script)
        print ("location .1 .9 .1 .88", >> igi_script)
        print ("data "//tmp_table_1, >> igi_script)
        print ("expand 1.0; xcolumn sig2rms; ycolumn number; limits",
               >> igi_script)
        print ("margin; box; step", >> igi_script )
        print ("ylabel Number", >> igi_script)
        print ("end", >> igi_script)
        igi (initcmd="", device=device, metacode="", append=no,
             debug=no,cursor="", < igi_script)

        # That's all folks.
        delete (igi_script, go_ahead=yes, verify=no, >& "dev$null")
        delete (sig2rms, go_ahead=yes, verify=no, >& "dev$null")
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")

end
#---------------------------------------------------------------------------
# End of pha_sig2rms
#---------------------------------------------------------------------------
