#---------------------------------------------------------------------------
# pha_summary -- Run all the PHA analysis tasks.
#---------------------------------------------------------------------------
procedure pha_summary(input, eefile, intphd, difphd, fitphd, table, text, diodes)

file    input           {prompt="Input PHA data file name template"}
file    eefile          {prompt="Input extracted engineering template"}
file    intphd          {"intphd",prompt="Integral pulse height distribution image"}
file    difphd          {"difphd",prompt="Differential pulse height distribution image"}
file    fitphd          {"fitphd",prompt="Fitted pulse height distribution image"}
file    table           {"phaz.tab",prompt="Threshold statistics table"}
file    text            {"phaz.txt",prompt="Text summary of PHA analysis"}
file    diodes          {"dev$null",prompt="File containing list of diodes to examine"}

real    t0              {30.,prompt="Initial threshold level in input"}
real    delt            {1.,prompt="Step size in theshold levels in input"}
string  device          {"stdplot",prompt="Graphics output device"}
real    left            {0.,min=0.,max=1.,prompt="Left edge of viewport"}
real    right           {1.,min=0.,max=1.,prompt="Right edge of viewport"}
real    bottom          {0.,min=0.,max=1.,prompt="Bottom edge of viewport"}
real    top             {1.,min=0.,max=1.,prompt="Top edge of viewport"}
bool    verbose         {no,prompt="Write status messages?"}

string  Version         {"11Dec92",prompt="Date of Installation"}

begin
        # Declarations
        string  ts1
        
        file    pdifphd
        file    peefile
        file    pfitphd
        file    pinput
        file    pintphd
        file    ptable
        file    ptext
        
        # Make sure the correct packages are loaded.
        ts1 = ""
        if (!defpac ("stplot"))
            ts1 = ts1//"stplot "
        if (!defpac ("ttools"))
            ts1 = ts1//"ttools "
	if (!defpac ("z_calib"))
	    ts1 = ts1//"z_calib "
        if (strlen (ts1) > 0)
            error (1, "Please load packages: "//ts1)

        # Get the queried parameters.
        pinput = input
        peefile = eefile
        pintphd = intphd
        pdifphd = difphd
        pfitphd = fitphd
        ptable = table
        ptext = text
        
        if (verbose)
            print ("pha_summary: Running phaz task")
        phaz (pinput, peefile, pintphd, pdifphd, pfitphd, ptable,
              t0=t0, delt=delt)
        
        if (verbose)
            print ("pha_summary: running pha_text")
        pha_text (table=ptable, > ptext)
        
        if (verbose)
            print ("pha_summary: Running pha_center")
        pha_center (table=ptable, device=device,
                    left=left, right=right, bottom=bottom, top=top, verbose=no)
        
        if (verbose)
            print ("pha_summary: Running pha_rms")
        pha_rms (table=ptable, device=device, 
                 left=left, right=right, bottom=bottom, top=top, verbose=no)
        
        if (verbose)
            print ("pha_summary: Running pha_sig2rms")
        pha_sig2rms (table=ptable, device=device, 
                     left=left, right=right, bottom=bottom, top=top, verbose=no)
        
        if (verbose)
            print ("pha_summary: Running pha_diode")
        pha_diode (diodes, intphd=pintphd, difphd=pdifphd, fitphd=pfitphd,
                   table=ptable, t0=t0, delt=delt, device=device, 
                   left=left, right=right, bottom=bottom, top=top,
                   verbose=verbose)

        if (verbose)
            print ("pha_summary: Running pha_all")
        pha_all (difphd=pdifphd, fitphd=pfitphd, table=ptable, t0=t0, delt=delt,
                 device=device, left=left, right=right,
                 bottom=bottom, top=top, verbose=verbose)

        if (verbose)
            print ("pha_summary: Summary completed")
        
end
#---------------------------------------------------------------------------
# End of pha_summary
#---------------------------------------------------------------------------
