#---------------------------------------------------------------------------
# pha_text -- Print text summary of PHA analysis.
#---------------------------------------------------------------------------
procedure pha_text

file    table           {prompt="PHA results table"}

string  Version         {"11Dec92",prompt="Date of Installation"}

begin
        # Declarations.
        real    hv              # High voltage.
        real    x, y            # Generic.
        
        int     detector        # Detector.
        int     exposure        # Exposure time (seconds)
        int     i, j            # Generic.

        file    tmp_table_1     # Temporary table.

        string  rootname        # Rootname.
        string  ts1             # Generic.

        # Set temporary file names.
        tmp_table_1 = mktemp("tmp$pha_table_1")//".tab"

        # Make sure the correct packages are loaded.
        ts1 = ""
        if (!defpac ("stplot"))
            ts1 = ts1//"stplot "
        if (!defpac ("ttools"))
            ts1 = ts1//"ttools "
	if (!defpac ("z_calib"))
	    ts1 = ts1//"z_calib "
        if (strlen (ts1) > 0)
            error (1, "Please load packages: "//ts1)

        # Retrieve the parameters.
        if (table == "") {
            ts1 = phaz.table.p_mode
            phaz.table.p_mode = "h"
            table = phaz.table
            phaz.table.p_mode = ts1
        }

        # Get some header parameters.
        keypar (table, "exposure")
        exposure = int (keypar.value)
        keypar (table, "hv")
        hv = real (keypar.value)

        # Print out title information.
        keypar (table, "rootname")
        print ("PHA SUMMARY             Observation "//keypar.value)
        keypar (table, "detector")
        print ("HRS Detector "//keypar.value//"    "//exposure//" Sec. Integrations     Photocathode "//hv//" KV")

        # Sort table on amps and channels.
        delete (tmp_table_1, go_ahead=yes, verify=no, >& "dev$null")
        tcopy (table, tmp_table_1, verbose=no)
        tsort (tmp_table_1, "AMP,CHAN", ascend=yes, casesens=yes)
        
        # Need some table arithmetic.  First calculate signal to noise.
        tcalc (tmp_table_1, outcol="SNR", equals="center / sigma",
               datatype="real", colunits="", colfmt="%4.1f")

        # Calculate counts per second.
        tcalc (tmp_table_1, outcol="CPS", equals="counts / "//exposure,
               datatype="real", colunits="", colfmt="%5.0f")

        # Calculate FWHM in KEV units.
        tcalc (tmp_table_1, outcol="FWHM", equals="fwhm*"//hv//"/center",
               datatype="real", colunits="kev", colfmt="%4.2f")

        # Create on Octal version of the threshold values.
        tcalc (tmp_table_1, outcol="OCTAL", equals="nint(thresh)",
               datatype="int", colunits="", colfmt="%2o")

        # Rename the data item column.
        tchcol (tmp_table_1, "DATAITEM", "ITEM", "", "", verbose=no)

        # Print some basic statistics.
        tstat (table, "THRESH", outtable="", lowlim=INDEF, highlim=INDEF,
               rows="-")
        printf ("THRESHOLDS     average = %4.2f    stdev = %4.2f\n",
                tstat.mean, tstat.stddev)
        
        tstat (table, "SIGMA", outtable="", lowlim=INDEF, highlim=INDEF,
               rows="-")
        printf ("RMS NOISE      average = %4.2f    stdev = %4.2f\n",
                tstat.mean, tstat.stddev)
        print ("")

        # Print contents of the pha table.
        tprint (tmp_table_1, prparam=no, prdata=yes, pwidth=80, plength=0,
                showrow=no, showhdr=yes,
                columns="AMP CHAN DIODE CENTER SIGMA SNR FWHM THRESH OCTAL BASE1 CPS ITEM",
                rows="-", option="plain", align=yes, sp_col="", lgroup=0)
        
        # That's all folks.
end
#---------------------------------------------------------------------------
# End of pha_text
#---------------------------------------------------------------------------
