        SUBROUTINE ZCOREL(FLUX,NS,TEMPLT,NT,IPOS,MAXDEV,OFFSET,ISTAT)
*
*  Module number: 
*
*  Module name: zcorel
*
*  Keyphrase:
*  ----------
*	Determine offset
*  Description:
*  ------------
*	This routine determines the offset of a template from the
*	nomimal position, IPOS.  The normalized mean and variance
*	correlation coefficient is used as the measure of the
*	correlation.
*
*  FORTRAN name: zcorel.for
*
*  Keywords of accessed files and tables:
*  --------------------------------------
*	none
*
*  Subroutines Called:
*  -------------------
*  SDAS:
*	umsput
*
*  History:
*  --------
*  Version      Date        Author          Description
*	1	Aug 88	D. Lindler	Designed and coded
*-------------------------------------------------------------------------------
*
* INPUT PARAMETERS
*
*	FLUX - FLUX VECTOR (REAL*8)
*	NS - NUMBER OF ELEMENTS IN FLUX (INTEGER)
*	TEMPLT - TEMPLATE VECTOR (REAL*8)
*	NT - NUMBER OF ELEMENTS IN TEMPLT (INTEGER)
*	IPOS - NOMINAL POSITION IN FLUX FOR FIRST DATA POINT IN TEMPLT
*		(INTEGER)
*	MAXDEV - MAXIMUM SEARCH DISTANCE FOR CORRELATION MAXIMUM (INTEGER)
*
* OUTPUT PARAMETERS
*
*	OFFSET - POSITION OFFSET FROM NOMINAL (REAL*8)
*	ISTAT - ERROR STATUS (INTEGER)
C
C     UMSPUT DESTINATIONS -- CB, DAO, 4-SEP-87
C
      INTEGER STDOUT
      PARAMETER (STDOUT = 1)
      INTEGER STDERR
      PARAMETER (STDERR = 2)

        DOUBLE PRECISION  FLUX(1),TEMPLT(1),OFFSET
        INTEGER NS,NT,IPOS,MAXDEV,ISTAT
C
C LOCAL VARIABLES
C
        DOUBLE PRECISION C(101),CMAX,RK,V1,V2
        INTEGER I,K,IOFF,IOFF1,IOFF2,CPOS
        DOUBLE PRECISION SUM12,SUM1SQ,SUM2SQ,DENOM,AVE1,AVE2,SUM
        CHARACTER*130 CONTXT
C
C-----------------------------------------------------------------------
C
C FIND FIRST AND LAST OFFSET OF TEMPLT IN FLUX ARRAY OVER THE
C SEARCH WIDTH
C
        IOFF1=IPOS-MAXDEV-1
        IOFF2=IPOS+MAXDEV-1
        IF(IOFF1.LT.0) IOFF1=0
        IF(IOFF2.GE.NS) IOFF2=NS-1
C
C COMPUTE AVERAGE AND SUMSQ OF TEMPLATE
C
        SUM=0.0
        DO 5 I=1,NT
                SUM=SUM+TEMPLT(I)
5       CONTINUE
        AVE2=SUM/NT
        SUM2SQ=0.0
        DO 6 I=1,NT
                SUM2SQ=SUM2SQ+(TEMPLT(I)-AVE2)**2
6       CONTINUE
C
C LOOP ON OFFSETS AND STORE CORRELATION IN VECTOR C
C
        K=0
        DO 100 IOFF=IOFF1,IOFF2
                SUM=0.0
                DO 10 I=1,NT
                        SUM=SUM+FLUX(IOFF+I)
10              CONTINUE
                AVE1=SUM/NT
C
C COMPUTE SUMSQ FOR FLUX ARRAY AND CROSS PRODUCT
C
                SUM1SQ=0.0
                SUM12=0.0
                DO 20 I=1,NT
                        V1=FLUX(IOFF+I)-AVE1
                        V2=TEMPLT(I)-AVE2
                        SUM1SQ=SUM1SQ+V1*V1
                        SUM12=SUM12+V1*V2
20              CONTINUE
C
C COMPUTE CORRELATION
C
                DENOM=SQRT(SUM1SQ)*SQRT(SUM2SQ)
                IF(DENOM.EQ.0.0)THEN
                    CONTXT='ERROR: Input data has constant value'
                    GO TO 999
                ENDIF
                K=K+1
                C(K)=SUM12/DENOM
100     CONTINUE
C
C FIND MAXIMUM IN CORRELATION MATRIX
C
        CMAX=0.0
        CPOS=0
        DO 200 I=1,K
                IF(C(I).GT.CMAX)THEN
                        CMAX=C(I)
                        CPOS=I
                ENDIF
200     CONTINUE
C
C DON'T USE IF ON EDGE
C
        IF((CPOS.EQ.1).OR.(CPOS.EQ.K))THEN
            CONTXT='Correlation peak on the edge of search region'
            GO TO 999
        ENDIF
C
C USE QUADRATIC REFINEMENT
C
        RK=(C(CPOS-1)-C(CPOS))/(C(CPOS-1)+C(CPOS+1)-2.0*C(CPOS))-0.5
        OFFSET=RK+CPOS+IOFF1-IPOS
        ISTAT=0
        GO TO 1000
C
C ERROR
C
999     CALL UMSPUT(CONTXT,STDOUT+STDERR,0,ISTAT)
        ISTAT=1
1000    RETURN
        END
