      SUBROUTINE RYLM(DCOLAT,DLON,IORDER,DYLMVAL)            
c
c  ------------------------------------------------------------------------
c  Part of "poa_calfos" (STECF/IPMG)
c  Part of "NEWBILCAL" program to produce IGRF geomagnetic model (GSFC)
c
c  History:
c  --------
c  Version   Date        Author          Description
c      1.0       Jul 00  M. Rosa         POA calibration code
c  ------------------------------------------------------------------------
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     $Revision: 1.1.1.1 $
c
c     The initial version of this subroutine was written by RADEX, INC.
c     for VAX/VMS systems.
c
c     Subsequent revisions for use with POSIX compliant systems
c     have been made by KBB at JHU/APL.  These revisions have
c     been managed using RCS, and a log of these revisions 
c     follows.
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    RCS   Revision History
c
c     $Log: rylm.f,v $
c     Revision 1.1.1.1  2000/07/12 19:25:10  ecf-poa
c     stpoa branch with a close version to the first release
c
c     Revision 2.2  2000/07/11 12:26:35  aalexov
c     Updated all the code to have a history (new history for poa
c     code) and appended a history for calfos files which were
c     updated by us.  Also deleted .f files from the reposutory
c     which were no longer being used (cleaning up).
c
c     Revision 2.1  2000/07/10 19:06:41  aalexov
c     These are Michael's changes, for the release.  Comments were
c     added for the headers of all poa new files.  Also, new files
c     were created in order to make sure that all subroutines were
c     in their own files.  mkpkg was updated to look "clean".
c
c
c Revision 1.3  1996/03/11  19:25:07  baker
c Revisions for 1995 version of AACGM
c
c Revision 1.2  1994/10/14  10:50:45  baker
c Added the SAVE instruction to make the variables static.
c
c Revision 1.1  94/10/12  15:24:21  15:24:21  baker (Kile Baker S1G)
c Initial revision
c 
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C            
C     Purpose:
C                                                                 
C         This subroutine computes the array of real spherical harmonic 
C         function values Y(L,M) for a given colatitude and longitude up 
C         to order I_ORDER. The function values are stored in the one-
C         dimensional array D_YLMVAL(N). The indexing scheme used is as 
C         follows:
C
C     L    0  1  1  1  2  2  2  2  2  3  3  3  3  3  3  3  4
C     M    0 -1  0  1 -2 -1  0  1  2 -3 -2 -1  0  1  2  3 -4  etc.
C
C     N    1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17
C 
C     Input Arguments:  
C
C         D_COLAT             - Double Precision  - The colatitude of the
C                               point for which the spherical harmonic 
C                               Y(L,M) will be calculated
C
C         D_LON               - Double Precision  - The longitude of the
C                               point for which the spherical harmonic
C                               Y(L,M) will be calculated
C
C         I_ORDER             - Integer  - The order of the spherical
C                               harmonic function expansion. The total
C                               number of terms computed will be
C                               (I_ORDER + 1) * (I_ORDER + 1)
C
C     Output Argument:
C                    
C         D_YLMVAL            - Double Precision array of spherical harmonic
C                               functions at the point (D_COLAT, D_LON)  
C                 
C     Local Variables:
C
C         Q_FAC               - Double Precision Complex number used for
C                               recursive computation of the spherical
C                               harmonic function values. Represents the
C                               longitude dependence in the recursion
C                               computation. 
C
C         Q_VAL               - Double Precision Complex number used to
C                               store spherical harmonic function for
C                               current L and M values for recursive
C                               computation for the next set of L and M
C                               values
C
C         COS_THET           - Double Precision Cosine of D_COLAT
C         SIN_THET           - Double Precision Sine of D_COLAT
C                               used to compute Spherical Harmonic functions
C
C         COS_LON             - Double Precision Cosine of Longitude
C         SIN_LON             - Double Precision Sine of Longitude
C                               used to compute Spherical Harmonic functions
C                             
C         CA, CB              - Double Precision Coefficients used in the
C                               recursion computations
C
C         LA, LB, LC,         - Integer - Pointers for the D_YLMVAL array
C         LD, LE, LF            corresponding to values of Y(L,M) for
C                               particular values of L, M for use in the
C                               recursive computation of Y(L.M) 
C
C
C     Constants:  None
C     Subroutines Required: None
C     Files Used at Compile Time:  None
C     Files Used at Run Time:  None
C     Databases Accessed:  None
C     Warnings:  None
C     Revision History:
C     Written by Radex, Inc., 3 Preston Court, Bedford, MA 01730
C
      SAVE    !make all variable static
c
      INTEGER  IORDER
C 
      REAL*8 DYLMVAL(*)
      REAL*8 DCOLAT
      REAL*8 DLON 
C
      COMPLEX*16 QFAC
      COMPLEX*16 QVAL

C     Local Variables:  
      INTEGER     L,LA,LB,LC,LD,LE,LF,M
      REAL*8   CA,CB,FAC

      REAL*8 COSTHET
      REAL*8 SINTHET
      REAL*8 COSLON
      REAL*8 SINLON
C-----------------------------------
C
      COSTHET =  DCOS(DCOLAT)
      SINTHET =  DSIN(DCOLAT)
      COSLON   =  DCOS(DLON)
      SINLON   =  DSIN(DLON)
C
      QFAC     = -SINTHET * DCMPLX(COSLON,SINLON)
C
C     Generate Zonal Harmonics (Y(L,0), L = 1, I_ORDER)
C     using Standard Recursion Relations (Equation 6.8.7, p 247,
C     Numerical Recipes in Fortran, 2. edition, by Press. W. et. al.
C     Cambridge University Press, 1992) for case for which m = 0
      DYLMVAL(1) = 1.0D0
      DYLMVAL(3) = COSTHET
C
      DO 50 L = 1, IORDER - 1
         LA = (L - 1) * L + 1
         LB = L * (L + 1) + 1
         LC = (L + 1) * (L + 2) + 1
         CA = (2.0D0 * L + 1.0D0)/(L + 1.0D0)
         CB = DFLOAT(L)/(L + 1.0D0)
         DYLMVAL(LC) = CA*COSTHET*DYLMVAL(LB) - CB*DYLMVAL(LA) 
50    CONTINUE
C
C     Generate Y(L,L) for L = 1 to I_ORDER. Algorithm based upon 
C     equation (6.8.8) in Press et al, but incorporates longitude
C     dependence. 
      QVAL = QFAC
      DYLMVAL(4) =  DREAL(QVAL)
      DYLMVAL(2) = -DIMAG(QVAL)

      DO 100 L = 2, IORDER
          QVAL = (2.0D0 * L - 1.0D0) * QFAC * QVAL
          LA = L * L + 2 * L + 1
          LB = L * L + 1
          DYLMVAL(LA) =  DREAL(QVAL)
          DYLMVAL(LB) = -DIMAG(QVAL)
100   CONTINUE
C     
C     Generate Y(L+1,L) to (Y(I_ORDER,L) function values. 
C     Algorithm based upon equation (6.8.9) in Press et al, but 
C     incorporates longitude dependence. 
      DO 150 L = 2, IORDER
          LA  = L * L
          LB  = L * L - 2 * (L - 1)
          LC  = L * L + 2 * L
          LD  = L * L + 2 
          FAC = 2.0D0 * L - 1.0D0
          DYLMVAL(LC) = FAC * COSTHET * DYLMVAL(LA)
          DYLMVAL(LD) = FAC * COSTHET * DYLMVAL(LB)
150   CONTINUE
C
C     Generate remaining Y(L+2,L) to YL(I_ORDER,L) function values.
C     Algorithm based upon equation (6.8.7) in Press et al, but 
C     incorporates longitude dependence. 
C
      DO 200 M = 1, IORDER - 2
C     INITIALIZATION OF POINTERS FOR CURRENT VALUE OF M
          LA  =  (M + 1)**2
          LB  =  (M + 2)**2 - 1
          LC  =  (M + 3)**2 - 2
          LD  =   LA - 2 * M
          LE  =   LB - 2 * M 
          LF  =   LC - 2 * M
          DO 180 L = M + 2, IORDER
            CA  = DFLOAT(2*L-1)/DFLOAT(L-M)
            CB  = DFLOAT(L+M-1)/DFLOAT(L-M)
            DYLMVAL(LC) = CA*COSTHET*DYLMVAL(LB) - CB*DYLMVAL(LA)
            DYLMVAL(LF) = CA*COSTHET*DYLMVAL(LE) - CB*DYLMVAL(LD)
C     UPDATE POINTERS FOR NEXT M VALUE
            LA  = LB
            LB  = LC
            LC  = LB + 2 * L + 2
            LD  = LA - 2 * M
            LE  = LB - 2 * M 
            LF  = LC - 2 * M
180       CONTINUE
200   CONTINUE
C
      RETURN
      END

